/***
*vm.h - Variable manipulation package declarations
*
*$Id: vm.h 6.89 2007/09/18 17:18:31 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file declares structures, defines symbols and macros, and
*   gives function prototypes for facilities associated with the
*   variable manipulation facilities of SPSM.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | SFG | 90/09/20 | added global i_veuvar to initialise veuvar in vmadd()   |
* | SFG | 90/07/18 | added macros vmlevlab, vmlevnum() and vmlevmax()        |
* | SFG | 90/05/25 | renamed veclass,venlev to veclnlev & added veuvar       |
* | SFG | 90/04/25 | recoded some macros, removed other unused macros        |
* | SFG | 90/04/22 | Added veclnlev to ve structure                          |
* | SFG | 90/04/17 | Made vmadd return the created ve                        |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Dependencies:
*   #include "spsm.h"
*   #include "vm.h"
*
*Definitions:
*   offset   [macro ]  Give offset of member in a structure
*   vardef   [macro ]  Define structure member as an SPSM variable
*   isHH     [macro ]  Is variable in hhld structure ?
*   isHD     [macro ]  Is variable in hd structure ?
*   isFX     [macro ]  Is variable in fx structure ?
*   isCT     [macro ]  Is variable in ct structure ?
*   isEF     [macro ]  Is variable in ef structure ?
*   isCF     [macro ]  Is variable in cf structure ?
*   isNF     [macro ]  Is variable in nf structure ?
*   isIN     [macro ]  Is variable in in structure ?
*   isID     [macro ]  Is variable in id substructure of in ?
*   isIM     [macro ]  Is variable in im substructure of in ?
*   isIC     [macro ]  Is variable in ic substructure of in ?
*   isEX     [macro ]  Is variable in ex substructure of in ?
*   isMOD    [macro ]  Is variable modelled ?
*   isVAR    [macro ]  Is variable modelled variant ?
*   varptr   [macro ]  Get pointer to variable value
*   varNUM   [macro ]  Get value of SPSM NUMBER variable
*   varINT   [macro ]  Get value of SPSM integer variable
*   vmlevlab [macro ]  Get label for specified level of specified variable
*   vmlevnum [macro ]  Get number of levels for specified variable
*   vmlevwid [macro ]  Get maximum width of level labels for specified variable
*
*****************************************************************************/



/* ==================== MANIFEST CONSTANTS ================================ */

/**
* The following define the width of the bit fields in the ve structure,
* so that functions can check to ensure that limits are not being exceeded.
**/

#define W_type    3
#define W_base    1
#define W_prec    4   /* end of unsigned #1 */
#define W_uvar    1
#define W_clnlev 10   /* end of unsigned #2 */

#define MAX_prec   ((1<<W_prec)-1) /* maximum value for veprec field */
#define MAX_clnlev ((1<<W_clnlev)-1) /* maximum value for veclnlev field */


/**
* The following define the arguments to the vmlev function, which 
* extracts information about a class variable from the string library.
* These values are negative to distinguish them from the request where vmlev
* returns the offset to the label with the specified level.
**/

#define VM_DIM   (-2)     /* vmlev returns number of levels */
#define VM_WID   (-1)     /* vmlev returns maximum label width */


/**
* These macros define whether a variable is a classification
* or an analysis variable, and are used as the 'class' argument to vmadd().
**/

#define V_ANAL   0
#define V_CLAS   1

/**
* The following macros define certain offsets useful for the
* definition of further macros below.
**/

#define OFFSET_id      0
#define OFFSET_im     (sizeof(id_))
#define OFFSET_imbase (sizeof(id_)+sizeof(im_))
#define OFFSET_ic     (sizeof(id_)+sizeof(im_)+sizeof(im_))
#define OFFSET_ex     (sizeof(id_)+sizeof(im_)+sizeof(im_)+sizeof(ic_))



/* ==================== MACRO DEFINITIONS ================================= */

/**
* The following macros are used in vsd*.c to define
* variables to be added to the var table.
**/



/*****************************************************************************
*offset - Give offset of member in a structure
*
*Synopsis:
*   size_t offset(S, V)
*   'struct_variable' S
*   'member_name' V
*
*Description:
*   This macro returns the offset, in bytes, of the start of the named member
*   within the specified structure.  S must be an actual variable.
*
*Return Value:
*   The specified offset is returned.
*
*Called by:
*   vardef   Define structure member as an SPSM variable
*
*****************************************************************************/
#define offset(S,V) ((char *)&S->V-(char *)S)



/*****************************************************************************
*vardef - Define structure member as an SPSM variable
*
*Synopsis:
*  vardef(N,S,V,T,C,L)
*   N = name
*   S = structure
*   V = variable in structure
*   T = C type
*   C = classification or analysis
*
*Description:
*   This macro prepares the arguments into a call to the 'vmadd' function,
*   which makes the given structure member accessible to the SPSM variable
*   manipulation facilities.
*
*Return Value:
*   None.
*
*Calls:
*   offset   Give offset of member in a structure
*   vmadd    Define an SPSM variable
*
*Called by:
*   exuvnew  Create a new user-defined black box variable
*   vsdd     Define SPSD/M variables
*   vsdm     Define SPSD/M variables
*
*****************************************************************************/
#define vardef(N,S,V,T,C) \
	vmadd(N, (char NEARHHLD * NEAR *)&S, offset(S,V), T, C)



/**
* The following macros can be used to determine which structure a variable
* is associated with.
**/



/*****************************************************************************
*isHH - Is variable in hhld structure ?
*
*Synopsis:
*   int isHH(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the hhld structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   ascout   Output SPSD/M variables in ASCII form
*
*****************************************************************************/
#define isHH(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&HH)



/*****************************************************************************
*isHD - Is variable in hd structure ?
*
*Synopsis:
*   int isHD(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the hd structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   ascout   Output SPSD/M variables in ASCII form
*   vmadd    Define an SPSM variable
*
*****************************************************************************/
#define isHD(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&HD)



/*****************************************************************************
*isFX - Is variable in fx structure ?
*
*Synopsis:
*   int isFX(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the fx structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   ascout   Output SPSD/M variables in ASCII form
*   uxini    Process user expression requests
*   vmadd    Define an SPSM variable
*   xtinit   Process user X-tab requests
*
*****************************************************************************/
#define isFX(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&FX)



/*****************************************************************************
*isCT - Is variable in ct structure ?
*
*Synopsis:
*   int isCT(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the ct structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   ascout   Output SPSD/M variables in ASCII form
*   isMOD    Is variable modelled ?
*   isVAR    Is variable modelled variant ?
*   vmadd    Define an SPSM variable
*   xtinit   Process user X-tab requests
*
*****************************************************************************/
#define isCT(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&CT)



/*****************************************************************************
*isEF - Is variable in ef structure ?
*
*Synopsis:
*   int isEF(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the ef structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   vmadd    Define an SPSM variable
*
*****************************************************************************/
#define isEF(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&EF)



/*****************************************************************************
*isCF - Is variable in cf structure ?
*
*Synopsis:
*   int isCF(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the cf structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   vmadd    Define an SPSM variable
*
*****************************************************************************/
#define isCF(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&CF)



/*****************************************************************************
*isNF - Is variable in nf structure ?
*
*Synopsis:
*   int isNF(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the nf structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   vmadd    Define an SPSM variable
*
*****************************************************************************/
#define isNF(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&NF)



/*****************************************************************************
*isIN - Is variable in in structure ?
*
*Synopsis:
*   int isIN(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the in structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   exvar    Perform special side effect actions for variables
*   isEX     Is variable in ex substructure of in ?
*   isIC     Is variable in ic substructure of in ?
*   isID     Is variable in id substructure of in ?
*   isIM     Is variable in im substructure of in ?
*   isMOD    Is variable modelled ?
*   marini   Process request to compute marginal tax rates
*   vmadd    Define an SPSM variable
*   vmcumvar Request family cumulation for a variable
*
*****************************************************************************/
#define isIN(ve) (ve->vestruct == (char NEARHHLD * NEAR *)&IN)



/*****************************************************************************
*isID - Is variable in id substructure of in ?
*
*Synopsis:
*   int isID(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the id subtructure of the in structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*****************************************************************************/
#define isID(ve) (isIN(ve) && (ve->veofs < sizeof(id_)))



/*****************************************************************************
*isIM - Is variable in im substructure of in ?
*
*Synopsis:
*   int isIM(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the md subtructure of the in structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   fmrvopen Open reference value SPSM result file (.mrs)
*   isVAR    Is variable modelled variant ?
*   vmadd    Define an SPSM variable
*
*****************************************************************************/
#define isIM(ve) (isIN(ve) && (ve->veofs >= OFFSET_im)\
                           && (ve->veofs < (OFFSET_im+sizeof(im_))))

									
									
/*****************************************************************************
*isIC - Is variable in ic substructure of in ?
*
*Synopsis:
*   int isIC(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the cntl subtructure of the in structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Called by:
*   vmcumvar Request family cumulation for a variable
*
*****************************************************************************/
#define isIC(ve) (isIN(ve) && (ve->veofs >= OFFSET_ic)\
                           && (ve->veofs < (OFFSET_ic+sizeof(ic_))))



/*****************************************************************************
*isEX - Is variable in ex substructure of in ?
*
*Synopsis:
*   int isEX(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a variable in the ex or cl array of the in structure or not.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Calls:
*   isIN     Is variable in in structure ?
*
*Called by:
*   exvar    Perform special side effect actions for variables
*   leaf     Create a new leaf for use in an expression tree
*
*****************************************************************************/
#define isEX(ve) (isIN(ve) && (ve->veofs >= OFFSET_ex))



/*****************************************************************************
*isMOD - Is variable modelled ?
*
*Synopsis:
*   int isMOD(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer refers
*   to a modelled or a database variable.  It depends on the order of the
*   substructures in the 'in' structure.  Specifically, the 'id'
*   structure must be first, followed by 'im' and other structures with
*   computed variables.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Calls:
*   isCT     Is variable in ct structure ?
*   isIN     Is variable in in structure ?
*
*Called by:
*   fmroopen Open new SPSM result file (.mrs)
*   leaf     Create a new leaf for use in an expression tree
*
*****************************************************************************/
#define isMOD(ve) (isCT(ve) || (isIN(ve) && ve->veofs >= OFFSET_im))



/*****************************************************************************
*isVAR - Is variable modelled variant ?
*
*Synopsis:
*   int isVAR(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine if the given variable entry pointer has a
*   counterpart in a base model structure.  This is the case for the 'ct'
*   structure, and for the 'md' substructure of the 'in' structure.
*
*Return Value:
*   Returns 1 if true, 0 if false.
*
*Calls:
*   isCT     Is variable in ct structure ?
*   isIM     Is variable in im substructure of in ?
*
*****************************************************************************/
#define isVAR(ve) (isCT(ve) || isIM(ve))


/**************************************************************************
* The following macros can be used to access the value of a variable.
* varptr(ve) gives a pointer (of type char *) to the variable
* with pointer ve, while varNUM and varINT refer to the
* actual value of the variable.  varNUM, varINT
* expand into valid lvalue's.
***************************************************************************/

/*****************************************************************************
*varptr - Get pointer to variable value
*
*Synopsis:
*   char * varptr(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine the address of the given variable in the
*   current structure of the appropriate type.  The return value must be
*   cast to an appropriate type before use.
*
*Return Value:
*   Returns a pointer to the variable's value.
*
*Called by:
*   varINT   Get value of SPSM integer variable
*   varNUM   Get value of SPSM NUMBER variable
*
*****************************************************************************/
#define varptr(ve)    (*(ve->vestruct)+ve->veofs)



/*****************************************************************************
*varNUM - Get value of SPSM NUMBER variable
*
*Synopsis:
*   NUMBER varNUM(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine the address of the given variable in the
*   current structure of the appropriate type.  This address is dereferenced
*   to a NUMBER.
*
*Return Value:
*   Returns the NUMBER value.
*
*Called by:
*   ascout   Output SPSD/M variables in ASCII form
*   dstupd   Update information for distribution report
*   exevl    Evaluates a previously parsed expression
*   fmriread Read record from old SPSM result file (.mrs)
*   fmrowrit Write record to new SPSM result file (.mrs)
*   fmrvread Read record from reference value SPSM result file (.mrs)
*   tbl      Increment hard-wired tables
*   tpeval   Set up variables for next household, and simulate
*   xtincr   Increment general X-tab tables
*
*****************************************************************************/
#define varNUM(ve)    (*(NUMBER NEARHHLD *) varptr(ve))



/*****************************************************************************
*varINT - Get value of SPSM integer variable
*
*Synopsis:
*   int varNUM(ve)
*   P_ve ve;
*
*Description:
*   This macro will determine the address of the given variable in the
*   current structure of the appropriate type.  This address is dereferenced
*   to an integer.
*
*Return Value:
*   Returns the integer value.
*
*Calls:
*   varptr   Get pointer to variable value
*
*Called by:
*   ascout   Output SPSD/M variables in ASCII form
*   dstupd   Update information for distribution report
*   exevl    Evaluates a previously parsed expression
*   fmriread Read record from old SPSM result file (.mrs)
*   fmrowrit Write record to new SPSM result file (.mrs)
*   fmrvread Read record from reference value SPSM result file (.mrs)
*   tbl      Increment hard-wired tables
*   xtincr   Increment general X-tab tables
*
*****************************************************************************/
#define varINT(ve)    (*(int NEARHHLD *) varptr(ve))



/*****************************************************************************
*vmlevlab - Get label for specified level of specified variable
*
*Synopsis:
*   char * vmlevlab(ve, j)
*   P_ve ve;
*   int j;
*
*Description:
*   As described.
*
*Return Value:
*   Returns a pointer to the string.
*
*Calls:
*   vmlevi   Obtain label for specified level of an integer variable
*
*Called by:
*   exevl    Evaluates a previously parsed expression
*   fmroopen Open new SPSM result file (.mrs)
*   getlab   Retrieve label for specified index in specified dimension
*
*****************************************************************************/
#define vmlevlab(ve,j)  vmlevi(ve, j)



/*****************************************************************************
*vmlevnum - Get number of levels for specified variable
*
*Synopsis:
*   char * vmlevnum(ve)
*   P_ve ve;
*
*Description:
*   As described.
*
*Return Value:
*   Returns the number of levels as an integer.  If the argument is not
*   a class variable, 0 is returned.
*
*Calls:
*   vmlev    Produce information on levels of a specified integer variable
*
*Called by:
*   exevl    Evaluates a previously parsed expression
*   fmroopen Open new SPSM result file (.mrs)
*   leaf     Create a new leaf for use in an expression tree
*   xtinit   Process user X-tab requests
*
*****************************************************************************/
#define vmlevnum(ve)  vmlev(ve, VM_DIM)



/*****************************************************************************
*vmlevwid - Get maximum width of level labels for specified variable
*
*Synopsis:
*   char * vmlevwid(ve)
*   P_ve ve;
*
*Description:
*   As described.
*
*Return Value:
*   Returns the maximum width as an integer.  If the argument is not
*   a class variable, 0 is returned.
*
*****************************************************************************/
#define vmlevwid(ve)  vmlev(ve, VM_WID)



/* ==================== STRUCTURE DECLARATION MACROS ====================== */

#define S_ve   struct ve_
#define P_ve   S_ve *



/* ==================== STRUCTURE DECLARATIONS ============================ */

/**
* The structure identified by the letters 've' holds information about each
* variable.  The functions vsd1, vsd2, and vsdu
* create, through another function call (vmadd), a 've' entry for
* each variable.  The pointer to a 've' structure corresponding
* to a particular variable can be obtained by a call to vmfind.
**/

typedef struct ve_ {
	P_ve venext;        /* next element in list [pointer]        */

	char NEARHHLD * NEAR * vestruct;  /* address of pointer to containing structure [pointer] */
	                   /* address of a pointer to the structure containing   */
	                   /* the variable. This value must be one of the        */
	                   /* globals HH, HD, FX, EF, CF, NF, CT ,IN.            */
	unsigned   veofs;  /* offset of the variable in its structure (*vestruct) */
	char NEAR *vename; /* name of the variable [pointer]                     */
	unsigned vetype   :W_type;   /* C type                                   */
	unsigned vebase   :W_base;   /* base model flag                          */
	unsigned veuvar   :W_uvar;   /* user variable flag                       */
	unsigned veprec   :W_prec;   /* output precision                         */
	unsigned veclnlev :W_clnlev; /* levels in class variable (0=analysis)    */
} ve_;



/* ==================== GLOBAL VARIABLE DECLARATIONS ====================== */

/**
* The values of the following global pointers are set before using a
* 've' entry.  The vestruct member of the 've' structure points to one of
* these global pointers.  These are allocated by vmadd().
* They have been explicitly declared NEAR for efficiency.
**/

extern P_hh   NEAR HH;
extern P_hd   NEAR HD;
extern P_fx   NEAR FX;
extern P_ct   NEAR CT;
extern P_ef   NEAR EF;
extern P_cf   NEAR CF;
extern P_nf   NEAR NF;
extern P_in   NEAR IN;

/**
* The following variable indicates which value will be initially placed
* in the veuvar member by vmadd().
**/

extern int i_veuvar;



/* ==================== FUNCTION PROTOTYPES =============================== */

extern void   vmcumvar(P_ve ve, int lvl);
extern void   vmroll(P_hh hhld, int lvl);
extern P_ve   vmfind(char *name);
extern P_ve   vmadd(char NEAR * name,char NEARHHLD * NEAR *strct,int ofs,\
	int type, int clnlev);
extern char * vmlab(P_ve ve);
extern int    vmlev(P_ve ve, int req);
extern char * vmlevi(P_ve ve, int j);
extern void   vmfree(void);
extern char * vmrollab(int lvl);
extern void   vmcuvar(void);
extern void   vmcroll(void);
