/***
*Ampd.cpp - Tax/transfer parameter definitions (Alternate algorithm)
*
*$Id: ampd.cpp 6.90 2007/09/18 17:23:48 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the Ampd.c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | SFG | 05/05/10 | Removed second argument to mpd, Ampd                    |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 95/10/18 | Add pragma to suppress near/far warnings in MSDOS 16-bit|
* | BBM | 93/01/12 | Add definition of F_LKTUR                               |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Usage:
*   This is a place holder module.  It is intended to be used to add new
*   tax/transfer parameters for the 'alternate' algorithm to the model.
*   Two files are used when adding a new 'alternate' algorithm parameter.
*
*   mpu.h    - is a header file which declares the new parameters.  It
*                should be modified first when adding a new parameter.
*   Ampd.c   - is a C-language program which enables the model to understand
*                how to read and modify the new parameters.  Each new
*                parameter in mpu.h must have a corresponding entry
*                in Ampd.c
*
*   This file contains example definitions of common Scalar parameter
*   definitions.  If the parameter to be added is one of these types,
*   the corresponding example can be copied and edited to add the new
*   parameter.
*   Further examples may be seen in the files /spsm/model/mpd[123].c
*
*Definitions:
*   Ampd     [global]  Tax/transfer parameter definitions (Alternate)
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "pm.h"     /* Parameter manipulation package declarations          */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Tmpd[] = "none/aucun"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== STATIC VARIABLE DEFINITIONS ======================= */

static char NEAR F_MONEY[]  = "8.2";
static char NEAR F_FRACT[]  = "8.5";
static char NEAR F_LKTUR[]  = "8.1 8.3 10.4";


/* ==================== COMPILER PRAGMAS ================================== */

// eliminate spurious 'near/far mismatch' from MS-DOS compiler
#if defined(_MSDOS)
#pragma warning(disable : 4762)
#endif

#ifdef MSCOVL
#pragma alloc_text(INIT_TEXT,Ampd)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */

/*****************************************************************************
*Ampd - Tax/transfer parameter definitions (Alternate)
*
*Synopsis:
*    void Ampd(           
*    P_pc pcp             
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Called by:
*   phase1   Perform initialisation tasks
*   pubmeta  Publish internal meta information in XML format
*
*****************************************************************************/
void  Ampd(
	P_pc pcp
)
{
	DEBUG_ON("Ampd");

	/**
	* Parameter definitions for the alternate algorithm follow.  As mentioned
	* above, new parameters should first be added to the file 'mpu.h'.  As
	* shown below, a parameter is defined to the SPSM parameter manipulation
	* facilities through a call to the 'pmaddent' function.  'pmaddent' takes
	* ten arguments, whose values are explained in the example below.    An
	* optional label for the parameter can be supplied through a call to the
	* 'stradd' function.  Please see the SPSM Programmer's Guide for more information.
	*
	* Please see the files '/spsm/defs/mp.h' and '/spsm/model/mpd[123].c' for
	* examples of parameter declarations and definitions.  The easiest way to
	* add a parameter is to copy and modify the definition of an existing
	* parameter.
	*
	*
	* A brief description of the meaning of the arguments to 'pmaddent' and
	* 'stradd' follows:
	*
	* pmaddent(
	*    pcp,                  <= parameter chain being extended (leave as is)
	*    "XXXXX",              <= name by which the parameter will be known
	*    (char *)&MP.UM.XXXXX, <= address of the parameter
	*    Format,               <= printing information for the parameter
	*    Agg_Type,             <= Aggregate type (scalar, vector, lookup, table)
	*    C_Type,               <= C-type (character, integer, number, string)
	*    Edit,                 <= Edits to be performed (eg. flag, fraction)
	*    Row_max,              <= Maximum number of rows, or option edit limit
	*    Rows_addr,            <= Address of int holding current number of rows
	*    Limit                 <= Number of columns
	*          );
	*
	*    stradd("XXXXX",       <= the name of the parameter, quoted
	*           "Label"        <= a printing label for the parameter
	*                  );
	*
	*
	* The following definitions can be used as templates if desired.
	*
	*  ----------------
	*  A flag parameter:
	*  ----------------

	pmaddent(pcp, "XXXXX",    (char *)&MP.UM.XXXXX,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	stradd("XXXXX", "Parameter label");

	*  ----------------------------------------------
	*  An option parameter with allowed values 1 or 2:
	*  ----------------------------------------------

	pmaddent(pcp, "XXXXX",    (char *)&MP.UM.XXXXX,    NULL,    P_SCL, C_INT,  E_OPT,  2, NULL, 0);
	stradd("XXXXX", "Parameter label");

	*  ---------------------------------------------------
	*  A numeric scalar parameter, such as a benefit level:
	*  ---------------------------------------------------

	pmaddent(pcp, "XXXXX",    (char *)&MP.UM.XXXXX,    NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	stradd("XXXXX", "Parameter label");

	*  ------------------------------------------------------
	*  A numeric scalar parameter, a fraction between 0 and 1:
	*  ------------------------------------------------------

	pmaddent(pcp, "XXXXX",    (char *)&MP.UM.XXXXX,    F_FRACT, P_SCL, C_NUM,  E_FRCT, 0, NULL, 0);

	*
	*
	* End of template examples.  Calls to 'pmaddent' should immediately follow
	* this comment block.
	**/


	DEBUG_OFF("Ampd");
}




