/***
*Aoas.cpp - Compute OAS for elderly
*
*$Id: Aoas.cpp 6.91 2007/09/18 17:22:09 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 01/02/28 | Removed Federal Seniors Benefit program                 |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 97/06/17 | Fixed year 2000 problem                                 | 
* | BBM | 96/05/27 | Add seniors benefits optimization with OAS/GIS          |
* | SFG | 91/05/16 | BM & BC's fixes to immigration phase-in                 |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Aoas     [global]  Compute OAS for elderly
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Toas[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Aoas - Compute OAS for elderly
*
*Synopsis:
*    void Aoas(            
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Structure Members:
*   MP.BOAS         (i)  Basic OAS
*   MP.OASFLAG      (i)  Old age security flag
*   MP.TARGETYEAR   (i)  Year of analysis
*   hd.hdageeld     (i)  Age of eldest in household
*   hd.hdseqhh      (i)  Household sequence number
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.idimmi       (i)  Years since immigration
*   im.imoasres     (io) Partial OAS fraction
*   im.imioas       (o)  OAS benefits
*   im.imoaspar     (o)  Partial OAS residency flag
*
*Calls:
*   maxi     Find the maximum of two quantities
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void Aoas(
	P_hh hh
)
{
	register P_in in;
	register int ini;
	register int immi;

	DEBUG_ON("Aoas");

	/* TEST TEST TEST TEST */
	if (hh->hd.hdseqhh>3390) {
		ini=0;
	}
	
	
	
	
	if (hh->hd.hdageeld < 60) {
		DEBUG1("%s skipping household, no-one aged 60+\n");
		DEBUG_OFF("Aoas");
		return;
	}

	/**
	* Calculate OAS residential eligibility variables for all 60+
	* persons and benefits received for all 65+ persons.
	* Do for each person  over age 59 in household
	**/

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
		         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

		/* calculate OAS residential variables for all 60+ persons */
		/* and benefits received for all 65+ persons               */
		if (in->id.idage < 60) {
			DEBUG1("%s skipping non-elderly person\n");
			continue;
		}

		/**
		* Determine years of residence in Canada prior to reaching age 65  
		* for non-Canadian born and over age 65.
		* Also, leave people reporting 40 at 40 (artefact of LFS Question)
		**/

		immi = in->id.idimmi;
		DEBUG2("%s immigration status=%d\n", immi)


		/* calculate OAS residential eligibility and benefits */


		if (immi<10) {
			/**
			* Less that 10 years residence get no OAS
			**/

			in->im.imoasres = (NUMBER) 0.0;
			DEBUG1("%s oasres=0\n");
			continue;
		}
		
		
		
		/**
		* Canadian Born Persons (immi=99) and Immigrants of over 39 years and
		* immigrants who were over 25 and in Canada in 1977 with 10 years
		* residency prior to application get full OAS.
		**/

		else if ((immi == 99 || immi >= 40) ||
		        ((MP.TARGETYEAR - immi) <= 1977)) {

			in->im.imoasres = ONE;
			DEBUG1("%s oasres=1\n");
			if (in->id.idage >= 65 && MP.OASFLAG) {
				in->im.imioas = MP.BOAS;
			}
		}
		else {
			/**	 PARTIAL OAS
			* persons with 10-39 years residence in Canada 
			* who immigrated after 1977 get partial OAS
			*
			* First adjust immi so that Partial OAS
			* will be received in the first year of eligibility 
			**/

			immi = maxi(10,immi-(in->id.idage - 65));

			in->im.imoasres = (NUMBER) (immi/40.0);
			/* set partial OAS residency flag */

			in->im.imoaspar = 1;
			DEBUG2("%s oasres=%.2f\n", in->im.imoasres);
			if (in->id.idage >= 65 && MP.OASFLAG) {
				in->im.imioas = in->im.imoasres * MP.BOAS;
			}
		}
	}

	DEBUG_OFF("Aoas");
}
