/***
*Atxccea.cpp - Compute child care expense allowance
*
*$Id: Atxccea.cpp 6.100 2007/09/18 17:22:10 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/02/06 | Minor changes to imicnet, iminet assignments            |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/18 | Removed imuccb adjustment - ccea uses real net income   |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/16 | Add imbcss to net income for income tests               |
* | LP  | 06/05/30 | Subtract imuccb from net income tests                   |
* | LP  | 06/05/05 | Removed CCEALIM - no longer used                        |
* | LP  | 05/04/15 | Added BXMEXT to eligibility test                        |
* | LP  | 04/02/18 | Added idiworkc to net income tests for cce              |
* | LP  | 03/12/19 | Added CCEPTFLG part time student eligibility for cce    |
* | CH  | 03/10/17 | Replaced ideducm by ideducfm                            |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/03/09 | Changed full-time student calculations (add CCESYNG/OLD)|
* | ER  | 96/04/16 | CCESFLAG - Full-time student flag                       |
* | SFG | 97/04/30 | MS-DOS: remove near/far & adjust overlay (memory probs.)|
* | BBM | 96/04/16 | Parameterize the maximum age for old kid to get CCEA    |
* | RSE | 94/03/01 | Ensure child's net income less than the basic amount    |
* | SFG | 91/06/18 | Bug fix: ensure earned income is nneg'd                 |
* | SFG | 90/07/17 | Use MP.CCEAEMPF instead of hard-coded 0.66667           |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxccea  [global]  Compute child care expense allowance
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxccea[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxccea - Compute child care expense allowance
*
*Synopsis:
*    void Atxccea(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   isnzero  Is argument non-zero ?
*   nneg     Change negative numbers to zero
*
*Structure Members:
*   MP.BXM          (i)  Basic personal exemption/amount
*   MP.BXMEXT       (i)  Basic personal amount extra top-up
*   MP.CCEAEMPF     (i)  CCEA fraction of employment income (Limit `B')
*   MP.CCEAOAGE     (i)  CCEA Maximum eligible age for oldest child
*   MP.CCEAOLD      (i)  CCEA old child limit
*   MP.CCEATFLG     (i)  CCEA eligible->claimable transformation activation flag
*   MP.CCEATFR      (i)  CCEA eligible->claimable fraction
*   MP.CCEATLL      (i)  CCEA eligible->claimable lower limit
*   MP.CCEAYNG      (i)  CCEA young child limit
*   MP.CCEPTFLG     (i)  CCE part-time student flag
*   MP.CCEROPT      (i)  Child care expense deduction recipient [1=mother,2=lower income]
*   MP.CCESFLAG     (i)  CCE full-time student flag
*   MP.CCESOLD      (i)  CCEA old child limit for full-time students
*   MP.CCESYNG      (i)  CCEA young child limit for full-time students
*   MP.CCEZOPT      (i)  CCE young kid optimization rules [1=use, 2=zero, 3=optimize]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   id.idage        (i)  Age
*   id.idccet       (i)  Child care expenses associated with child
*   id.idccett      (i)  Child care expenses (Limit A, Form T778)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.idiemp       (i)  Wages & salaries
*   id.idisefm      (i)  Self-employed income - farming
*   id.idisenf      (i)  Self-employed income - non-farming
*   id.idsex        (i)  Sex
*   im.imccez       (i)  Is CCE zeroed?
*   im.imictot      (i)  Total Income (line 150)
*   im.imitot       (i)  Total income
*   im.imccec       (io) Child care expenses claimed on behalf of child
*   im.imdedft      (io) Deductions from total income
*   im.imicnet      (io) Net Income (line 236)
*   im.imccea       (o)  Child care expenses allowed
*   im.iminet       (o)  Net income
*
*Called by:
*   txinet   Compute net income
*
*****************************************************************************/
void Atxccea(
	P_hh hh
)
{
	int spogets;    /* 0=elder gets, 1=spouse gets */
	P_in ineld;
	P_in inspo;
	P_in recip;		 /* Pointer to recipent */
	P_in highre;	 /* Pointer to recipent with higher income*/
	int cfi;
	P_in in;
	int ini;
	register P_cf cf;
	int spoflg;
	NUMBER earnings; 	  /* income CCE deduction is applied to - Earned Only */
	NUMBER income; 	      /* income CCE deduction is applied to - Net income for students */
	NUMBER highearn; 	  /* income CCE deduction is applied to - Earned Only for spouse with high income */
	NUMBER highinc; 	  /* income CCE deduction is applied to - Net income for students for spouse with high income */
	NUMBER ccee = ZERO;   /* eligible child care expenses for child */
	NUMBER ccec = ZERO;   /* child care expenses claimed for child */
	NUMBER ccea = ZERO;   /* child care expenses claimed for spouse with lower income */
	NUMBER highccea = ZERO;   /* child care expenses claimed spouse with higher income */
	NUMBER limitA = ZERO; /* limitation `A' in form (amount paid) */
	NUMBER limitB = ZERO; /* limitation `B' in form (CCEAEMPF * earned income) */
	NUMBER highlimB = ZERO; /* limitation `B' in form (CCEAEMPF * earned income) for spouse with higher income */
	NUMBER limitC = ZERO; /* limitation `C' in form (2000*old+4000*young) */
	NUMBER excess = ZERO; /* excess amount of child care expenses */
	NUMBER limitD = ZERO; /* limitation due to being in school full time (90*old+150*young in 1996) */
	NUMBER highlimD = ZERO; /* limitation due to being in school full time (90*old+150*young in 1996) for spouse with high income */
	int highflg;          /* flag for case with person with lower income in school full time */
	int earnflg;          /* flag when ccea is reduced due to earnings for spouse with higher earnings */
	NUMBER schamt = ZERO; /* amount added to ccea if person is in school full time */
	NUMBER highsch = ZERO; /* amount added to ccea if spouse with higher income is in school full time */


	DEBUG_ON("Atxccea");

	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {


		limitA = ZERO; /* amount paid in child care expenses */
		limitC = ZERO; /* maximum by age of children*/
		limitD = ZERO; /* maximum by age of children for additional school amounts  */
		highflg = 0;
		earnflg = 0;


		/* Use FAMEX-derived SPSD amount that varies with child, and fudge */

		/* first determine allowed amount for family */
		/* process kids in census family to derive limits */
		for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++) {
			ini++;

        	 /* determine total amount of child care expenses per child */
			if (MP.CCEATFLG) {
   				/* fudge eligible amount from FAMEX total child care expenses */
            	if ((in->id.idccet < MP.CCEATLL) || (in->im.imicnet > (MP.BXM + MP.BXMEXT))) {
				   ccee = ZERO;
	   			}
		   		else {
			   		ccee = MP.CCEATFR * in->id.idccet;
				}
			}
	        else {  /* Use Greenbook imputed amount per child */
		   		ccee = in->id.idccett;
        	}

			/* subject to limits as a function of age */
			if (in->id.idage <= 6) {

				/* apply optimization or test option */
				switch (MP.CCEZOPT) {

					case 1:  /* always use CCE */
						limitC += MP.CCEAYNG;
						ccec = ccee;
						limitD += MP.CCESYNG;
						break;

					case 2:  /* always set CCE to zero */
						ccec = ZERO;
						break;

					case 3:  /* look at variable uvccez to determine method */
						if (in->im.imccez) {
							ccec = ZERO;
						}
						else {
							limitC += MP.CCEAYNG;
							ccec = ccee;
							limitD += MP.CCESYNG;
						}
						break;

				}
			}

			else if (in->id.idage <= MP.CCEAOAGE) {
				limitC += MP.CCEAOLD;
				ccec = ccee;
				limitD += MP.CCESOLD;
			}

			else {
				/* child too old to be eligible */
				continue;
			}

			/* assign amount claimed on behalf of child */
			in->im.imccec = ccec;

			/* cumulate amount for family */
			limitA += ccec;
		}

		/* let ccea be the minimum of limitations A and C */
		/* in 1996, limit A = Part B, line 4 - total child care expenses 
		            limit C = Part B, line 3 - max per child */
		ccea = (limitA < limitC) ? limitA : limitC;

		/* initialise who gets the credit*/
		spogets = 0;

		/* get elder's pointer */
		ineld = cf->cfineld;

		/* determine if spouse is present in census family */
		spoflg = cf->cfspoflg;
		if (spoflg) {
			DEBUG1("%s spouse present\n");
			/* get spouse's pointer */
			inspo = cf->cfinspo;
		}
		
		if (isnzero(ccea)) {
			/* determine who receives ccea */
			if (MP.CCEROPT == 1) {
				DEBUG1("%s mother (if present) receives ccea\n");
				if (spoflg && (inspo->id.idsex == FEMALE)) {
					DEBUG2("%s spouse receives ccea=%.2f\n", ccea);
					spogets = 1;
					recip=inspo;
				}
				else {
					DEBUG2("%s elder receives ccea=%.2f\n", ccea);
					recip=ineld;
				}
			}
			else /* (MP.CCEROPT == 2) */ {
				DEBUG1("%s partner with lower net income receives ccea\n");
				if (spoflg && (inspo->im.imicnet <= ineld->im.imicnet)) {
					DEBUG2("%s spouse receives ccea=%.2f\n", ccea);
					spogets = 1;
					recip=inspo;
					/* exception to the rule: if person with lower income was in school full
					   time, then both parents can claim benefit */
					/* beginning in 1998, part time students were included */ 
					if ((MP.CCESFLAG && inspo->id.ideducfm>0) ||
						(MP.CCEPTFLG && inspo->id.ideducpm>0)) {
					   highre=ineld; /* pointer to person with higher net income */
					   highflg = 1;
					   highccea = ccea;
					}
				}
				else {
					DEBUG2("%s elder receives ccea=%.2f\n", ccea);
					recip=ineld;
					/* exception to the rule: if person with lower income was in school full
					   time, then both parents can claim benefit */
					/* beginning in 1998, part time students were included */
					if ((spoflg && MP.CCESFLAG && ineld->id.ideducfm>0) ||
						(spoflg && MP.CCEPTFLG && ineld->id.ideducpm>0)) {
					   highre=inspo; /* pointer to person with hihger net income */
					   highflg = 1;
					   highccea = ccea;
					}
				}
			}


			/* if CCESFLAG is on, and if one spouse is in school full time, then spouse with higher 
			   income can claim part of the credit.  Do this part first */
			/* extended to part-time students in 1998, highflg will reflect those eligible */
		    if (highflg) {
				/* Note that limits due to child care expenses and limits per child remain the 
				   same for both spouses (e.g. limitA and limitC) */
				highearn = nneg(highre->id.idiemp + highre->id.idisenf + highre->id.idisefm);
				/* apply overall limit of CCEAEMPF percent of earnings */
				highlimB = MP.CCEAEMPF * highearn; 
				/* in 1996, highlimB = Part B, line 5 - % of earnings */
				if (highccea > highlimB) {
					highccea = highlimB;
					earnflg = 1;
				}
				/* in 1996, we have finished Part B, line 6.
				   Start part C 
				   Multiply the weeks the spouse with the lower income was in school full-time 
				     with limitD.  (Note that weeks in high school are also eligible as of 1997,
				     but are not modeled)*/
				/* beginning in 1998, part-time school attendance is also considered */
				
				if (MP.CCEPTFLG) {
					highlimD = limitD * ((recip->id.ideducfm * 4) + recip->id.ideducpm);
				}
				else {
					highlimD = limitD * recip->id.ideducfm * 4;
				}
				/* in 1996, Part C, line 14 */
				if (highccea > highlimD) {
					highccea = highlimD;
				}
				/* if both spouses in school, and if highccea was reduced due to low
				   earnings, then go to part D. */
				/* beginning in 1998 part time students are considered */

				if ((highre->id.ideducfm>0 && earnflg) || 
					(highre->id.ideducpm>0 && earnflg && MP.CCEPTFLG)) {

					/* in 1996, Part D, line 19 
					   Multiply the weeks the spouse with the higher income was in school full-time 
  				       with limitD.  (Note that weeks in high school are also eligible as of 1997,
				       but are not modeled)*/
				    /* beginning in 1998, part time school attendance is also considered*/
				    
				    
				    
				    highsch = limitD * ((highre->id.ideducfm * 4) 
				    					+ (highre->id.ideducpm * MP.CCEPTFLG));

					/* in 1996, Part D, line 20.
					   difference of actual child care expenses (line 4) and highccea (line 14)*/
					if (nneg(limitA-highccea)<highsch) {
						highsch = nneg(limitA-highccea);
					}

					/* in 1996, Part D, line 22.
					   difference in max per child (line 3) and highccea (line 14)*/
					if (nneg(limitC-highccea)<highsch) {
						highsch = nneg(limitC-highccea);
					}

					/* in 1996, Part D, line 21.
					   use net income times MP.CCEAEMPF */
					highinc = highre->im.imicnet;
					if ((highinc * MP.CCEAEMPF)<highsch) {
						highsch = highinc * MP.CCEAEMPF;
					}

					/* in 1996, Part D, line 23.
					   weeks low income spouse in school times amount per child (line 13) minus
					   earned income * MP.CCEAMPF (line 5) */
					if (nneg(highlimD-highlimB)<highsch) {
						highsch = nneg(highlimD-highlimB);
					}

					/* add this extra amount (in 1996, line 24) to basic ccea (line 14) */
					highccea += highsch;
				}

				/* if ccea for spouse with higher net income is bigger than zero, then
				   add to permanent variables */
				if (highccea>0) {
					highre->im.imccea = highccea;
					highre->im.imdedft += highccea;
		    		highre->im.imicnet = nneg(highre->im.imictot - highre->im.imdedft);
					highre->im.iminet = nneg(highre->im.imitot - highre->im.imdedft);
				}
			}

			/* Back to spouse with lower income (or single parent)
			   Limit ccea expenses due to income, amount for high income spouse, and 
			   add extra school amount when appropriate */
			earnings = nneg(recip->id.idiemp + recip->id.idisenf + recip->id.idisefm);
			
			DEBUG1("%s child care expenses deduction\n");
			/* apply overall limit of CCEAEMPF percent of income */
			/* in 1996, line 5 */
			limitB = MP.CCEAEMPF * earnings;
			if (ccea > limitB) {
				ccea = limitB;
			}

			/* if spouse with higher income deducted ccea, then substract it.
			   In 1996, line 7 */
			if (highccea>0) {
				ccea = nneg(ccea - highccea);
			}

			/* if in school full time, and if ccea was reduced due to earnings, and
			   if spouse with higher income did NOT deduct ccea, then go to part D and add
			   a school amount */
			if (((MP.CCESFLAG && recip->id.ideducfm > 0) || (MP.CCEPTFLG && recip->id.ideducpm > 0))
			     && ccea==limitB && highccea==0) {
				/* in 1996, Part D, line 19 
				   Multiply the weeks in school full-time 
			       with limitD.  (Note that weeks in high school are also eligible as of 1997,
			       but are not modeled)*/
				schamt = limitD * ((recip->id.ideducfm * 4) + (recip->id.ideducpm * MP.CCEPTFLG));

				/* in 1996, Part D, line 20.
				   difference of actual child care expenses (line 4) and ccea (line 8)*/
				if ((limitA - ccea) < schamt) {
					schamt = limitA - ccea;
				}

				/* in 1996, Part D, line 22.
				   difference in max per child (line 3) and ccea (line 8)*/
				if ((limitC - ccea) < schamt) {
					schamt = limitC - ccea;
				}

				/* in 1996, Part D, line 21.
				   use net income times MP.CCEAEMPF */
				income = recip->im.imicnet;
				if ((income * MP.CCEAEMPF) < schamt) {
					schamt = income * MP.CCEAEMPF;
				}

				/* add schamt (in 1996, line 24) to ccea (in 1996, line 8) */
				ccea += schamt;
			}
		
			recip->im.imccea = ccea;
			recip->im.imdedft += ccea;
     		recip->im.imicnet = nneg(recip->im.imictot - recip->im.imdedft);
			recip->im.iminet = nneg(recip->im.imitot - recip->im.imdedft );
			DEBUG2("%s revised icnet =%.2f\n", recip->im.imicnet);
			DEBUG2("%s revised inet =%.2f\n", recip->im.iminet);
			DEBUG2("%s revised dedft=%.2f\n", recip->im.imdedft);
		
  			/**
			* At this point, reduce the amounts claimed on behalf
			* of young children, so that the supplement for young
			* children on the CTC can be maximized.
			* Only do this if the use of the child-oriented expenditure
			* variable has been activated with the `fudge' parameters.
			* Also, this is only applicable if the amount of ccea has been
			* reduced through limitB or limitC.
			* Another way to put this is, we are redistributing the expenses
			* allowed (determined by limitB and limitC) back to the kids by
			* reducing the amounts claimed for each child.
			**/

			if (MP.CCEATFLG && limitA > ccea) {

				excess = limitA - ccea;

				/* Note that kids are in order of increasing age */
				for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++, ini++) {

					if (in->im.imccec > ZERO && excess > ZERO) {
						if (excess > in->im.imccec) {
							excess -= in->im.imccec;
							in->im.imccec = ZERO;
						}
						else {
							in->im.imccec -= excess;
							excess = ZERO;
							continue;
						}
					}
				}
			}
		}
	}


	DEBUG_OFF("Atxccea");
	return;
}
