 /***
*Atxman.cpp - Compute provincial taxes for Manitoba
*
*$Id: Atxman.cpp 6.126 2007/09/18 17:22:11 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/07/30 | add flag so people with amt cannot have foreign tx credit |
* | LP  | 07/07/08 | Added conceptual design of the Manitoba Child Benefit   |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/18 | Removed MUCCBITF-UCCB removed from income as per forms  |
* | LP  | 06/12/27 | Removed MTCINCFL - unnecessary given net income changes |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/09 | Removed MAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added MUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate MDTCRLC|
* | LP  | 06/04/20 | Added a minimum age parameter for cost of living credit |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 04/02/17 | Added idiworkc to income concept when MCTINCFL set to 1 |
* |     |          | and updated net income concepts for tax and credit calc |
* | LP  | 03/12/22 | Added MMEDALL and MMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/09 | Added impothpe, MODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added MCGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/13 | Added ideducpm and MEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred |
* | LP  | 01/06/11 | Changed the options for idtenur due to database change  | 
* | LP  | 01/05/09 | Added MAMTPCTM and added another option to MAMTOPT      |
* | LP  | 01/03/12 | Removed pre-1988 program parameters                     |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | LP  | 00/05/16 | Added MTRNIOF Man. tax reduction net income off set     |
* | CH  | 00/05/15 | Modified Manitoba Homeowners Assitance                  |
* | CH  | 00/05/05 | Modified CGTC                                           |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/04/23 | Modified Manitoba Learning Tax credit and added older   |
* |     |          |    disabled reductions MRODISEX MSODISEX MCODISEX       |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | CH  | 99/03/09 | Added the homeowners' school tax assistance             |
* | BBM | 99/02/11 | Drop idshltr in calculating the property tax credit V7.0|
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CCH | 98/04/01 | Added max to learning tax credit                        |
* | CCH | 98/02/27 | Added MCEMXM and spouse's disability                    |
* | CCH | 97/09/10 | Add learning tax credit                                 |
* | BBM | 92/07/22 | Incorporated non-refundable provincial tax credit var   |
* | BBM | 92/07/09 | Add reference to impnit net income tax variable         |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxman   [global]  Compute provincial taxes for Manitoba
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxman[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxman - Compute provincial taxes for Manitoba
*
*Synopsis:
*    void Atxman(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   txhhexp  Compute and pro-rate household taxes, rent, etc.
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   minn     Find the minimum of two numbers
*   lkup1    Table look-up with interpolation/extrapolation
*   maxn     Find the maximum of two numbers
*
*Structure Members:
*   MP.EDXPM        (i)  Education amount per month
*   MP.MANLTCF      (i)  Manitoba learning tax credit factor
*   MP.MANLTMAX     (i)  Manitoba learning tax credit maximum amount per student
*   MP.MANRD        (i)  Manitoba tax reduction table (MNRDOPT=1) [federal reduction, net income]
*   MP.MANRDrows    (i)  number of rows in MANRD
*   MP.MCAXM        (i)  Manitoba cost-of-living age credit
*   MP.MCBCR        (i)  Manitoba cost-of-living basic credit rate
*   MP.MCBXM        (i)  Manitoba cost-of-living basic credit
*   MP.MCDISEX      (i)  Manitoba cost-of-living disability credit
*   MP.MCEMXM       (i)  Manitoba cost-of-living married equivalent credit
*   MP.MCFINCRR     (i)  Manitoba cost-of-living family income reduction rate
*   MP.MCMXM        (i)  Manitoba cost-of-living married credit
*   MP.MCOLOPT      (i)  Manitoba cost-of-living tax credit calculation option
*   MP.MCSAXM       (i)  Manitoba cost-of-living spouse age credit
*   MP.MCSDISEX     (i)  Manitoba cost-of-living spouse disabled credit
*   MP.MCYCXM       (i)  Manitoba cost-of-living young dependants credit
*   MP.MNITR        (i)  Manitoba net income tax rate
*   MP.MNRDOPT      (i)  Manitoba tax reduction calculation option
*   MP.MNSUROPT     (i)  Manitoba surtax calculation option
*   MP.MPAXM        (i)  Manitoba property tax credit senior citizen credit
*   MP.MPBXM        (i)  Manitoba property tax credit basic credit
*   MP.MPFINCRR     (i)  Manitoba property tax credit family income reduction rate
*   MP.MPMINC       (i)  Manitoba property tax credit minimum credit
*   MP.MPRIR        (i)  Manitoba property tax credit rental inclusion rate
*   MP.MPTC         (i)  Manitoba political contribution table [total donations,donation allowed]
*   MP.MPTCBEN      (i)  Maximum Manitoba political tax credit allowed
*   MP.MPTCrows     (i)  number of rows in MPTC
*   MP.MPTF         (i)  Manitoba provincial tax fraction
*   MP.MRAXM        (i)  Manitoba tax reduction age credit
*   MP.MRBXM        (i)  Manitoba tax reduction basic credit
*   MP.MRDISEX      (i)  Manitoba tax reduction disability credit
*   MP.MREMXM       (i)  Manitoba tax reduction married equivalent credit
*   MP.MRMXM        (i)  Manitoba tax reduction married credit
*   MP.MRSAXM       (i)  Manitoba tax reduction spouse age credit
*   MP.MRYCXM       (i)  Manitoba tax reduction young dependants credit
*   MP.MSAXM        (i)  Manitoba surtax age credit
*   MP.MSBXM        (i)  Manitoba surtax basic credit
*   MP.MSDISEX      (i)  Manitoba surtax disability credit
*   MP.MSEMXM       (i)  Manitoba surtax married equivalent credit
*   MP.MSMXM        (i)  Manitoba surtax married credit
*   MP.MSSAXM       (i)  Manitoba surtax spouse age credit
*   MP.MSTC         (i)  Manitoba surtax cut-in
*   MP.MSTR         (i)  Manitoba surtax rate
*   MP.MSYCXM       (i)  Manitoba surtax young dependants credit
*   MP.MTCINCFL     (i)  Manitoba tax credit income definition flag
*   MP.MTRBR        (i)  Manitoba tax reduction basic amount
*   MP.MTRF         (i)  Manitoba tax reduction fraction
*   MP.MTRSF        (i)  Manitoba tax reduction spousal factor
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducm      (i)  Eligible months of education allowance
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idsheltr     (i)  Manitoba shelter allowance (T1C-Man)
*   id.idspoflg     (i)  Person has spouse
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imatxc       (i)  Age tax credit
*   im.imbft        (i)  Basic federal tax
*   im.imdisex      (i)  Disability exemption
*   im.imedrcv      (i)  Education and tuition transfered from others
*   im.imedtrf      (i)  Education and tuition transfered to others
*   im.imeduc       (i)  Education allowance for student (322)
*   im.imftr        (i)  Federal tax reduction
*   im.imftrt       (i)  Federal tax reduction transferred from spouse
*   im.imigis       (i)  GIS benefits
*   im.iminet       (i)  Net income
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imispa       (i)  Spouse's allowance
*   im.imitax       (i)  Taxable income
*   im.immartxc     (i)  Married tax credit claimed
*   im.imnfach      (i)  Number of family allowance children claimed
*   im.impex        (i)  All personal exemptions and deductions
*   im.improptx     (i)  Imputed property tax paid
*   im.imrentpd     (i)  Imputed rent paid
*   im.imstxcrt     (i)  Tax credits transfered from spouse
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imbpt        (io) Basic provincial tax
*   im.immanltc     (io) Manitoba learning tax credit
*   im.imnptc       (io) Non-refundable provincial tax credits
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.impsur       (io) Provincial surtax
*   im.imptr        (io) Provincial tax reduction
*   im.imtxp        (io) Provincial income tax payable
*   im.impnit       (o)  Provincial net income tax
*   im.impptxtc     (o)  Provincial Property tax tax credit
*   im.imptc        (o)  Refundable provincial tax credits
*
*****************************************************************************/
void Atxman(
	P_hh hh
)
{
    register int ini;
    register P_in in;
    register P_in inspo;
    register    P_nf nf;
    register    P_in nfeld;
    register    P_in nfspo;
    register int nfi;
    register P_in ineld;
    register P_cf cf;
    register int cfi;
    int prtflg;    /* Property Tax credit flag */
    NUMBER manb = ZERO;
    NUMBER mantr = ZERO;
    NUMBER mannit = ZERO;    /* Manitoba net income tax */
	NUMBER adjnet = ZERO;    /* Manitoba adjusted net income concept */
    NUMBER mptr = ZERO;      /* Manitoba tax reduction credits */
    NUMBER mpsur = ZERO;     /* Manitoba surtax credits */
    NUMBER surth = ZERO;     /* Manitoba surtax income cut in */
    NUMBER colc = ZERO;      /* Manitoba cost of living credit */
    NUMBER netcolc = ZERO;   /* Manitoba cost of living credit reduced */
    NUMBER oc = ZERO;        /* Occupancy Cost */
    NUMBER prtc = ZERO;      /* Property Tax credit temporary variable */
    NUMBER netprtc = ZERO;   /* Property Tax credit reduced */
    NUMBER faminet = ZERO;   /* family net income */
    NUMBER pfaminet = ZERO;  /* %age of family net income (for reduction)*/
    NUMBER poltcr = ZERO;    /* Political tax credit (raw) */
    NUMBER poltc = ZERO;     /* Political tax credit */
	NUMBER schpd = ZERO;     /* Manitoba school taxes paid */
	NUMBER tmpsch = ZERO;    /* Temporary maximum amount of school tax assistance */
	NUMBER schmax = ZERO;    /* Maximum amount of school taxes paid */
    NUMBER lvctc = ZERO;   /* Labour-sponsored venture capital tax cred */
	NUMBER mtrioffs = ZERO;  /* Manitoba tax reduction net income offset */
	NUMBER nfinet = ZERO;    /* family net income concept for Man. Child Benefit */
	NUMBER nfiemp = ZERO;    /* family earned income for Man. Child Benefit */
	NUMBER mcb = ZERO;       /* Manitoba Child Benefit */
	NUMBER mcbrr = ZERO;     /* Manitoba Child Benefit reduction rate */

    DEBUG_ON("Atxman");

    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.MTXFLG) { 

       /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;

          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */

          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni ;
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.MPNTCR * nneg(MP.MEMXM - nneg(cinet - MP.MEMXMT));
             }
          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption 
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.MPNTCR * nneg(MP.MMXM - nneg(inspo->im.imdepni - MP.MMXMT));
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.MPNTCR * nneg(MP.MMXM - nneg(ineld->im.imdepni - MP.MMXMT));
             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
          }
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/

			 /* calculate caregivers tax credit */
			 if (MP.MCGTCFLG && isnzero(in->id.idcgtc)) {
				/* if MGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.MCGTCOPT == 1) {
					in->im.impcgtc = MP.MCGTC * MP.MPNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.MPNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }

             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.MPNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.MPNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.MPNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.MPNTCR * MP.MBXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc = in->im.imicnet;

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.MAXM, MP.MAXRR * nneg(netinc - MP.MAXTD));
                adjamt = MP.MAXM - agered;
                in->im.impatxc = MP.MPNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.MPNTCR * minn(MP.MYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.MMEDALL, MP.MMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.MPNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

			 /* calculate amount for infirm dependents age 18 or older */
			 /*  if the filer has an amount for idothpe, then give max credit */

			 if (isnzero(in->id.idothpe)) {
   				in->im.impothpe = MP.MPNTCR * MP.MODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
  			 }

             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.MCHATL1, in->im.imchara)  * MP.MCHATR1
                            + nneg(in->im.imchara - MP.MCHATL1) * MP.MCHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.MPNTCR * MP.MMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.MPNTCR * MP.MMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.MPNTCR * (in->id.ideducfm * MP.MEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.MPNTCR * (in->id.ideducpm * MP.MEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.MPNTCR * (in->id.ideducpm * MP.MEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }
			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.MPNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }
             /* calculate federal dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.MDTCR)
								+ (in->im.imildivt * MP.MDTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }

			 /* calculate the Manitoba minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.MAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use MAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.MAMTPCTF;
				 }
				 else if (MP.MAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using MAMTTX. */
						in->im.impminco = in->id.idmincar * MP.MAMTTX;
					  }
				 else if (MP.MAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using MAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.MAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
            
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */
			 if (in->im.imamtfg && MP.MAMTOPT == 3) {
                in->im.imprvtax = MP.MAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.MPTX, MP.MPTXrows, in->im.imitax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
                        + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
						+ in->im.impminco    /* provincial minimum tax carryover credit */
						+ in->im.impintsl   /* provincial interest on student loans credit */
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.MPNTCR * MP.MMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.MPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.MPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                       + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco  /* provincial minimum tax carryover credit */
	        		   + in->im.impintsl   /* provincial interest on student loans credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
				    + in->im.impcgtc    /* caregiver tax credit                  */
                    + in->im.imppentc   /* pension income tax credit             */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.MPNTCR * MP.MMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.MPNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.MBXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic ));
                                                         
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

    /* call the function which performs the inputation of paid rent
    *  and property taxes to individuals in the hhld
    */
    txhhexp(hh);


    /* process each person in household */
    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
           ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);
        
        /* initialize pointers */
        inspo = in->id.idinspo;

        /* original tax as percentage of federal tax */
        if (!MP.MTXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.MPTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if MTXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.MAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.MAMTPCTF;
			}
			else {
				if ( in->im.imamtfg && MP.MAMTOPT == 4 ) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.MAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}

        /* Calculate manitoba net income tax and add to tax paid */
		/* Manitoba adjusts net income for this test - imicnet less line 250 */
		/* for 1992 to 2000 only */
		if (MP.TARGETYEAR >= 1992 && MP.TARGETYEAR <= 2000 ) {
			adjnet = nneg(in->im.imicnet - in->im.imisa - in->im.imispa
						 - in->im.imigis - in->id.idiworkc);
		}
		else {  /* imicnet is ok for 1991, right concept for 2001 onwards */	
			adjnet = in->im.imicnet; 
		}
        mannit = MP.MNITR * adjnet;
        in->im.impnit = mannit;
        in->im.imtxp += mannit;


        DEBUG4("%s man bpt=%.2f, txp=%.2f neti=%.2f\n",
                 in->im.imbpt, in->im.imtxp, in->im.imicnet);

        /* CALCULATE PROVINCIAL SURTAX */


        DEBUG1("%s man surtax based on credits\n");

        /* calculate the threshold at which one pays the surtax */
        /* we will use the threshold to skip all ind whose netinc */
        /* is too low.                                            */
        surth = MP.MSBXM / MP.MNITR; 

		/* Here imicnet less line 250 is the right concept as the adjusted net income */
		/* used in the surtax calculation excludes sa, wcb and net fed supp */
		/* for 1992 to 2000 (adjusted above) */

        if (adjnet >= surth) {
            DEBUG1("%s man sur: persons above net inc threshold\n");

            DEBUG1("%s man sur: basic credit for self and dep kids\n");
            mpsur = MP.MSBXM + (in->im.imnfach * MP.MSYCXM);

            if (isnzero(in->im.imatxc)) {
                DEBUG1("%s man sur: age credit for self\n");
                mpsur+= MP.MSAXM;
            }

            if (isnzero(in->id.iddisslf)) {
                DEBUG1("%s man sur: disability credit for self\n");
                mpsur += MP.MSDISEX;
            }

            if (isnzero(in->id.iddisoth)) {
                DEBUG1("%s man sur: disability credit for others\n");
                mpsur += MP.MSDISEX;
            }

            if (isnzero(in->id.idothpe)) {
                DEBUG1("%s man sur: disability credit for other disabled dependents\n");
                mpsur += MP.MSODISEX;
            }

            if (in->id.idspoflg) {
                if (isnzero(in->im.immartxc)) {
                    DEBUG1("%s man sur: credit for dependent spouse\n");
                    mpsur+= MP.MSMXM;
                }
                if (isnzero(in->im.imstxcrt) && isnzero(inspo->im.imatxc)){
                    DEBUG1("%s man sur: age credit for spouse\n");
                    mpsur += MP.MSSAXM;
                }
                if (isnzero(in->im.imstxcrt) && isnzero(inspo->id.iddisslf)){
                    DEBUG1("%s man sur: disabled credit for spouse\n");
                    mpsur += MP.MSDISEX;
                }
            }
            else {
                if (isnzero(in->im.immartxc)) {
                    DEBUG1("%s man sur: credit for equiv to married\n");
                    mpsur+= MP.MSEMXM;
                }
            }

            /* calc net surtax */
            in->im.impsur = nneg(mannit-mpsur);
                DEBUG3("%s man sur: mannit=%.2f mpsur=%.2f\n", mannit, mpsur);
        }
        else {
            DEBUG1("%s man sur: persons below net inc threshold\n");
            in->im.impsur = ZERO;
        }

        /* apply surtax */
        in->im.imtxp += in->im.impsur;
        DEBUG3("%s man sur: surth=%.2f impsur=%.2f\n", surth, in->im.impsur);


        /* CALCULATE AND APPLY PROVINCIAL FOREIGN TAX CREDIT */

        /* calculate and apply provincial foreign tax credit */
        if (MP.MAMTFTCFLG && in->im.imamtfg) {
            /* When the flag is turned on, these people aren't eligible for the foreign tax credit */
        }
        else {
            /* Eligible for the foreign tax credit.  Note that people who have the 
               amt would have a different way of calculating the credit.  It is not 
               possible to do that in the SPSM */
            in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
            in->im.imtxp -= in->im.imnptc;
        }


        /* CALCULATE TAX REDUCTION */


        DEBUG1("%s man tax reduction based on credits\n");

        if ( in->im.imtxp > ZERO ) {
		  NUMBER netinc = ZERO;
          DEBUG1("%s man red: calc for those with imtxp>0\n");

          DEBUG1("%s man red: basic credit for self and dep kids\n");
          mptr = MP.MRBXM + (in->im.imnfach * MP.MRYCXM);

 	      if (in->im.imatxc) {
 	          DEBUG1("%s man red: age credit for self\n");
 	          mptr+= MP.MRAXM;
 	      }

 	      if (isnzero(in->id.iddisslf)) {
 	          DEBUG1("%s man red: disability credit for self\n");
 	          mptr += MP.MRDISEX;
 	      }

 	      if (isnzero(in->id.iddisoth)) {
 	           DEBUG1("%s man red: disability credit for spouse\n");
 	           mptr += MP.MRDISEX;
 	       }

 	       if (isnzero(in->id.idothpe)) {
 	           DEBUG1("%s man red: disability credit for other dependent\n");
 	           mptr += MP.MRODISEX;
 	       }

 	       if (in->id.idspoflg) {
 	          if (isnzero(in->im.immartxc)) {
 	                    DEBUG1("%s man red: credit for dependent spouse\n");
 	                    mptr += MP.MRMXM;
 	           }
 	           if (isnzero(in->im.imstxcrt) && isnzero(inspo->im.imatxc)){
 	               DEBUG1("%s man red: age credit for spouse\n");
 	               mptr += MP.MRSAXM;
 	           }
 	           if (isnzero(in->im.imstxcrt) && isnzero(inspo->id.iddisslf)){
 	               DEBUG1("%s man red: disability credit for spouse\n");
 	               mptr += MP.MRDISEX;
 	            }
 	       }
           else {
                if (isnzero(in->im.immartxc)) {
                   DEBUG1("%s man red: credit for equiv to married\n");
                   mptr += MP.MREMXM;
                }
            }
			/* Calculate the income offset to be used in final result of tax reduction */
			/* For the years 1992 to 2000, reduce net income (line 236) by SA, WCB, GIS, SPA*/
			/* For 1991, line 236 does not contain SA, WCB, GIS and SPA */
			if (MP.TARGETYEAR >= 1992 && MP.TARGETYEAR <= 2000 ) {
				netinc = nneg(in->im.imicnet - in->im.imisa - in->im.imispa
							- in->im.imigis - in->id.idiworkc);
			}
			else {
				netinc = in->im.imicnet ;
			}
			mtrioffs = MP.MTRNIOS * netinc;

            in->im.imptr = nneg(mptr-mtrioffs);
            DEBUG3("%s man red: mtrioffs=%.2f mptr=%.2f\n", mtrioffs, mptr);
        }

        else {
            DEBUG1("%s man red: zero for those with imtxp=0\n");
            in->im.imptr = ZERO;
        }

        /* apply tax reduction */
        /* tax reduction cannot be more than total provincial tax */
        in->im.imptr = minn(in->im.imptr, in->im.imtxp);
        in->im.imtxp -= in->im.imptr;

        DEBUG2("%s man red imptr=%.2f\n", in->im.imptr);

        /* Man labour-sponsored venture capital credit */
        if ( isnzero(in->id.idlabtxg) ) {
           lvctc = minn(MP.MLVCMAX, MP.MLVCRT * in->id.idlabtxg);
           in->im.implvctc = minn(lvctc, in->im.imtxp);
           in->im.imnptc += in->im.implvctc;
           in->im.imtxp  -= in->im.implvctc;
        }

        
        /* REFUNDABLE LEARNING TAX CREDIT */
        if ((in->id.idtuitn + (in->id.ideducfm * MP.EDXPM) +
            (in->id.ideducpm * MP.EDXPMPT) + in->im.imedrcv) > 0) {

            in->im.immanltc = nneg( minn(in->id.idtuitn + (in->id.ideducfm * MP.EDXPM)
            							  + (in->id.ideducpm * MP.EDXPMPT),
                                	     MP.MANLTMAX)
                                    - in->im.imedtrf
                                    + in->im.imedrcv )
                              * MP.MANLTCF;
            in->im.imptc += in->im.immanltc;
        }


        /* MANITOBA POLITICAL CONTRIBUTION TAX CREDIT */
        if ( isnzero(in->id.idprvpol) ) {
            poltcr = (NUMBER) lkup1(MP.MPTC, MP.MPTCrows, in->id.idprvpol);
            poltc = minn(MP.MPTCBEN, poltcr);
            in->im.impptc = minn(poltc, in->im.imtxp);
            DEBUG4("%s man pol: idpolcon=%.2f poltcr=%.2f poltc=%.2f\n",
               in->id.idprvpol, poltcr, poltc);

            /* apply political contribution tax credit */
            in->im.imnptc += in->im.impptc;
            in->im.imtxp -= in->im.impptc;

            DEBUG3("%s man idpolcon =%.2f impptc =%.2f\n",
                     in->id.idprvpol, in->im.impptc);
        }

    }


    /* MANITOBA FAMILY BASED TAX CREDITS */

    /* initialise temp variables */
    prtflg = 0 ;
	mcb = 0;
	nfinet = 0;
	mcbrr = 0;
	nfiemp = 0;

    /* process each nuclear family in household */
    for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

        /* initialize pointers */
        nfeld = nf->nfineld;
        nfspo = nf->nfinspo;

        /* PROPERTY TAX CREDIT */

        /* calculate the family's occupancy costs */
        /* assume kids have not been imputed rental
            or property tax expenses               */
        /* note: this credit is only received by the first NF in the hhld */
        /* When MHMASOPT is set to 1, the residential homeowners assistance is modelled.
            We assume that any family with non-zero property taxes received the full amount */
        /* When it is set to 2, the imputed idhomstu is used instead */

        if (MP.MHMASOPT == 1) {
            if (!nf->nfspoflg) {
                if (nfeld->im.improptx > 0) {
                    nfeld->im.immhmas = MP.MPMINC;
                }
            }
            /* if either the elder or the spouse had property taxes, set elder homeowner assistance 
               to minimum amount */
            else {
                if ((nfeld->im.improptx > 0) || (nfspo->im.improptx > 0)) {
                    nfeld->im.immhmas = MP.MPMINC;
                }
            }
        }
        /* set to idhomstu */
        else {
            nfeld->im.immhmas = nfeld->id.idhomstu;
            if (nf->nfspoflg) {
                nfspo->im.immhmas = nfspo->id.idhomstu;
            }
        }

        /* Derive Occuancy cost */
            
        if (!nf->nfspoflg) {
            DEBUG1("%s man prtc: no spouse\n");
            oc   = MP.MPRIR * nfeld->im.imrentpd + nfeld->im.improptx + nfeld->im.immhmas;
        }
        else {
            DEBUG1("%s man prtc: spouse present\n");
            oc   = MP.MPRIR * (nfeld->im.imrentpd + nfspo->im.imrentpd)
                + (nfeld->im.improptx + nfspo->im.improptx)
                + (nfeld->im.immhmas  + nfspo->im.immhmas);
        }   
		oc = nneg(oc - MP.MPMNTH);

        if ( isnzero(oc) && !prtflg ) {

            DEBUG1("%s man prtc: qualifier basic credit\n");
            prtc = MP.MPBXM;

            if (isnzero(nfeld->im.imatxc)) {
                DEBUG1("%s man prtc: bump up basic credit for the aged\n");
                prtc += MP.MPAXM;
            }
            DEBUG3("%s man prtc: oc=%.2f prtc=%.2f\n", oc, prtc);

            /* Calculate Net Property Tax Credit   												*/
				/* The deduction of family shetler costs has been removed for version 7.0	*/
				/* due to a lack of data for the imputation.  An overestimate will arise   */

            if (nfeld->id.idspoflg) { 
                DEBUG1("%s man net prtc: spouse present\n");
  				 faminet = nneg(nfeld->im.imicnet + nfspo->im.imicnet
                                - nfeld->im.imuccb - nfspo->im.imuccb ); 
				 /* for 1992 only, the net income concept for the tax credit programs */
				 /* uses line 236 less line 250 */
				 if (MP.TARGETYEAR == 1992) {
					faminet = nneg(faminet - nfeld->im.imisa - nfeld->im.imispa 
								- nfeld->im.imigis - nfeld->id.idiworkc 
								- nfspo->im.imisa - nfspo->im.imispa 
								- nfspo->im.imigis - nfspo->id.idiworkc);
				 }
            }
            else {    
                DEBUG1("%s man net prtc: no spouse present\n");
                faminet = nneg(nfeld->im.imicnet - nfeld->im.imuccb) ;
				/* for 1992 only, the net income concept for the tax credit programs */
				/* uses line 236 less line 250 */
				if (MP.TARGETYEAR == 1992) {
					faminet = nneg(faminet - nfeld->im.imisa - nfeld->im.imispa 
								- nfeld->im.imigis - nfeld->id.idiworkc); 
				}
     
            }
            pfaminet = faminet * MP.MPFINCRR;
            netprtc = maxn(MP.MPMINC, prtc - pfaminet);

			/* Homeowners' school tax assistance */

			/* only those aged 55 or older and who own their own house and have paid school tax are
			   eligible */
			/* note that people who are life tenants of their principle residences are also 
			   eligible, but people in the Manitoba Department of Finance indicated that they
			   would still be classified as home owners */
			if (MP.MSCHFLAG && nfeld->id.idage>=MP.MSCHAGE && hh->hd.hdtenur!=1) {
				schpd = nfeld->id.idmschpd;
				if (nf->nfspoflg) {
					schpd += nfspo->id.idmschpd;
				}
				schpd = nneg(schpd - MP.MSCHMNTH);
				tmpsch = (int) ((nneg(faminet-MP.MSCHTD)*MP.MSCHRR)/MP.MSCHRND);
				schmax = nneg(MP.MSCHBXM - (MP.MSCHRND*tmpsch));
				netprtc += minn(schpd,schmax);
			}

            netprtc = minn(netprtc,oc);
            if (nf->nfspoflg) {
                netprtc = nneg(netprtc - nfeld->im.immhmas - nfspo->im.immhmas);
            }
            else {
                netprtc = nneg(netprtc - nfeld->im.immhmas);
            }

            if (isnzero(netprtc)) {
                DEBUG1("%s man prtc: set prtflg to 1\n");
                prtflg = 1 ;
            }
            DEBUG2("%s man net prtc: prtc=%.2f\n",
                   prtc);
            DEBUG3("%s man net prtc: netprtc=%.2f faminet=%.2f\n",
                   netprtc, faminet);
        }

        else {
            DEBUG1("%s man prtc: nf did not qualify\n");
            netprtc = ZERO;
        }

        /* apply property tax credit */
        nfeld->im.impptxtc = netprtc;
        nfeld->im.imptc += netprtc;
        DEBUG3("%s man net prtc: impptxtc=%.2f imptc=%.2f\n",
               nfeld->im.impptxtc, nfeld->im.imptc);

        /* MANITOBA REFUNDABLE COST OF LIVING TAX CREDIT */

        /* calculate GROSS cost of living tax credit */

        /* Note: assume that spouses claim together */
		/*  Beginning in 2006, the minimum age to claim this */
		/*  credit was increased from 18 to 19.  Prior to 2006, */
		/*  the rules allowed those under 18 to claim the credit */
		/*  if they had a spouse, were a parent or could claim */
		/*  the education property tax credit */

		if (nfeld->id.idage >= MP.MCMINAGE || ( nfeld->id.idage < MP.MCMINAGE
			&& (nf->nfspoflg || nf->nfnkids > 0 || nfeld->im.impptxtc > 0)) ) { 

        	DEBUG1("%s man col: basic credit and kids\n");
        	colc = MP.MCBXM + (nf->nfnkids * MP.MCYCXM) ;

        	if (isnzero(nfeld->im.imatxc)) {
           		DEBUG1("%s man col: for aged head\n");
           		colc += MP.MCAXM;
        	}

        	if (isnzero(nfeld->id.iddisslf)) {
            	DEBUG1("%s man col: disability for head\n");
            	colc += MP.MCDISEX;
        	}

        	if (isnzero(nfeld->id.iddisoth)) {
            	DEBUG1("%s man col: disability for other\n");
            	colc += MP.MCDISEX;
        	}

        	if (isnzero(nfeld->id.idothpe)) {
            	DEBUG1("%s man col: disability for other dependents\n");
            	colc += MP.MCODISEX;
        	}

        	if (nf->nfspoflg) {
            	DEBUG1("%s man col: +basic married\n");
            	colc += MP.MCMXM;

            	if (isnzero(nfspo->im.imatxc)) {
                	DEBUG1("%s man col: +old spouse\n");
                	colc += MP.MCSAXM;
            	}

            	if (isnzero(nfspo->id.iddisslf)) {
                	DEBUG1("%s man col: +disabled spouse\n");
                	colc += MP.MCSDISEX;
            	}
        	}
       		else {   
            	if (isnzero(nfeld->im.immartxc)) {
                	DEBUG1("%s man col: plus equivtomarried\n");
                	colc += MP.MCEMXM;
            	}
        	}
        


        	/* calculate NET cost of living tax credit  
         	* by reducing credit by percentage of family income 
        	* assign to higher income spouse           */

        	if (!nf->nfspoflg) {
            	DEBUG1("%s man net col: no spouse\n");
				/* for 1992 only take line 236 less line 250 for cost of living tax credit */
				/* all other years, take line 236 net income */
				if (MP.TARGETYEAR == 1992) {
					pfaminet = nneg( nfeld->im.imicnet - nfeld->im.imisa - nfeld->im.imispa
							- nfeld->im.imigis - nfeld->id.idiworkc) * MP.MCFINCRR;
				}
				else {
               		pfaminet = nneg( nfeld->im.imicnet - nfeld->im.imuccb )	* MP.MCFINCRR; 
				}
				
            	netcolc = nneg(colc - pfaminet);
            	nfeld->im.imptc += netcolc; 
            	DEBUG4("%s man net col: elder netcolc=%.2f colc=%.2f eldinet=%.2f\n",
                	      netcolc, colc, nfeld->im.imicnet);
            	DEBUG2("%s man net col: imptc=%.2f\n", nfeld->im.imptc);
        	}
        	else {
            	DEBUG1("%s man net col: spouse present\n");
				if (MP.TARGETYEAR == 1992 ) {
					pfaminet = nneg(nfeld->im.imicnet + nfspo->im.imicnet 
									- nfeld->im.imisa - nfeld->im.imispa - nfeld->im.imigis
									- nfeld->id.idiworkc - nfspo->im.imisa - nfspo->im.imispa
									- nfspo->im.imigis - nfspo->id.idiworkc) * MP.MCFINCRR;
				}
				else {
               		pfaminet = nneg( nfeld->im.imicnet + nfspo->im.imicnet
                                     - nfeld->im.imuccb - nfspo->im.imuccb ) * MP.MCFINCRR; 
				}
            	netcolc = nneg(colc - pfaminet);

            	if (nfeld->im.imicnet >= nfspo->im.imicnet) {
               		nfeld->im.imptc += netcolc; 
                	DEBUG5("%s man net col: elder netcolc=%.2f colc=%.2f eldinet=%.2f spoinet=%.2f\n",
                          netcolc, colc, nfeld->im.imicnet, nfspo->im.imicnet);
                	DEBUG3("%s man net col: nfled->imptc=%.2f nfspo->imptc=%.2f\n",
                         nfeld->im.imptc, nfspo->im.imptc);
            	}
            	else {
                	nfspo->im.imptc += netcolc;
                	DEBUG5("%s man net col: spouse netcolc=%.2f colc=%.2f eldinet=%.2f spoinet=%.2f\n",
                          netcolc, colc, nfeld->im.imicnet, nfspo->im.imicnet);
                	DEBUG3("%s man net col: nfled->imptc=%.2f nfspo->imptc=%.2f\n",
                         nfeld->im.imptc, nfspo->im.imptc);
            	}
        	}
		}

		/* Manitoba Child Benefit */
		/* starts in January 2008, $420 per year per child, */
		/* partial benefits where income is between 15,000 and 20,000 */

		if (MP.MCBFLAG) {
   		    /* if there are no kids, then skip */
        	if (nf->nfnkids == 0) {
                DEBUG1("%s cben: skipping nuclear family with no `children'\n");
                continue;
            }
            /* Compute net income and employment income */
            nfinet = MP.PYINC * nneg( nfeld->im.imicnet - nfeld->im.imuccb);
			nfiemp = MP.PYINC * ( nfeld->id.idiemp + nneg(nfeld->id.idisenf) + nneg(nfeld->id.idisefm));

            if (nf->nfspoflg) {
        	    nfinet += MP.PYINC * nneg(nfspo->im.imicnet - nfspo->im.imuccb);
				nfiemp += MP.PYINC * (nfspo->id.idiemp + nneg(nfspo->id.idisenf) + nneg(nfspo->id.idisefm));
            }

    	    /* Scale income with user specified rates */
            nfinet = nfinet * MP.CTCINC;
			nfiemp = nfiemp * MP.CTCINC;

			if (nf->nfnkids > 0 && nfiemp >= MP.MCBMINE) {

				/* in case there is no working income component, set up the code */
				/* to work for those with 0 earnings */
				if (MP.MCBMINE > 0) {
					mcb = minn(MP.MCBAMT * nf->nfnkids, (MP.MCBEIR * nf->nfnkids) * (nfiemp - MP.MCBMINE));
				}
				else { /* no working income threshold used */
					mcb = MP.MCBAMT * nf->nfnkids;
				}
				
				mcbrr = mcb / (MP.MCBPO - MP.MCBTD);
				mcb = nneg(mcb - nneg((nfinet-MP.MCBTD)*mcbrr));
			}
            
            /* assign child benefits to mother if present */

            if (nf->nfspoflg && (nfspo->id.idsex == FEMALE)) {
                DEBUG1("%s mcb: spouse is the mother, gets MCB\n");
                nfspo->im.immcb  = mcb ;

            }
            else {
                 DEBUG1("%s mcb: elder receives MCB\n");
                 nfeld->im.immcb  = mcb ;
            }
		}
    }

    DEBUG_OFF("Atxman");
}
