/***
*Atxqitax.cpp - Compute taxable income and individual credits (Quebec)
*
*$Id: Atxqitax.cpp 6.121 2007/09/18 17:22:12 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/02/11 | Que. pension credit now based on imqipnst, not idipens  |
* | LP  | 07/01/25 | Apply QEEXPDED to idclergy - treat the same as imqalexp |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/17 | Moved imqempdd from imqdedfn to imqdedft/tax form verify|
* | LP  | 07/01/16 | Deleted imuccb from net inc test, imuccb add to imqitax |
* | LP  | 07/01/12 | Removed parameter QHSCDIR - no longer needed, fixed bug |
* | LP  | 06/06/01 | Removed imuccb from income tested credits               |
* | CH  | 06/01/04 | Added imqqpsedd                                         |
* | LP  | 05/05/30 | Added QRNCGFLG to calculation of disablity tax credit   |
* | LP  | 05/05/11 | Added new deduction for workers (QEMPDEDP and QEMPDEDM) |
* | LP  | 04/02/13 | Updated income concepts and deductions where necessary  |
* | LP  | 02/04/30 | Added QNORTHDED flag to deduction from net income calc. |
* | LP  | 01/03/12 | Removed pre-1988 programs (QREFOPT, etc.)               |
* | LP  | 00/11/16 | Added QSPBFLAG, QSADED, QNFSDED (social benefits flags) |
* | LP  | 00/05/03 | Apply QEEXPDED to employment expenses                   |
* | CH  | 00/04/14 | Apply QCAPGIR to capgex stkded and iloss                |
* | LP  | 00/04/11 | Added improved tax system for self-employed workers     |
* | LP  | 00/04/03 | Added QGIFTL1 and QGFTCR second credit rate for gifts   |
* | CH  | 00/03/30 | Renamed imqchartc to imqchatc to be under 8 characters  |
* | LP  | 00/03/22 | Added QMEDINC for income concept for medical expenses   |
* | CH  | 99/05/21 | Moved Medical credit from txqhstr, added simplified tax |
* |     |          |   form items, and fixed combined age credit item        |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | EGA | 98/05/26 | error in imqritc                                        |
* | EGA | 98/04/24 | imcapgex instead of idcapgex                            |
* | EGA | 98/04/23 | imrepay problem fixed                                   |
* | EGA | 98/04/22 | Added Que max to gifts to crowns                        |
* | EGA | 98/03/09 | add QREPNETFG and QPARTLOFLAG                           |
* | EGA | 98/02/12 | add QMEDOPT                                             |
* | EGA | 98/02/12 | add QYPPI, QYPTD, QYPRR                                 |
* | EGA | 98/02/11 | add QAXRR, QAXTD, QAXPI                                 |
* | EGA | 97/09/10 | Add idemplo,idstkded,idnorth,idpartlo,idcapgex          |
* | BBM | 92/09/29 | Add health services fund tax and tax credit             |
* | BBM | 92/07/27 | Base medical contributions on total expenses            |
* | BBM | 92/07/27 | replace imqmetc with imqmeda                            |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxqitax [global]  Compute taxable income (Quebec)
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxqitax[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxqitax - Compute taxable income (Quebec)
*
*Synopsis:
*    void Atxqitax(        
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   isnzero  Is argument non-zero ?
*   nneg     Change negative numbers to zero
*   minn     Find the minimum of two numbers
*   lkup1    Table look-up with interpolation/extrapolation
*
*Structure Members:
*   MP.QAXM         (i)  Quebec age exemption/amount
*   MP.QAXPI        (i)  Quebec age exemption reduction phase in
*   MP.QAXRR        (i)  Quebec age exemption reduction rate
*   MP.QAXTD        (i)  Quebec age exemption income turn down
*   MP.QBXM         (i)  Quebec basic personal exemption/amount
*   MP.QCALRFLG     (i)  Quebec combined age, living alone retirement credit
*   MP.QCAPGIR      (i)  Quebec capital gains inclusion rate
*   MP.QCHATNF      (i)  Quebec charitable donations maximum % net income
*   MP.QEEXPDED     (i)  Quebec employment expenses deduction application flag
*   MP.QGFTCR       (i)  Quebec credit rate for gifts over upper limit
*   MP.QGIFTFLG     (i)  Quebec max to gifts to Federal/Province/Crown activation flag
*   MP.QGIFTHCR     (i)  Quebec higher tax credit on large gifts activation flag
*   MP.QGIFTL1      (i)  Quebec minimum gift limit subject to higher credit rate
*   MP.QGIFTOPT     (i)  Quebec gifts as tax credit
*   MP.QHSC         (i)  Quebec Health Services Fund Contribution table
*   MP.QHSCFLAG     (i)  Quebec Health Services Fund Contribution calculation flag
*   MP.QHSCrows     (i)  number of rows in QHSC
*   MP.QHSOASFG     (i)  Quebec Health Services Fund Contribution OAS Deduction flag
*   MP.QHSSFALM     (i)  Quebec Health Services Fund Contrib. Alimony Paid Deduction flag
*   MP.QHSSFCAR     (i)  Quebec Health Services Fund Contrib. Carrying Charges Deduction flag
*   MP.QHSSFLSS     (i)  Quebec Health Services Fund Contrib. Allowable Losses Deduction flag
*   MP.QHSSFUIR     (i)  Quebec Health Services Fund Contrib. UI/EI Repayment Deduction flag
*   MP.QITSFSEW     (i)  Quebec improved tax system for self-employed worker activation flag
*   MP.QMAXDX       (i)  Quebec maximum disability deduction/amount
*   MP.QMEDALL      (i)  Quebec medical allowance maximum lower limit
*   MP.QMEDANF      (i)  Quebec medical allowance lower limit net income fraction
*   MP.QMEDINC      (i)  Income definition for Que. refundable tax credit for medical expenses
*   MP.QMEDOPT      (i)  Quebec Medical Tax Credit option
*   MP.QNFSDED      (i)  Quebec deduction of net federal supplements (SPA and GIS) from net income flag
*   MP.QNORTHDED    (i)  Quebec northern deductions - 0=for taxable income, 1=for net income
*   MP.QNTCR        (i)  Quebec nominal tax credit rate
*   MP.QREPFAMI     (i)  Quebec repayment of UI/OAS benefits deduct from income flag
*   MP.QRNCGFLG     (i)  Quebec Natural Caregivers Tax Credit activation flag
*   MP.QSADED       (i)  Quebec deduction of social assistance benefits from net income flag
*   MP.QSPBFLAG     (i)  Quebec inclusion of social program benefits in income flag
*   MP.QYPDL        (i)  Quebec deduction limit for pension income
*   MP.QYPPI        (i)  Quebec deduction phase-in for pension income
*   MP.QYPRR        (i)  Quebec deduction reduction rate for pension income
*   MP.QYPTD        (i)  Quebec deduction turn down for pension income
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.idcapgex     (i)  Capital gains exemptions (254)
*   id.idcarry      (i)  Carrying charges (221)
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcharit     (i)  Charitable donations
*   id.idclergy     (i)  Clergy residential deduction
*   id.idcloss      (i)  Allowable other years capital loss (253)
*   id.iddalimo     (i)  Alimony paid (220)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.idemplo      (i)  Employee home relocation loan dedn (248)
*   id.idgifts      (i)  Gifts to Canada/provinces/culture (342)
*   id.idialimo     (i)  Alimony income received
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idnclos      (i)  Allowable other years non-capital loss (252)
*   id.idnorth      (i)  Northern deductions (255)
*   id.idpartlo     (i)  Limited partnership losses (251)
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idspoflg     (i)  Person has spouse
*   id.idstkded     (i)  Stock option deduction (249)
*   im.imigis       (i)  GIS benefits
*   im.imioas       (i)  OAS benefits
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imispa       (i)  Spouse's allowance
*   im.imoasr       (i)  OAS recovery
*   im.imqalexp     (i)  Quebec allowable employment expenses
*   im.imqcapgt     (i)  Quebec taxable capital gains
*   im.imqcppse     (i)  Quebec self-employ. QPP/CPP Cont. deduction
*   im.imqidivt     (i)  Quebec taxable dividends
*   im.imqiloss     (i)  Quebec Modelled Business investment losses (234)
*   im.imqinet      (i)  Quebec net income
*   im.imqipnst     (i)  Quebec taxable pension income (after splitting)
*   im.imqitot      (i)  Quebec total income
*   im.imqqpsedd    (i)  Quebec parental insurance plan premium deduction for self-employed (Quebec)
*   im.imqsfflg     (i)  Quebec simplified tax form flag 0=general 1=simplified
*   im.imrepay      (i)  Social Benefits Repayments
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuibr       (i)  UI benefit recovery
*   im.imqcapgx     (io) Quebec modelled capital gains deduction (254)
*   im.imqchara     (io) Quebec allowable charitable donations(calc)
*   im.imqdedfn     (io) Quebec all deductions from net income
*   im.imqhsfc      (io) Quebec Health Services Fund Contributions
*   im.imqstkdd     (io) Quebec modelled Stock option deduction (249)
*   im.imqatc       (o)  Quebec age tax credit
*   im.imqbtc       (o)  Quebec basic tax credit
*   im.imqchatc     (o)  Quebec allowable charitable donations tax credit
*   im.imqdistc     (o)  Quebec disability tax credit
*   im.imqhsftc     (o)  Quebec Health Services Fund Cont.tax credit
*   im.imqitax      (o)  Quebec taxable income
*   im.imqmeda      (o)  Quebec medical expenses allowed
*   im.imqritc      (o)  Quebec retirement income tax credit
*
*Called by:
*   txque    Compute provincial taxes for Quebec
*
*****************************************************************************/
void Atxqitax(
	P_hh hh
)
{
	register P_in in;
	register int ini;
	register P_in inspo;  /* pointer to spouse */

	DEBUG_ON("Atxqitax");

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
	    NUMBER agered = ZERO;
	    NUMBER ernred = ZERO;
	    NUMBER adjamt = ZERO;

        NUMBER hscinc = ZERO; /* Health Services Contribution Income */
		NUMBER famnet = ZERO; /* family net income */
		NUMBER medgro = ZERO; /* gross medical expenses */
		NUMBER totgifts = ZERO;  /* sum of allowable charitable donations and other gifts */

		DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
		         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

		/* initialize spouse pointer */
	    inspo = in->id.idinspo;

		/* compute allowable medical expenses using the value on the database */
		if (MP.QMEDOPT == 1) {
		        if (isnzero(in->id.idmedgro)) {
		        	in->im.imqmeda = nneg(in->id.idmedgro -	
		        			minn(MP.QMEDALL, MP.QMEDANF * in->im.imqinet));
		        	DEBUG3("%s idmedgro= %.2f imqmeda =%.2f\n",
		        		in->id.idmedgro, in->im.imqmeda);
		        }
		}

	   /****
	   * The medical exemption tax credit depends on family net income
	   * and is assigned to the spouse with the largest net income
	   ****/
		else if (MP.QMEDOPT == 2) {
		    if (in->id.idspoflg) {
		        /* check if this spouse has higher income.  Otherwise skip */
		        if ((in->im.imqinet > inspo->im.imqinet)
		                 || ((in->im.imqinet == inspo->im.imqinet) && in->id.idcfrh==0)) {
	                if (MP.QMEDINC == 1) {
	                  famnet = in->im.imqinet + inspo->im.imqinet;
	               }
	               else if (MP.QMEDINC == 2) {	/* changes to the income concept in 1998 */
						famnet = nneg(in->im.imqitot  + inspo->im.imqitot
						         - in->id.idrpp - inspo->id.idrpp
						         - in->id.idrrsp - inspo->id.idrrsp
 								 - (MP.QREPFAMI?in->im.imrepay:ZERO)  /* Repayment of OAS/UI */
								 - (MP.QREPFAMI?inspo->im.imrepay:ZERO)  /* Repayment of OAS/UI */
						         - (MP.QITSFSEW?in->im.imqcppse:ZERO)
						         - (MP.QITSFSEW?inspo->im.imqcppse:ZERO)
                                 - in->im.imqqpsedd
                                 - inspo->im.imqqpsedd
						         );
					}											    
	               medgro = in->id.idmedgro + inspo->id.idmedgro;
	               if (isnzero(famnet)) {
               	      in->im.imqmeda = nneg(medgro -	 MP.QMEDANF * famnet);
				   }
				}
			}
			/* no spouse present */
			else {
			   if (MP.QMEDINC == 1) {	
                    famnet = in->im.imqinet;
			   }
			   else if (MP.QMEDINC == 2) {	   /* changes to the income concept in 1998 */
			     	famnet = nneg(in->im.imqitot
			     	            - in->id.idrpp
                                - in->id.idrrsp
								- (MP.QREPFAMI?in->im.imrepay:ZERO)  /* Repayment of OAS/UI */
 						        - (MP.QITSFSEW?in->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
                                - in->im.imqqpsedd
                                );

			   } 
               medgro = in->id.idmedgro ;
               if (isnzero(famnet)) {
               	      in->im.imqmeda = nneg(medgro - MP.QMEDANF * famnet);
			   }
			}
		}

      	/* compute allowable charitable expenses using the database value */
		/* Charitable donations are subject to a ceiling of QCHATNF % of net income */
		/* Note gifs will be added to imqchara later in the function */
        /* When QGIFTOPT == 2, charity and gifts are tax credits */
		if (MP.QGIFTOPT == 1) {
			if (isnzero(in->id.idcharit)) {
				in->im.imqchara = minn(in->id.idcharit, MP.QCHATNF *  in->im.imqinet);
			}
			DEBUG4("%s idcharit= %.2f imqinet= %.2f imqchara =%.2f\n",
				in->id.idcharit, in->im.imqinet, in->im.imqchara);
        } 
		if (MP.QGIFTOPT == 2) {
			if (isnzero(in->id.idcharit)) {
				in->im.imqchatc = (minn(in->id.idcharit, MP.QCHATNF * in->im.imqinet ))
									*MP.QNTCR;
			}
		}


		/**
		* Calculate Contribution to the Health Services Fund
		**/
		if (MP.QHSCFLAG) {
		    /**
			* SPSM's HSF Income does not consider profit sharing plans, 
			* recovery of QSSP deductions, or RRSP/Homebuyers provisions due to lack of data
			* Deductions do not consider transfers to an RRSP/RPP, overpayments, RHOSPs or
			* employment provisions
			**/

			/* The amount of dividend income to remove from total income for HSF
			   has always been the difference between the actual and taxable amounts.
			   With the 2006 split in dividend types (eligible/non-eligible) with
			   different gross up rates, the tax form now just calculates the difference.
			   Removed parameter QHSCDIR as it is no longer needed. */

        	hscinc = nneg(in->im.imqitot - (in->id.idiemp   + 
											in->id.idialimo +
        									(in->im.imqidivt - in->id.ididiv) +
        									in->im.imioas * MP.QHSOASFG ) 
							    		 - (in->im.imoasr * !MP.QHSOASFG)) ;
			 /*  Eligible income items included in the calculation of eligible income
			 *   for Health Services Fund Contribution differ between general and simplified
			 *   tax form used.  */

			 if (in->im.imqsfflg == 0 ) {  /* General Form */
				hscinc -= (in->im.imqiloss +
						   in->im.imuibr   +
						   in->id.iddalimo +
						   in->id.idcarry);
			 }
			 else {  /* Simplified form */
				hscinc -= (in->im.imqiloss * MP.QHSSFLSS +
						   in->im.imuibr   * MP.QHSSFUIR +
						   in->id.iddalimo * MP.QHSSFALM +
						   in->id.idcarry * MP.QHSSFCAR );
			 }			   
			/* In 1998 and beyond, employment expenses were no longer included in
			   total deductions but were used to reduce employment income in the 
			   calculation of total income.  If QEEXPDED is set to 0, then the
			   employment expenses are not applied as a deduction and must be reduced
			   from the income to be used for the Health Services Fund contribution. 
			   Note that the clergy residence deduction is treated the same as the
			   employment expense deduction (same line on tax form) */

			if (!MP.QEEXPDED) {
			   hscinc -= (in->im.imqalexp + in->id.idclergy);
			}
			  
			/* improved tax system for self-employed workers */
			/* when activated, the income used to determine the contribution to the
			   Health Services Fund by individuals is reduced by a % (QITSSEF) of the tax 
			   credit granted in respect of amounts payable as contributions on 
			   self-employed earnings to the CPP/QPP. */
		
			if (MP.QITSFSEW == 1) {
			   hscinc -= in->im.imqcppse;
			}

            /* QPIP deduction for self-employed workers */
            hscinc -= in->im.imqqpsedd;

			/* Beginning in 1994, social program benefits (SA, GIS, SPA, WCB) are to be
			**  included in the calculation of total income and then deducted from 
			**  net income as appropriate.
			**  These benefits are always deducted from the income used to calculate 
			**  the Health Services Fund Contribution.    */

			if (MP.QSPBFLAG == 1) {
				hscinc -= ( in->im.imisa + in->im.imispa + in->im.imigis + in->id.idiworkc) ;
			}
			
			if (hscinc < 0) {
				hscinc = 0;
			}

			in->im.imqhsfc = nneg((NUMBER) lkup1(MP.QHSC, MP.QHSCrows, hscinc));


		}

		/* calculate basic personal credit */
		in->im.imqbtc = MP.QNTCR * MP.QBXM;

		/* calculate age credit */
		/* If combined age, retirement, living alone credit is in place, 
		   then this credit is not applicable and imqcalr is used instead */
		if (in->id.idage >= 65 && !MP.QCALRFLG) {
			adjamt = ZERO;
			agered = minn(MP.QAXM, MP.QAXRR * nneg(in->im.imqinet - MP.QAXTD));
			adjamt = MP.QAXM - MP.QAXPI*agered;
			in->im.imqatc = MP.QNTCR * adjamt;

   		DEBUG2("%s qatxc   =%.2f\n", in->im.imqatc);
		}

		/* calculate disability credit */
		/* set disability tax credit to maximum CK */
		if (isnzero(in->id.iddisslf) || (isnzero(in->id.iddisoth) && !MP.QRNCGFLG)) {
			in->im.imqdistc = MP.QNTCR * MP.QMAXDX;
			DEBUG2("%s qdistc =%.2f\n", in->im.imqdistc);
		}

		/* calculate retirement income credit */
		/* If combined age, retirement, living alone credit is in place, 
		   then this credit is not applicable and imqcalr is used instead */
		if (isnzero(in->im.imqipnst) && !MP.QCALRFLG) {
			adjamt = ZERO ;
			ernred = minn(MP.QYPPI, MP.QYPRR * nneg(in->im.imqinet - MP.QYPTD));
			adjamt = nneg(MP.QYPDL - ernred);

			in->im.imqritc = MP.QNTCR * minn(adjamt,nneg(in->im.imqipnst-MP.QYPDL));
			DEBUG2("%s qritc =%.2f\n", in->im.imqritc);
		}
		
		/* calculate health services fund contribution tax credit */
		if (isnzero(in->im.imqhsfc)) {
			in->im.imqhsftc = MP.QNTCR * in->im.imqhsfc;
			DEBUG2("%s imqhsftc =%.2f\n", in->im.imqhsftc);
		}

     /* Add gifts to charitable donations allowed */
	 /* When QGIFTOPT == 1, it is a deduction  */
	 /* When QGIFTOPT == 2, it is a tax credit *
	  * When QGIFTFLG == 1, there is a maximum on gifts similar
	  * to charity */
		if (MP.QGIFTOPT == 1) {
			if (isnzero(in->id.idgifts)) {
				in->im.imqchara += in->id.idgifts;
				DEBUG4("%s idcharit= %.2f idgifts= %.2f imqchara =%.2f\n",
				   in->id.idcharit, in->id.idgifts, in->im.imqchara);
			}
		}
		if (MP.QGIFTOPT == 2) {
			if (isnzero(in->id.idgifts)) {
				if (MP.QGIFTFLG == 0 ) {
					in->im.imqchatc += in->id.idgifts*MP.QNTCR;
                }
				else {
					/* apply the income limit to allowable gifts  */
					in->im.imqchatc += (minn(in->id.idgifts, MP.QCHATNF *  in->im.imqinet))*MP.QNTCR;
			    }
			}
		    if (isnzero(in->id.idgifts) || isnzero(in->id.idcharit)) {
			  	if (MP.QGIFTHCR == 1) {
				   if (MP.QGIFTFLG == 0) {
						totgifts = in->id.idcharit + in->id.idgifts;
				   }
				   else {
						/* apply the income limit to allowable gifts and donations */	
				   		totgifts = minn(in->id.idcharit, MP.QCHATNF * in->im.imqinet );
				   		totgifts += minn(in->id.idgifts, MP.QCHATNF * in->im.imqinet );
    		       }
    		    	/* calculate the amount of tax credit for the various levels of gifts */
			  	   in->im.imqchatc = minn(totgifts,MP.QGIFTL1) * MP.QNTCR;
			  	   in->im.imqchatc += nneg(totgifts - MP.QGIFTL1) * MP.QGFTCR;
			   }
	    	}
		}

     /* calculate stock option deduction */
     in->im.imqstkdd = in->id.idstkded * (ONE - MP.QCAPGIR) ;

     /* calculate deductions from net income   */

		in->im.imqdedfn = 
                	  in->id.idcloss   /* allowable other yrs capital loss     */
                	+ in->id.idnclos   /* allowable prior yrs non-capital loss */
					+ in->im.imqchara  /* charitable contributions & gifts */
                    + in->id.idemplo   /*home relocation loan deduction include in other ded*/
					+ in->im.imqstkdd /*stock option deductio as part of other deduction */
					+ (MP.QNORTHDED?ZERO:in->id.idnorth)  /*Northern ded used as designated remote area */
                    + in->id.idpartlo   /* partner loss */
				    + (MP.QSADED?in->im.imisa:ZERO)   /* deduction of social assistance */
				    + (MP.QNFSDED?in->im.imigis:ZERO)   /* deduction of Net Federal Supplements (GIS) */
				    + (MP.QNFSDED?in->im.imispa:ZERO)   /* deduction of Net Federal Supplements (SPA) */
				    + (MP.QNFSDED?in->id.idiworkc:ZERO)  /* deduction of Workers Compensation (WCB)  */	   
				    ;

		/* add in the lifetime capital gains exemption */
    	if (isnzero(in->im.imqcapgt)){
	    	in->im.imqcapgx = in->id.idcapgex * MP.QCAPGIR;
	   	    in->im.imqdedfn += in->im.imqcapgx;
		}

		DEBUG2("%s qdedfn  =%.2f\n", in->im.imqdedfn);

		/* calculate Quebec taxable income */
		/* if general tax form */
		/* new in 2006, the universal child care benefit is added to taxable income
		   after net income is calculated */
		if (in->im.imqsfflg==0) {
			in->im.imqitax = nneg(in->im.imqinet + in->im.imuccb - in->im.imqdedfn);
	  			DEBUG2("%s qitax   =%.2f\n", in->im.imqitax);
		}
		/* used simplified form */
		else {
			in->im.imqitax = nneg(in->im.imqinet 	   /* net income under simplied system */
									+ in->im.imuccb
							   		- (MP.QSADED?in->im.imisa:ZERO)   /* deduction of social assistance */
					   		   		- (MP.QNFSDED?in->im.imigis:ZERO)   /* deduction of Net Federal Supplements (GIS) */
					   		   		- (MP.QNFSDED?in->im.imispa:ZERO)   /* deduction of Net Federal Supplements (SPA) */
									- (MP.QNFSDED?in->id.idiworkc:ZERO))  /* deduction of Workers Compensation Benefits */
									; 
   			DEBUG2("%s qitax   =%.2f\n", in->im.imqitax);
		}

	}
	DEBUG_OFF("Atxqitax");
}
