/***
*apdu.cpp - Database adjustment parameter definitions (user's)
*
*$Id: apdu.cpp 6.87 2007/09/18 17:23:49 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 95/10/18 | Add pragma to suppress near/far warnings in MSDOS 16-bit|
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Usage:
*   This is a place holder module.  It is intended to be used to add new
*   database adjustment parameters to the model.  Two files are used when
*   adding a new database adjustment parameter to the model.
*
*   apu.h    - is a header file which defines the new parameters.  It
*                should be modified first when adding a new parameter.
*   apdu.c   - is a C-language program which enables the model to understand
*                how to read and modify the new parameters.  Each new
*                parameter in apu.h must have a corresponding entry
*                in apdu.c
*
*   This file contains example definitions of common Scalar parameter
*   definitions.  If the parameter to be added is one of these types,
*   the corresponding example can be copied and edited to add the new
*   parameter.
*
*Definitions:
*   apdu     [global]  Database adjustment parameter definitions (user's)
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "pm.h"     /* Parameter manipulation package declarations          */
#include "apu.h"    /* Database adjustment parameter (user's) declarations  */
#include "ap.h"     /* Database adjustment parameter declarations           */


/* ==================== STATIC VARIABLE DEFINITIONS ======================= */

static char NEAR F_MONEY[]  = "8.2";
static char NEAR F_FRACT[]  = "12.5";


/* ==================== COMPILER PRAGMAS ================================== */

// eliminate spurious 'near/far mismatch' from MS-DOS compiler
#if defined(_MSDOS)
#pragma warning(disable : 4762)
#endif

#ifdef MSCOVL
#pragma alloc_text(INIT_TEXT,apdu)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */

/*****************************************************************************
*apdu - Database adjustment parameter definitions (user's)
*
*Synopsis:
*    void apdu(           
*    P_pc pap             
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Called by:
*   phase1   Perform initialisation tasks
*   pubmeta  Publish internal meta information in XML format
*
*****************************************************************************/
void  apdu(
	P_pc pap
)
{
	DEBUG_ON("apdu");

	pap;   /* reference to avoid compiler warnings */

	/**
	* Parameter definitions for database adjustment follow.  As mentioned
	* above, new parameters should first be added to the file 'apu.h'.  As
	* shown below, a parameter is defined to the SPSM parameter manipulation
	* facilities through a call to the 'pmaddent' function.  'pmaddent' takes
	* ten arguments, whose values are explained in the example below.  An
	* optional label for the parameter can be supplied through a call to the
	* 'stradd' function.  Please see the SPSM Programmer's Guide for more
	* information.
	*
	* Please see the files 'mp.h' and 'mpd[123].c' (in /spsm/model) for
	* examples of parameter declarations and definitions.  The easiest way to
	* add a parameter is to copy and modify the definition of an existing
	* parameter.
	*
	*
	* A brief description of the meaning of the arguments to 'pmaddent' and
	* 'stradd' follows:
	*
	* pmaddent(
	*    pap,                  <= parameter chain being extended (leave as is)
	*    "XXXXX",              <= name by which the parameter will be known
	*    (char *)&AP.UA.XXXXX, <= address of the parameter
	*    Format,               <= printing information for the parameter
	*    Agg_Type,             <= Aggregate type (scalar, vector, lookup, table)
	*    C_Type,               <= C-type (character, integer, number, string)
	*    Edit,                 <= Edits to be performed (eg. flag, fraction)
	*    Row_max,              <= Maximum number of rows, or option edit limit
	*    Rows_addr,            <= Address of int holding current number of rows
	*    Limit                 <= Number of columns
	*          );
	*
	*    stradd("XXXXX",       <= the name of the parameter, quoted
	*           "Label"        <= a printing label for the parameter
	*                  );
	*
	*
	* The following definitions can be used as templates if desired.
	*
	*  ----------------
	*  A flag parameter:
	*  ----------------

	pmaddent(pap, "XXXXX",    (char *)&AP.UA.XXXXX,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	stradd("XXXXX", "Parameter label");

	*  ----------------------------------------------
	*  An option parameter with allowed values 1 or 2:
	*  ----------------------------------------------

	pmaddent(pap, "XXXXX",    (char *)&AP.UA.XXXXX,    NULL,    P_SCL, C_INT,  E_OPT,  2, NULL, 0);
	stradd("XXXXX", "Parameter label");

	*  ---------------------------------------------------
	*  A numeric scalar parameter, such as a benefit level:
	*  ---------------------------------------------------

	pmaddent(pap, "XXXXX",    (char *)&AP.UA.XXXXX,    NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	stradd("XXXXX", "Parameter label");

	*  ------------------------------------------------------
	*  A numeric scalar parameter, a fraction between 0 and 1:
	*  ------------------------------------------------------

	pmaddent(pap, "XXXXX",    (char *)&AP.UA.XXXXX,    F_FRACT, P_SCL, C_NUM,  E_FRCT, 0, NULL, 0);

	*
	*
	* End of template examples.  Calls to 'pmaddent' should immediately follow
	* this comment block.
	**/

	DEBUG_OFF("apdu");
}



