/***
*ctmod.cpp - Compute commodity taxes for individuals and households
*
*$Id: ctmod.cpp 6.104 2007/09/18 17:22:08 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/07/23 | Added new commodity tax parameters                      |
* | CH  | 07/07/10 | Fixed problem with capping of tobacco                   |
* | CH  | 07/06/25 | Added GST on housing                                    |
* | CH  | 07/02/22 | Changed definition of imishri                           |
* | CH  | 07/02/19 | Added array variables and capping of tobacco            |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | CH  | 05/12/14 | Removed property taxes and ipac                         |
* | LP  | 05/12/01 | Removed CTFOEN & CTFMFG (parms and variables) - pre 91  |
* | LP  | 04/01/06 | Modified code for Version 10 (SHS PUMF variables)       |
* | EGA | 01/07/10 | Add net consumption SNA adjusted                        |
* | CH  | 99/09/02 | Modified code in order to work with PROVMAP             |
* | EGA | 98/06/22 | CTFAMSNA by region                                      |
* | EGA | 98/06/19 | add and remove changes                                  |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | BBM | 97/05/27 | remove fxhreno from fxnes calculation                   |
* | GJC | 94/01/17 | remove MP.CTPUPST,MP.CTPUGAS, ctpupst and ctpugas       |
* | SFG | 92/02/19 | implement CTFAMSNA parameter                            |
* | SFG | 92/02/06 | add ctnes, ctsave, ctothmon to complete expend. identity|
* | SFG | 92/02/06 | remove cttotexp; add ctnexp instead                     |
* | SFG | 92/02/06 | ctpct -> cttxpc_; ctfct -> cttxfc_; ctexp -> ctnexp     |
* | SFG | 92/02/06 | Delete CTPICT; add CTUPST, CTUGAS                       |
* | SFG | 90/10/04 | don't scale fx - use ctexp[] instead                    |
* | SFG | 90/09/18 | remove base year commodity taxes in fmfxread()          |
* | SFG | 89/11/25 | Ensure that res. dummy (#39) excluded from total exp.   |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   ctmod    [global]  Compute commodity taxes
*   ctfx     [static]  Compute commodity taxes for FAMEX spending unit
*   ctfxd    [static]  Compute commodity taxes for FAMEX spending unit (detailed)
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */
#include <string.h> /* declarations for string manipulation functions       */
#include <math.h>   /* definitions and declarations for math library        */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */
#include "apu.h"    /* Database adjustment parameter (user's) declarations  */
#include "ap.h"     /* Database adjustment parameter declarations           */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Sctmod[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== FUNCTION PROTOTYPES =============================== */

static void ctfx(P_ct ct, P_fx fx, int prov);
static void ctfxd(P_ct ct, P_fx fx, int prov);


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,ctmod,ctfx)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*ctmod - Compute commodity taxes
*
*Synopsis:
*    void ctmod(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   ctfx     Compute commodity taxes for FAMEX spending unit
*   nneg     Change negative numbers to zero
*   isnneg   Is argument non-negative ?
*   isnzero  Is argument non-zero ?
*
*Structure Members:
*   MP.CTCAPTOBFLG  (i)  Flag to cap tobacco expenditures
*   MP.CTFGSTHOUSE  (i)  Gst on housing to be allocated [prov][tenure]
*   MP.CTFLAG       (i)  Commodity tax activation flag
*   MP.CTOPT        (i)  Commodity tax calculation method [1=SHS,2=SPSM]
*   MP.CTTOBMAX     (i)  Maximum value of tobacco expenditures
*   ct.ctothmon     (i)  Household money from other sources
*   fx.fxgstidx     (i)  Index for distributing GST on housing
*   hd.hdprov       (i)  Province
*   hd.hdtenur      (i)  Tenure
*   hh.ct           (i)  Commodity tax variables [struct]
*   hh.fx           (i)  SHS data [struct]
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idhhrh       (i)  Relationship to head of household
*   id.iditrrsp     (i)  Taxable RRSP withdrawals
*   im.immdisp      (i)  Disposable income
*   ct.ctfcid       (io) Federal custom import duties
*   ct.ctfcid_      (io) Federal custom import duties [array]
*   ct.ctfexd       (io) Federal excise duties
*   ct.ctfexd_      (io) Federal excise duties [array]
*   ct.ctfext       (io) Federal excise taxes
*   ct.ctfext_      (io) Federal excise taxes [array]
*   ct.ctfgst       (io) Federal GST
*   ct.ctfgst_      (io) Federal GST [array]
*   ct.ctfnew       (io) New federal commodity tax
*   ct.ctfnew_      (io) New federal commodity tax [array]
*   ct.ctishrh      (io) Shared income concept (SHS)
*   ct.ctnes        (io) Household expenditure not elsewhere specified
*   ct.ctnexp       (io) Household expenditure net of taxes
*   ct.ctnexp_      (io) Household expenditure net of taxes [array]
*   ct.ctnsna_      (io) Household expenditure net of taxes [array] -SNA adjusted
*   ct.ctpamu       (io) Provincial amusement tax
*   ct.ctpgas       (io) Provincial gasoline tax
*   ct.ctpgas_      (io) Provincial gasoline tax [array]
*   ct.ctplgl       (io) Provincial liquor gallonage taxes
*   ct.ctpnew       (io) New provincial commodity tax
*   ct.ctpnew_      (io) New provincial commodity tax [array]
*   ct.ctpplq       (io) Provincial profits on liquor commissions
*   ct.ctprst       (io) Provincial retail sales tax
*   ct.ctprst_      (io) Provincial retail sales tax [array]
*   ct.ctptob       (io) Provincial tobacco tax
*   ct.ctsave       (io) Household savings
*   ct.cttxfc       (io) Federal commodity taxes (total)
*   ct.cttxfc_      (io) Federal commodity taxes [array]
*   ct.cttxpc       (io) Provincial commodity taxes (total)
*   ct.cttxpc_      (io) Provincial commodity taxes [array]
*   im.imishri      (io) Shared income concept (SHS & SPSD)
*   ct.ctfgsths     (o)  Federal GST on housing
*   ct.ctlprop      (o)  Local property taxes on owned dwellings
*   ct.ctnsna       (o)  Household expenditure net of taxes -SNA adjusted
*   im.imtxfc       (o)  Federal commodity taxes
*   im.imtxpc       (o)  Provincial commodity taxes
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void ctmod(
	P_hh hh
)
{
	register P_ct ct;
	register P_in in;
	P_fx fx;
	register int ini;
	NUMBER ishrh = ZERO;
	NUMBER nnishrh = ZERO;
	NUMBER fcrh = ZERO;
	NUMBER pcrh = ZERO;
	NUMBER adjfact = ZERO;
	int perverse;
	int comi;

	NUMBER ctfcid   = ZERO;
	NUMBER ctfexd   = ZERO; 
	NUMBER ctfext   = ZERO; 
	NUMBER ctfgst   = ZERO;
	NUMBER ctfnew   = ZERO;
	NUMBER ctplgl   = ZERO; 
	NUMBER ctpplq   = ZERO; 
	NUMBER ctpgas   = ZERO; 
	NUMBER ctpamu   = ZERO; 
	NUMBER ctptob   = ZERO; 
	NUMBER ctprst   = ZERO;
	NUMBER ctpnew   = ZERO;
	NUMBER total    = ZERO;
	NUMBER totalsna = ZERO;

    NUMBER tobadj = ZERO;
    NUMBER tobtot = ZERO;

	DEBUG_ON("ctmod");

	if (!MP.CTFLAG) {
		DEBUG2("%s commodity tax modelling inactivated by CTFLAG=%d\n", MP.CTFLAG);
		DEBUG_OFF("ct");
		return;
	}

	ct = &hh->ct;
	fx = &hh->fx;


	/* compute commodity taxes associated with the FAMEX vector */
	/* expenditure pattern and the province given by hh       */

	ctfx(ct, &hh->fx, hh->hd.hdprov);

    /* allocate GST on housing to household */
    /* Simply take the sum of GST on housing to be allocated (CTFGSTHOUSE) and multiply by the index */
    /* Since the index sums to 1,000,000,000 and the parameter is in thousands of dollars,
       need to divide by 1,000,000 to get to dollars */
   	ct->ctfgsths = fx->fxgstidx * MP.CTFGSTHOUSE[hh->hd.hdprov][hh->hd.hdtenur] / 1000000.0;

	/**
	* allocate federal & provincial commodity taxes among
	* the individuals in the household somehow.
	**/

	/**
	* calculate shared income concept (FAMEX-SPSD) for individuals
	* and total for household
	* This is disposable income plus omr + dissavings
	* omr plus dissavings come from FAMEX and are added to the
	* household head.
	**/

	/* process each person in household */

    /* create two income sums. */
	ishrh = ZERO; 
	nnishrh = ZERO;
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		/* add in FAMEX funny stuff to household head */

		if (in->id.idhhrh == 0) {
			in->im.imishri = ct->ctothmon;
		}

		DEBUG6("%s person[%d]: mdisp=%.2f dues=%.2f rpp=%.2f rrsp=%.2f\n",
			ini, in->im.immdisp, in->id.iddues, in->id.idrpp, in->id.idrrsp);

		/* add in disposable income  (allow negatives since programs like
		   pension income splitting can allow 1 person's income to be negative, but
		   increasing the household's disposable income as a whole) */
		in->im.imishri += in->im.immdisp;

		/* add in rrsp withdrawals which aren't included in disposable income */
		in->im.imishri += in->id.iditrrsp;

		/* cumulate the hhld shared income concept */
		ishrh += in->im.imishri;

        /* add a second concept for household income excluding the negatives for 
           splitting the income within the household */
        nnishrh += nneg(in->im.imishri);

		DEBUG4("%s person[%d]: ishr=%.2f hdishr=%.2f\n",
				 ini, in->im.imishri, ishrh);
	}
    /* If total income for the household is negative, then set to zero */
    if (ishrh<0) { 
        ishrh = ZERO;
    }


	/**
	* calculate the adjustment factor based upon the ratio of
	* shared income concepts
	**/

	if (ishrh > ZERO && ct->ctishrh > ZERO) {
		DEBUG1("%s compute SPSM/FAMEX adjustment factor\n");
		perverse = 0;
		adjfact = ishrh / ct->ctishrh;
        /* if tobacco exceeds the max, then cap tobacco */
        if (MP.CTCAPTOBFLG) {
            /* start by creating a tobacco expenditure for all */
            tobtot = ct->ctnexp_[IO_TOAC] + ct->cttxpc_[IO_TOAC] + ct->cttxfc_[IO_TOAC] ;

            /* if tobacco expenditures already bigger than max */
            if (tobtot > MP.CTTOBMAX ) {
                /* if adjusment would make things better, use */
                if (tobtot * adjfact < MP.CTTOBMAX) {
                    tobadj=adjfact;
                }
                else {
                    /* cap tobacco */
                    /* start by checking if income and expenditures greater than the max tobacco */
                    /* If so, set the tobacco adjustment to hit the max and then the other adjusment to compensate */
                    if ( (ishrh - MP.CTTOBMAX)>0.0 && (ct->ctishrh - MP.CTTOBMAX)>0.0 ) {
                        tobadj = MP.CTTOBMAX / tobtot;
                        adjfact = (ishrh - MP.CTTOBMAX) / (ct->ctishrh - tobtot);
                    }
                    /* in this case, either income or expenditures are smaller than tobacco.  Perverse case */
                    else {
		                perverse = 1;
		                adjfact = ONE;
                        tobadj = ONE;
                    }
                }
            }
            /* if tobacco expenditures times the adjusment 
               factor would be greater than the max, set to max 
               and reduce the adjustment factor
               of the other items */
            else if ((tobtot * adjfact) > MP.CTTOBMAX ) {
                if ( (ishrh - MP.CTTOBMAX)>0.0 && (ct->ctishrh - MP.CTTOBMAX)>0.0 ) {
                    tobadj = MP.CTTOBMAX / tobtot;
                    adjfact = (ishrh - MP.CTTOBMAX) / (ct->ctishrh - tobtot);
                }
                else {
                    /* if  total income and total expenditures bigger than tobacco, keep tobacco as is
                       and adjust the other expenditures */
                    if (ishrh>tobtot && ct->ctishrh>tobtot) {
                        /* keep tobacco as is */
                        tobadj = 1.0;
                        adjfact = (ishrh - tobtot) / (ct->ctishrh - tobtot);
                    }
                    /* either total income is smaller than tobacco, or total expenditures is equal */
                    /* perverse case.  Don't adjust */
                    else {
		                perverse = 1;
		                adjfact = ONE;
                        tobadj = ONE;
                    }
                }
            }
            /* otherwise set to be the same as others */
            else {
                tobadj = adjfact;
            }
        }
        else {
            /* set tobacco adjustment to be the same as other commodities */
            tobadj = adjfact;
        }
	}
	else {
		DEBUG1("%s set SPSM/FAMEX adjustment factor to 1\n");
		perverse = 1;
		/* should create an adjfact that takes us back to a FAMEX-based
			expenditure (rather than stuff that may be lying around from
			previous household).
		*/
		adjfact = ONE;
        tobadj = ONE;
	}
	DEBUG4("%s ishrh=%.2f ctishrh=%.2f adjfact=%.4f\n",
		     ishrh, ct->ctishrh, adjfact);


	/**
	* Scale ct stuff so that it will match imtxfc and imtxpc variables
	* for CTOPT==2.  This involves scaling the appropriate ct variables
	* by the ratio of SPSM to ct 'shared
	* income'.  
	**/

	if (MP.CTOPT == 2) {
	    DEBUG4("%s before adj: ctnexp=%.2f cttxfc=%.2f cttxpc=%.2f\n",
		        ct->ctnexp, ct->cttxfc, ct->cttxpc);

        /* Due to the possibility of different adjusment factors
           for tobacco as opposed to other commodities, need to 
           first adjust commodities separately, and then add them 
           back up */


        /* start by initializing temporary variables to zero */
        ctfcid   = ZERO;
        ctfexd   = ZERO; 
        ctfext   = ZERO; 
        ctfgst   = ZERO;
        ctfnew   = ZERO;

        ctpgas   = ZERO; 
        ctprst   = ZERO;
        ctpnew   = ZERO;

        total    = ZERO;
        totalsna = ZERO;



	    for (comi=0; comi<NUMCOM; comi++) {
            if (IO_TOAC == comi) {
			    ct->cttxfc_[comi] *= tobadj;
			    ct->cttxpc_[comi] *= tobadj;
			    ct->ctnexp_[comi] *= tobadj;
			    ct->ctnsna_[comi] *= tobadj;

			    ct->ctfcid_[comi] *= tobadj;
			    ct->ctfexd_[comi] *= tobadj;
			    ct->ctfext_[comi] *= tobadj;
			    ct->ctfgst_[comi] *= tobadj;
			    ct->ctfnew_[comi] *= tobadj;

			    ct->ctpgas_[comi] *= tobadj;
			    ct->ctprst_[comi] *= tobadj;
			    ct->ctpnew_[comi] *= tobadj;
            }
            else {
			    ct->cttxfc_[comi] *= adjfact;
			    ct->cttxpc_[comi] *= adjfact;
			    ct->ctnexp_[comi] *= adjfact;
			    ct->ctnsna_[comi] *= adjfact;

			    ct->ctfcid_[comi] *= adjfact;
			    ct->ctfexd_[comi] *= adjfact;
			    ct->ctfext_[comi] *= adjfact;
			    ct->ctfgst_[comi] *= adjfact;
			    ct->ctfnew_[comi] *= adjfact;

			    ct->ctpgas_[comi] *= adjfact;
			    ct->ctprst_[comi] *= adjfact;
			    ct->ctpnew_[comi] *= adjfact;
            }
            /* do the sums across commodities */
            ctfcid += ct->ctfcid_[comi];
            ctfexd += ct->ctfexd_[comi];
            ctfext += ct->ctfext_[comi];
            ctfgst += ct->ctfgst_[comi];
            ctfnew += ct->ctfnew_[comi];

            ctprst += ct->ctprst_[comi];
            ctpnew += ct->ctpnew_[comi];
            ctpgas += ct->ctpgas_[comi];

            total    += ct->ctnexp_[comi];
            totalsna += ct->ctnsna_[comi];
	    }

        /* assign the sums to the correct values */
        ct->ctfcid = ctfcid;
        ct->ctfexd = ctfexd;
        ct->ctfext = ctfext;
        ct->ctfgst = ctfgst;
        ct->ctfnew = ctfnew;

        ct->ctprst = ctprst;
        ct->ctpnew = ctpnew;
        ct->ctpgas = ctpgas;

        ct->ctnexp = total;
        ct->ctnsna = totalsna;

        /* adjust these taxes which only go to one commodity */
        ct->ctplgl *= adjfact; 
        ct->ctpplq *= adjfact; 
        ct->ctpamu *= adjfact; 
        ct->ctptob *= tobadj; 

        /* create total federal and provincial commodity taxes by summing up */
        ct->cttxfc = ct->ctfcid + ct->ctfexd + ct->ctfext + ct->ctfgst + ct->ctfnew;
        ct->cttxpc = ct->ctprst + ct->ctpnew+ ct->ctpgas + ct->ctplgl + ct->ctpplq + ct->ctpamu + ct->ctptob;

        /* these expenditures get adjusted with the default */
	    ct->ctlprop *= adjfact;
	    ct->ctnes   *= adjfact;
	    ct->ctsave  *= adjfact;

        /* and finally the total expenditures */
  	    ct->ctishrh = ct->ctnexp
	                + ct->cttxfc
	                + ct->cttxpc
	                + ct->ctnes
	                + ct->ctsave;
        DEBUG4("%s after adj: ctnexp=%.2f cttxfc=%.2f cttxpc=%.2f\n",
		        ct->ctnexp, ct->cttxfc, ct->cttxpc);
    }

	/**
	* Compute federal and provincial commodity taxes as a proportion
	* of total income as measured by ishrh (or ctishrh if CTOPT==2).
	* These ratios will be applied to each individual in the household
	* to allocate commodity taxes to individuals.
	**/

	if (!perverse && nnishrh>0.0) {
		fcrh = ct->cttxfc / nnishrh;
		pcrh = ct->cttxpc / nnishrh;
	}

	/* calculate individual federal and provincial commodity taxes */
	/* process each person in household */

	for (ini=0, in=&hh->in[0], in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		/* need to handle pathological case of zero ishrh */
		/* in such a case assign all commodity taxes to household head */
		if (perverse) {
			if (in->id.idhhrh == 0) {
				in->im.imtxfc = ct->cttxfc;
				in->im.imtxpc = ct->cttxpc;
			}
			else {
				in->im.imtxfc = ZERO;
				in->im.imtxpc = ZERO;
			}
		}
		else {
			if (isnzero(in->im.imishri) && isnneg(in->im.imishri)) {
				in->im.imtxfc = in->im.imishri * fcrh;
				in->im.imtxpc = in->im.imishri * pcrh;
			}
		}
		DEBUG4("%s person[%d]: txfc=%.2f, txpc=%.2f\n",
		        ini, in->im.imtxfc, in->im.imtxpc);
	}


	DEBUG_OFF("ctmod");
}



/*****************************************************************************
*ctfx - Compute commodity taxes for FAMEX spending unit
*
*Synopsis:
*    static void ctfx(    
*    P_ct ct,             
*    P_fx fx,             
*    int prov             
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*C-Runtime Calls:
*   memset   Uses a given character to initialize a specified number of bytes in the buffer
*
*Structure Members:
*   MP.CTFLAG       (i)  Commodity tax activation flag
*   MP.CTLPROP      (i)  Local property taxes
*   MP.CTNES        (i)  Expenditures NES
*   MP.CTSAVE       (i)  All positive savings
*   ct.ctnexp       (i)  Household expenditure net of taxes
*   ct.cttxfc       (i)  Federal commodity taxes (total)
*   ct.cttxpc       (i)  Provincial commodity taxes (total)
*   fx.fxalimo      (i)  Alimony and child support
*   fx.fxanncon     (i)  Annuity contracts and transfers to RRIFs
*   fx.fxfomr       (i)  Other money receipts
*   fx.fxgvpen      (i)  gvt pension plan contributions
*   fx.fxhomexp     (i)  Expenses assoc with principal residence
*   fx.fxnes        (i)  Not elsewhere stated
*   fx.fxnsave      (i)  Negative savings
*   fx.fxprtax      (i)  Property tax
*   fx.fxpsave      (i)  Positive savings
*   fx.fxpvpen      (i)  private pension plan contributions
*   fx.fxrepr       (i)  Repairs paid
*   fx.fxsaldur     (i)  Sale of durables
*   fx.fxseqhv      (i)  SHS record sequence number
*   fx.fxtptax      (i)  Transfer of Property taxes
*   ct.ctnes        (io) Household expenditure not elsewhere specified
*   ct.ctsave       (io) Household savings
*   ct.ctishrh      (o)  Shared income concept (SHS)
*   ct.ctlprop      (o)  Local property taxes on owned dwellings
*   ct.ctothmon     (o)  Household money from other sources
*   ct.ctprov       (o)  Province for COMTAX calculations
*   ct.ctseqhv      (o)  SHS record sequence number
*
*Calls:
*   ctfxd    Compute commodity taxes for FAMEX spending unit (detailed)
*
*Called by:
*   ctmod    Compute commodity taxes
*
*****************************************************************************/
static void ctfx(
	register P_ct ct,
	P_fx fx,
	int prov
)
{
	DEBUG_ON("ctfx");

	if (!MP.CTFLAG) {
		DEBUG2("%s commodity tax modelling inactivated by CTFLAG=%d\n", MP.CTFLAG);
		DEBUG_OFF("ctfx");
		return;
	}

	/* compute commodity taxes associated with the FAMEX vector  */
	/* if FAMEX record# differs from ct record# or if prov differs */

	DEBUG5("%s fxseqhv=%d ctseqhv=%d prov=%d ctprov=%d\n",
		fx->fxseqhv, ct->ctseqhv, prov, ct->ctprov);

 	/*  This code makes the model more efficient, but it runs into problems when
	    PROVMAP is turned on.  So it was disabled. 
 	if (fx->fxseqhv == ct->ctseqhv && prov == ct->ctprov) {
		DEBUG1("%s no need to recompute commodity tax\n");
		DEBUG_OFF("ctfx");
		return;
	}
	*/

	DEBUG1("%s compute commodity tax for FAMEX expenditure\n");

	/* initialise commodity tax modelled variable section */
	memset(ct, '\0', sizeof(*ct));

	/* Following line indicates which FAMEX record the ct applies to */
	/* This allows this computation to be skipped for contiguous         */
	/* households sharing the same FAMEX expenditure vector and province.*/

	ct->ctseqhv = fx->fxseqhv;
	ct->ctprov = prov;


	/** There used to be an option to only model selected
	** commodity tax variables.  This facility has been turned off
    ** and all levels of detail are now always modeled.
	*/

	ctfxd(ct, fx, prov);

	/**
	* Compute additional components of the income/expenditure identity.
	* Note the adjustment of the property taxes, exp nes, and savings 
	* by their respective model parameters. This allows users to vary
	* their assumptions regarding the share of income going to savings
	* versus the share going to spending. Other money receipts is
	* considered a windfall gain and is not adjusted.
	**/
	ct->ctnes    = MP.CTNES * (
						fx->fxnes + fx->fxalimo + fx->fxhomexp + fx->fxanncon
		          + fx->fxrepr);
	ct->ctsave   = MP.CTSAVE * (
						fx->fxgvpen + fx->fxpvpen + fx->fxpsave );
	ct->ctothmon = fx->fxnsave + fx->fxfomr + fx->fxsaldur;

    /* As ctlprop is included in fxio9, this is no longer part of the identity.  
       but the variable is retained */
	ct->ctlprop  = MP.CTLPROP * (fx->fxprtax + fx->fxtptax);

	/**
	* Compute the 'shared income concept' income of the
	* FAMEX expenditure pattern.  This is disposable income
	* plus omr plus dissavings.
	* Note that since we are going to make expenditures match income,
	* when CTOPT=2, there is no need to include the account balancing difference 
	**/

  	ct->ctishrh = ct->ctnexp
	            + ct->cttxfc
	            + ct->cttxpc
	            + ct->ctnes
	            + ct->ctsave;

  	DEBUG5("%s ctishrh=%.2f ctnexp=%.2f cttxfc=%.2f cttxpc=%.2f\n",
			  ct->ctishrh, ct->ctnexp, ct->cttxfc, ct->cttxpc);
  	DEBUG4("%s ctlprop=%.2f ctnes=%.2f ctsave=%.2f\n",
			  ct->ctlprop, ct->ctnes, ct->ctsave);

	DEBUG_OFF("ctfx");
}



/*****************************************************************************
*ctfxd - Compute commodity taxes for FAMEX spending unit (detailed)
*
*Synopsis:
*    static void ctfxd(   
*    P_ct ct,             
*    P_fx fx,             
*    int prov             
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Structure Members:
*   AP.CTFAMSNA     (i)  SHS->SNA conceptual conversion factor [com]
*   MP.CTFCID       (i)  Federal custom import duties [com]
*   MP.CTFEXD       (i)  Federal excise duties [com]
*   MP.CTFEXT       (i)  Federal excise taxes [com]
*   MP.CTFGST       (i)  Federal GST [com]
*   MP.CTFNEW       (i)  New federal commodity tax [com]
*   MP.CTPAMU       (i)  Provincial amusement tax [prov]
*   MP.CTPGAS       (i)  Provincial gasoline tax [com x prov]
*   MP.CTPLGL       (i)  Provincial liquor gallonage tax [prov]
*   MP.CTPNEW       (i)  New provincial commodity tax [com x prov]
*   MP.CTPPLQ       (i)  Provincial profits on liquor commissions [prov]
*   MP.CTPRST       (i)  Provincial retail sales tax [com x prov]
*   MP.CTPTOB       (i)  Provincial tobacco tax [prov]
*   fx.fxio         (i)  I/O expenditure categories [array]
*   ct.ctfcid       (io) Federal custom import duties
*   ct.ctfexd       (io) Federal excise duties
*   ct.ctfext       (io) Federal excise taxes
*   ct.ctfgst       (io) Federal GST
*   ct.ctfnew       (io) New federal commodity tax
*   ct.ctpamu       (io) Provincial amusement tax
*   ct.ctpgas       (io) Provincial gasoline tax
*   ct.ctplgl       (io) Provincial liquor gallonage taxes
*   ct.ctpnew       (io) New provincial commodity tax
*   ct.ctpplq       (io) Provincial profits on liquor commissions
*   ct.ctprst       (io) Provincial retail sales tax
*   ct.ctptob       (io) Provincial tobacco tax
*   ct.ctfcid_      (o)  Federal custom import duties [array]
*   ct.ctfexd_      (o)  Federal excise duties [array]
*   ct.ctfext_      (o)  Federal excise taxes [array]
*   ct.ctfgst_      (o)  Federal GST [array]
*   ct.ctfnew_      (o)  New federal commodity tax [array]
*   ct.ctnexp       (o)  Household expenditure net of taxes
*   ct.ctnexp_      (o)  Household expenditure net of taxes [array]
*   ct.ctnsna       (o)  Household expenditure net of taxes -SNA adjusted
*   ct.ctnsna_      (o)  Household expenditure net of taxes [array] -SNA adjusted
*   ct.ctpgas_      (o)  Provincial gasoline tax [array]
*   ct.ctpnew_      (o)  New provincial commodity tax [array]
*   ct.ctprst_      (o)  Provincial retail sales tax [array]
*   ct.cttxfc       (o)  Federal commodity taxes (total)
*   ct.cttxfc_      (o)  Federal commodity taxes [array]
*   ct.cttxpc       (o)  Provincial commodity taxes (total)
*   ct.cttxpc_      (o)  Provincial commodity taxes [array]
*
*Called by:
*   ctfx     Compute commodity taxes for FAMEX spending unit
*
*****************************************************************************/
static void ctfxd(
	register P_ct ct,
	P_fx fx,
	int prov
)
{
	NUMBER ctfcid   = ZERO;
	NUMBER ctfexd   = ZERO; 
	NUMBER ctfext   = ZERO; 
	NUMBER ctfgst   = ZERO;
	NUMBER ctfnew   = ZERO;
	NUMBER ctplgl   = ZERO; 
	NUMBER ctpplq   = ZERO; 
	NUMBER ctpgas   = ZERO; 
	NUMBER ctpamu   = ZERO; 
	NUMBER ctptob   = ZERO; 
	NUMBER ctprst   = ZERO;
	NUMBER ctpnew   = ZERO;
	NUMBER total    = ZERO;
	NUMBER totalsna = ZERO;
	NUMBER expend   = ZERO;
	int comi;

	DEBUG_ON("ctfxd");

	/* Process each commodity in expenditure vector */

	for (comi=0; comi<NUMCOM; comi++) {

		/* get net FAMEX expenditure on commodity */
		expend = fx->fxio[comi];
		if (expend == ZERO) {
			continue;
		}

		/* cumulate total net FAMEX expenditure */
		if (comi != IO_UNUSED) {
			total += expend;
		}

		/* record tax-exclusive expenditure by commodity */
		ct->ctnexp_[comi] = expend;

		/* perform conceptual adjustment to convert FAMEX-based */
		/* to SNA-based expenditure. */
		expend *= AP.CTFAMSNA[comi][prov];
		totalsna +=	 expend;
		ct->ctnsna_[comi] = expend;
		/* compute federal taxes by type */

		ctfcid = expend * MP.CTFCID[comi];
		ct->ctfcid   += ctfcid;
		ct->ctfcid_[comi] = ctfcid;
		ct->cttxfc_[comi] += ctfcid;

		ctfexd = expend * MP.CTFEXD[comi];
		ct->ctfexd   += ctfexd;
		ct->ctfexd_[comi] = ctfexd;
		ct->cttxfc_[comi] += ctfexd;

		ctfext = expend * MP.CTFEXT[comi];
		ct->ctfext   += ctfext;
		ct->ctfext_[comi] = ctfext;
		ct->cttxfc_[comi] += ctfext;

		ctfgst = expend * MP.CTFGST[comi];
		ct->ctfgst   += ctfgst;
		ct->ctfgst_[comi] = ctfgst;
		ct->cttxfc_[comi] += ctfgst;

		ctfnew = expend * MP.CTFNEW[comi];
		ct->ctfnew   += ctfnew;
		ct->ctfnew_[comi] = ctfnew;
		ct->cttxfc_[comi] += ctfnew;

		/* the following 4 provincial taxes are commodity specific */
		if (IO_ALCBEX == comi) {
			ctplgl = expend * MP.CTPLGL[prov];
			ct->ctplgl = ctplgl;
			ct->cttxpc_[IO_ALCBEX] += ctplgl;

			ctpplq = expend * MP.CTPPLQ[prov];
			ct->ctpplq = ctpplq;
    		ct->cttxpc_[IO_ALCBEX] += ctpplq;
		}
		else if (IO_TOAC == comi) {
			ctptob = expend * MP.CTPTOB[prov];
			ct->ctptob = ctptob;
			ct->cttxpc_[IO_TOAC] += ctptob;
		}
		else if (IO_RECSRV == comi) {
			ctpamu = expend * MP.CTPAMU[prov];
			ct->ctpamu = ctpamu;
			ct->cttxpc_[IO_RECSRV] += ctpamu;
		}

		/* Do provincial gasoline and retail sales tax. */

		ctpgas = expend * MP.CTPGAS[comi][prov];
		ct->ctpgas += ctpgas;
		ct->ctpgas_[comi] = ctpgas;
		ct->cttxpc_[comi] += ctpgas;

		ctprst = expend * MP.CTPRST[comi][prov];
		ct->ctprst += ctprst;
		ct->ctprst_[comi] = ctprst;
		ct->cttxpc_[comi] += ctprst;

		ctpnew = expend * MP.CTPNEW[comi][prov];
		ct->ctpnew += ctpnew;
		ct->ctpnew_[comi] = ctpnew;
		ct->cttxpc_[comi] += ctpnew;
	}

	/* compute derived household quantities */
	ct->cttxfc =
			+ ct->ctfcid
			+ ct->ctfexd
			+ ct->ctfext
			+ ct->ctfgst
			+ ct->ctfnew;

	ct->cttxpc = 
				+ ct->ctplgl
				+ ct->ctpplq
				+ ct->ctpgas
				+ ct->ctpamu
				+ ct->ctptob
				+ ct->ctprst
				+ ct->ctpnew;

	/* store the sum of net expenditures (not adjusted to SNA basis) */
	ct->ctnexp = total;
	/* store the sum of net expenditures adjusted to SNA basis */
	ct->ctnsna = totalsna;

	DEBUG4("%s ctnexp=%.2f cttxfc=%.2f cttxpc=%.2f\n",
		    ct->ctnexp, ct->cttxfc, ct->cttxpc);
	DEBUG_OFF("ctfxd");
}
