/***
*famod.cpp - Compute family allowance
*
*$Id: famod.cpp 6.98 2007/09/18 17:22:08 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/02/06 | Assign UCCB to older spouse here, reassign in txinet    |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 06/12/12 | Switched iminet to imicnet (income tax form concept)    |
* | CH  | 06/10/16 | Set non-receiving spouse's uccb to 0.0.                 |
* | LP  | 01/03/12 | Added Universal Child Care Benefit UCCBFLAG & UCCBAMT   |
* | LP  | 01/03/12 | Removed pre-1988 paramters (QFATFLAG)                   |
* | CH  | 00/04/06 | Modified newborn allowance                              |
* | SFG | 99/12/09 | Change fa->famod and sa->samod                          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | EGA | 98/02/18 | Quebec Family Allowance Tax Credit                      |
* | EGA | 97/10/10 | Random assign. of rank in QC newborn modified           |
* | EGA | 97/10/07 | Increase randomly the rank of children in QC newborn fa |
* | SFG | 96/02/07 | Move to ROOT segment to avoid busting 64K limit (MSDOS) |
* | BBM | 92/07/07 | Insert Flag to cancel Quebec/Alberta configurations     |
* | SFG | 90/09/27 | calculated imqtfa (Quebec taxable FA)                   |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   famod    [global]  Compute family allowance
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Sfamod[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(ROOT_TEXT,famod)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*famod - Compute family allowance
*
*Synopsis:
*    void famod(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   lkup2    Table look-up without interpolation
*   lkup1    Table look-up with interpolation/extrapolation
*
*Structure Members:
*   MP.AFAC1        (i)  Alberta FA benefit per child aged 0 - 6
*   MP.AFAC2        (i)  Alberta FA benefit per child aged 7 - 11
*   MP.AFAC3        (i)  Alberta FA benefit per child aged 12- 15
*   MP.AFAC4        (i)  Alberta FA benefit per child aged 16- 17
*   MP.FAFLAG       (i)  Family allowance flag
*   MP.PFACFLAG     (i)  Use provincial configuration flag
*   MP.QAAFA        (i)  Quebec availability allowance (supplement) [parity,$]
*   MP.QAAFAFLAG    (i)  Availability Allowance: Supplement to Quebec FA
*   MP.QAAFArows    (i)  number of rows in QAAFA
*   MP.QFAIFLAG     (i)  Quebec inclusion of federal family allowance in income flag
*   MP.QFATCOPT     (i)  Quebec Family Allowance Tax Credit Option
*   MP.QFFSL        (i)  Federal family allowance: Quebec Configuration[parity,$]
*   MP.QFFSLrows    (i)  number of rows in QFFSL
*   MP.QFPSL        (i)  Quebec provincial family allowance [parity,$]
*   MP.QFPSLrows    (i)  number of rows in QFPSL
*   MP.QFS          (i)  Federal supplement per child 12-17 in Quebec configuration
*   MP.QNBFA        (i)  Quebec newborn family allowance [parity,age]
*   MP.QNBFAFLAG    (i)  Quebec newborn family allowance flag
*   MP.QNBPOFLAG    (i)  Quebec newborn family allowance phase out flag
*   MP.QNBPOPYR     (i)  Quebec newborn family allowance phase-out year proportion
*   MP.QNBPOYR      (i)  Quebec newborn family allowance phase out year
*   MP.QNBRFLAG     (i)  Quebec newborn family random increase of child rank
*   MP.STDFA        (i)  Standard federal family allowance per child
*   MP.TARGETYEAR   (i)  Year of analysis
*   MP.UCCBAMT      (i)  Universal child care benefit amount for each child < 6
*   MP.UCCBFLAG     (i)  Universal child care benefit activation flag
*   hd.hdprov       (i)  Province
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idrand       (i)  Random numbers [array]
*   id.idsex        (i)  Sex
*   nf.nfinch       (i)  First child in nuclear family [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   nf.nfineld      (io) Eldest person in nuclear family [pointer]
*   nf.nfinspo      (io) Spouse of eldest [pointer]
*   im.imffa        (o)  Federal portion of family allowances
*   im.imnfach      (o)  Number of family allowance children claimed
*   im.impfa        (o)  Provincial family allowance
*   im.imqaafa      (o)  Quebec Availability Allowance FA Supplement
*   im.imqnbfa      (o)  Quebec newborn Allowance
*   im.imqtfa       (o)  Quebec taxable family allowances
*   im.imtfa        (o)  Taxable family allowances
*   im.imuccb       (o)  Universal Child Care Benefit amount
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void famod(
	P_hh hh
)
{
	register P_in in;  /* pointer to data for current person     */
	register P_nf nf;  /* pointer to nuclear family */

	register int nfi, ini;
	int prov;
	int age;
	int nch, nch12_17, nch16_17, nch0_5, kidrank, indexr;

	NUMBER tfa = ZERO;
	NUMBER qtfa = ZERO;
	NUMBER ffa = ZERO;
	NUMBER ffa1 = ZERO;
	NUMBER ffa2 = ZERO;
	NUMBER pfa = ZERO;
	NUMBER qaafa = ZERO;
	NUMBER qnbfa = ZERO;
	NUMBER kidben = ZERO;
    int    rrank = 0;
	NUMBER uccb = 0.0;


 	DEBUG_ON("famod");

	if (!MP.FAFLAG && !MP.UCCBFLAG) {   
		DEBUG1("%s Turned off by FAFLAG\n");
		DEBUG_OFF("famod");
		return;
	}

	if ( MP.UCCBFLAG ) {
		/* process each nuclear family in household */
		for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

			DEBUG2("%s processing nuclear family %d\n", nfi);

			/* skip to next family if no kids */
			if (nf->nfnkids == 0) {
				DEBUG1("%s skipping nuclear family with no kids\n");
				continue;
			} 
			nch0_5 = 0;
			/* process kids in nuclear family */
			for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++, ini++) {
				age = in->id.idage;
 				if (age <= 5) {
					nch0_5++;
				}
			}

			uccb = nch0_5 * MP.UCCBAMT;

			/* Give to the older spouse for now. Will reassign in txinet if necessary */
			nf->nfineld->im.imuccb = uccb;
			/* Zero out any remaining value for spouse */
            if (nf->nfspoflg) {
      			nf->nfinspo->im.imuccb = 0.0;
            }
		}
   	}


	if ( MP.FAFLAG ) {
		/* save province of residence */
		prov = hh->hd.hdprov;
		DEBUG2("%s prov=%d\n", prov);

		/* process each nuclear family in household */
		for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

			DEBUG2("%s processing nuclear family %d\n", nfi);

			/* skip to next family if no kids */
			if (nf->nfnkids == 0) {
				DEBUG1("%s skipping nuclear family with no kids\n");
				continue;
			}

			/* initialise working variables for each nuclear family */
			nch = nf->nfnkids;
			tfa = ZERO;
			qtfa = ZERO;
			ffa = ZERO;
			pfa = ZERO;
			nch0_5 = 0;
			nch12_17 = 0;
			nch16_17 = 0;
			qaafa = ZERO;
			qnbfa = ZERO;
			rrank = 0;
   
			if (prov == PRQUE && (MP.QFATCOPT == 1 || MP.QNBFAFLAG==1)) {
				DEBUG1("%s special Quebec calculation\n");

				/* process kids in nuclear family */
				for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++) {
            		/**	 NB: ini is pre-incremented !!!! **/
					ini++;

					DEBUG2("%s processing kid %d in nuclear family\n", ini);

					/**
					* Compute rank of child in family.
					* Note that we use the fact that kids
					* are ordered by increasing age in the family structure.
					**/
   
					kidrank=(nf->nfnkids+1)-ini;
					if (MP.QNBRFLAG) {
			       		if (in->id.idage ==2) {
				      		if ((kidrank == 2) &&(in->id.idrand[1] < (0.08) ))  rrank +=1;
				      		if ((kidrank == 1) &&(in->id.idrand[1] < (0.15) ))  rrank +=1;
			      		}
                   		if (in->id.idage ==1) {
				      		if ((in->id.idrand[1] < (0.13)) && (kidrank == 2)) rrank += 1;
				      		if ((in->id.idrand[1] < (0.16)) && (kidrank == 1)) rrank += 1;
				   		}
				   		kidrank=(nf->nfnkids+1)-ini+rrank;
					}

					/* compute needed counts by age group */

					age = in->id.idage;
					if (age <= 5) {
						nch0_5++;
					}
					if (age >= 12 && age <= 17) {
						nch12_17++;
					}
					if (age >= 16 && age <= 17) {
						nch16_17++;
					}

					/**
					* Calculate the Quebec allowance for newborn children 
					* Note that the program begins in 1988.  The QNBFA parameter
					* has values that refer to amounts being paid in the current
					* year.  These amounts may have been mandated in a previous year.
					**/

					if (MP.QNBFAFLAG) {

						/* Give benefits for each child of appropriate age */
						if (age < QNBFACMAX) {

							indexr=kidrank - 1;
							/* Stop at last row of table */
							if (indexr >= QNBFARMAX) {
								indexr = QNBFARMAX - 1;
							}
							kidben = MP.QNBFA[indexr][age];

							DEBUG4("%s age=%d kidrank=%d newborn allowance=%.2f\n",
								age, kidrank, kidben);
						}
						else {
							DEBUG2("%s Too old for newborn allowance, age=%d\n", age);
							kidben = ZERO;
						}

                    	/* if program is being phased out, randomly
                       	   set one quarter of the benefits to zero since the program was phased
                      	   out in October 1997.  So one quarter of people in 1997 are ineligible
                           for the program */
                    	if (MP.QNBPOFLAG == 1) {
                        	if ((in->id.idrand[1] > MP.QNBPOPYR) && (in->id.idage == MP.TARGETYEAR - MP.QNBPOYR)) {
                            	kidben = ZERO;
                        	}
                    	}

						/**
						* Cumulate child amount into total newborn allowance
						* for family
						**/

						qnbfa += kidben;
					}
		          
				    /* calculate the Quebec young child availability allowance */
				    if (MP.QAAFAFLAG && age <= 5) {
					    qaafa += (NUMBER) lkup2(MP.QAAFA, MP.QAAFArows, (NUMBER) kidrank);
					    DEBUG2("%s Quebec availability allowance=%.2f\n", qaafa);
				    }
				}

			    /* The total Quebec birth allowance */
			    DEBUG2("%s Quebec birth allowance=%.2f\n", qnbfa);
           
			    /* calc Quebec family allowance */
			    pfa = (NUMBER) lkup1(MP.QFPSL, MP.QFPSLrows, (NUMBER) nch);
			    DEBUG2("%s prov portion=%.2f\n", pfa);

			    if (MP.PFACFLAG) {   /* Use Quebec configuration */
			   	    /* calc federal family allowance */
			   		ffa1 = (NUMBER) lkup1(MP.QFFSL, MP.QFFSLrows, (NUMBER) nch);
			   	    DEBUG2("%s fed  portion=%.2f\n", ffa1);

			   	    /* calc federal FA supplement for children 12-17   */
			   	    ffa2 = nch12_17 * MP.QFS;
			   	    DEBUG2("%s federal supplement=%.2f\n", ffa2);
			    }
			    else {   /* Impose standard federal fa rules on Quebec  */
			   	    DEBUG1("%s standard FA calculation for Quebec\n");
			    	ffa1 = nch * MP.STDFA;    /* taxable family allowances         */
			    }

			    ffa = ffa1 + ffa2;   /* Total federal family allowance */
			    tfa = ffa;           /* federally taxable family benefits */

			    /* federal payments might be provincially taxable */
			    if (MP.QFAIFLAG) {
			   		qtfa = ffa;          /* provincially taxable FA */
			    }
			    else {
			   	    qtfa = ZERO;
			    }
            }

		    else if (prov == PRALTA) {

			    DEBUG1("%s special Alberta calculation\n");

				if (MP.PFACFLAG) {

					/* Process each kid in nuclear family.       */
					for (ini=0, in=nf->nfinch;  ini<nf->nfnkids;  in++) {
						ini++;
						age = in->id.idage;

						if (age <= 6)  ffa += MP.AFAC1;    /* aged  0-6  */
						else if (age <= 11) ffa += MP.AFAC2;    /* aged  7-11 */
						else if (age <= 15) ffa += MP.AFAC3;    /* aged 12-15 */
						else if (age <= 17) ffa += MP.AFAC4;    /* aged 16-17 */
					}

					tfa = ffa;
				}
				else {  /* Impose standard federal fa rules on Alberta  */
					DEBUG1("%s standard FA calculation for Alberta\n");
					tfa = nch * MP.STDFA;    /* taxable family allowances         */
					ffa = tfa;               /* federal part of family allowances */
				}
			}

			else {
				DEBUG1("%s standard FA calculation\n");
				tfa = nch * MP.STDFA;         /* taxable family allowances         */
				ffa = tfa;                 /* federal part of family allowances */
			}

			DEBUG3("%s tfa=%.2f, ffa=%.2f\n", tfa, ffa);

			/**
			* Associate the taxable amount of family allowances, and the number of
			* family allowance children, with the eldest in the nuclear family.
			* The function txinet will reassign to the spouse if necessary.
			**/

			nf->nfineld->im.imtfa = tfa;
			nf->nfineld->im.imqtfa = qtfa;
			nf->nfineld->im.imnfach = (NUMBER) nch;

			/* assign FA to mother if present */
			if (nf->nfspoflg && (nf->nfinspo->id.idsex == FEMALE)) {
				DEBUG1("%s spouse is the mother\n");
				in = nf->nfinspo;
			}
			else {
				DEBUG1("%s head receives FA\n");
				in = nf->nfineld;
			}

			in->im.imffa = ffa;
			in->im.impfa = pfa;
			in->im.imqaafa = qaafa; 	 /*  Quebec Availability Supplement */
    		in->im.imqnbfa = qnbfa;		 /*  Quebec Newborn Allowance        */


		}

	}
	
	DEBUG_OFF("famod");
}
