/***
*mpc.cpp - Calculate derived model parameters and do edits
*
*$Id: mpc.cpp 6.95 2007/09/18 17:22:09 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/02/22 | Added checks for pension income splitting               |
* | CH  | 07/02/19 | Removed CTFTOT and CTPTOT                               |
* | LP  | 05/12/01 | Removed CTFOEN & CTFMFG (parms and variables) - pre 91  |
* | LP  | 04/02/23 | Removed error check to SAMNTHPR vector (max 12 months)  |
* | LP  | 02/06/06 | Added error check to SAMNTHPR vector (max 12 months)    |
* | CH  | 01/06/12 | Removed ei stuff related to new database                |
* | LP  | 01/03/12 | Removed pre-1988 programs (PEROPT and QREFOPT)          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 98/02/18 | change printf to lprintf for log output                 |
* | SFG | 92/02/07 | ctupst -> ctpupst; ctugas -> ctpugas; also CTUPST,CTUGAS|
* | SFG | 92/02/06 | Delete CTPICT; add CTUPST, CTUGAS                       |
* | --- | 89/10/13 | Release 4.0                                             |
* | SFG | 90/03/01 | Compute FTOT & PTOT always, for glass-box convenience   |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   mpc      [global]  Calculate derived model parameters and do edits
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "cpu.h"    /* Control parameter (user's) declarations              */
#include "cp.h"     /* Control parameters declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Smpc[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(DIALOG_TEXT,mpc)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*mpc - Calculate derived model parameters and do edits
*
*Synopsis:
*    int mpc()            
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   iszero   Is argument zero ?
*   STRN     Retrieve a string constructed from a number
*   lprintf  print formatted message to console/log
*
*Structure Members:
*   CP.FXVFLAG      (i)  Read SHS expenditure vector file
*   MP.BGISM        (i)  Basic GIS - married
*   MP.BGISS        (i)  Basic GIS - single
*   MP.BOAS         (i)  Basic OAS
*   MP.FNTCR        (i)  Federal non-refundable tax credit rate
*   MP.GISRLM       (i)  Basic GIS reduction level: married pensioners
*   MP.GISRLS       (i)  Basic GIS reduction level: single pensioners
*   MP.GISRRM       (i)  Basic GIS reduction rate: married pensioners
*   MP.OASFLAG      (i)  Old age security flag
*   MP.PENSPLTINCR  (i)  Increment by which to split pension income
*   MP.QNTCR        (i)  Quebec nominal tax credit rate
*   MP.QPENSPLTINCR (i)  Quebec increment by which to split pension income
*   MP.QSPLTPNSFLAG (i)  Quebec flag to split pension income
*   MP.SPAOASRR     (i)  OAS portion of SPA taxback rate
*   MP.SPLTPNSFLAG  (i)  Flag to split pension income
*   MP.UIBAF        (i)  UI benefit recovery base amount factor (UI and EI)
*   MP.UIBAFNR      (i)  UI EI benefit recovery base amount factor for non-repeaters (EI only)
*   MP.UIBASRATE    (i)  Benefit rate for basic phase (UI only)
*   MP.UIEIMFSP     (i)  EI Maximum family supplement percent of earnings (EI only)
*   MP.UIEIOPT      (i)  UI Employment Insurance reform option [1=UI, 2=EI Dec'95]
*   MP.UIERNMAX     (i)  Maximum insurable earnings (UI and EI)
*   MP.CTFLAG       (io) Commodity tax activation flag
*   MP.GISBE1       (io) Breakeven for GIS one pensioner couple
*   MP.GISBE2       (io) Breakeven for GIS/SPA couple
*   MP.GISFLAG      (io) Federal GIS/SPA/ESPA flag
*   MP.GISTFLAG     (io) Provincial GIS top-up flag
*   MP.UIBRA        (io) UI repayment base amount (UI and EI)
*   MP.SPAXO        (o)  Benefit Cross-over GIS/SPA vs GIS one pensioner couple
*   MP.UIBRANR      (o)  EI repayment base amount for non-repeaters (EI only)
*
*Called by:
*   Ampc     Compute derived parameters and edits (user-defined)
*   mpin     Obtain model parameters from user
*
*****************************************************************************/
int mpc()
{
	int parmerr = FALSE;

	DEBUG_ON("mpc");

	/**
	* Calculate UI repayment base amount.
	**/

	MP.UIBRA    = MP.UIERNMAX * (NUMBER) 52.0 * MP.UIBAF;
	MP.UIBRANR  = MP.UIBRA * MP.UIBAFNR;

	/** 
	* Calculate Breakeven Points and Crossover for GIS/SPA
	* and 1 Pensioner Couple benefits.
	**/

	MP.GISBE1 = MP.BGISS/MP.GISRRM+MP.BOAS+MP.GISRLS;
	MP.GISBE2 = (MP.BGISM*2)/(MP.GISRRM*2)+MP.BOAS/MP.SPAOASRR+MP.GISRLM;
	MP.SPAXO  = 2*MP.GISBE2-MP.GISBE1;

	DEBUG4("%s Breakevens gis=%.2f gisspa=%.2f crossover=%.2f\n",
				MP.GISBE1, MP.GISBE2, MP.SPAXO);


	/* perform edit checks */
	if ( iszero(MP.QNTCR)) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d'\n" */
		lprintf(STRN(929), "QNTCR", MP.QNTCR);
		parmerr = TRUE;
	}

	if ( iszero(MP.FNTCR)) {
		/* 929 ==> "error(929): incompatible parameter values:  '%s=%d'\n" */
		lprintf(STRN(929), "FNTCR", MP.FNTCR);
		parmerr = TRUE;
	}

	if (MP.CTFLAG && !CP.FXVFLAG) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d' and '%s=%d'\n" */
		lprintf(STRN(929),"FXVFLAG", CP.FXVFLAG, "CTFLAG", MP.CTFLAG);
		/* 930 ==> "error(930): invalid value '%d' for %s, setting to '%d'\n" */
		lprintf(STRN(930), MP.CTFLAG, "CTFLAG", FALSE);
		MP.CTFLAG = FALSE;
		parmerr = TRUE;
	}

	if (MP.GISFLAG && !MP.OASFLAG) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d' and '%s=%d'\n" */
		lprintf(STRN(929),"OASFLAG", MP.OASFLAG, "GISFLAG", MP.GISFLAG);
		/* 930 ==> "error(930): invalid value '%d' for %s, setting to '%d'\n" */
		lprintf(STRN(930), MP.GISFLAG, "GISFLAG", FALSE);
		MP.GISFLAG = FALSE;
		parmerr = TRUE;
	}

	if (MP.GISTFLAG && !MP.GISFLAG) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d' and '%s=%d'\n" */
		lprintf(STRN(929),"GISFLAG", MP.GISFLAG, "GISTFLAG", MP.GISTFLAG);
		/* 930 ==> "error(930): invalid value '%d' for %s, setting to '%d'\n" */
		lprintf(STRN(930), MP.GISTFLAG, "GISTFLAG", FALSE);
		MP.GISTFLAG = FALSE;
		parmerr = TRUE;
	}

	if (MP.UIEIOPT==2 && (MP.UIEIMFSP < MP.UIBASRATE)) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d' and '%s=%d'\n" */
		lprintf(STRN(929),"UIEIMFSP", MP.UIEIMFSP, "UIBASRATE", MP.UIBASRATE);
		parmerr = TRUE;
	}

	if (MP.SPLTPNSFLAG && (MP.PENSPLTINCR==0.0)) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d' and '%s=%d'\n" */
		lprintf(STRN(929),"SPLTPNSFLAG", MP.SPLTPNSFLAG, "PENSPLTINCR", MP.PENSPLTINCR);
		parmerr = TRUE;
	}

	if (MP.QSPLTPNSFLAG && (MP.QPENSPLTINCR==0.0)) {
		/* 929 ==> "error(929): incompatible parameter values: '%s=%d' and '%s=%d'\n" */
		lprintf(STRN(929),"QSPLTPNSFLAG", MP.QSPLTPNSFLAG, "QPENSPLTINCR", MP.QPENSPLTINCR);
		parmerr = TRUE;
	}

	DEBUG_OFF("mpc");
	return(parmerr);
}
