/***
*txnb.cpp - Compute provincial taxes for New Brunswick
*
*$Id: txnb.cpp 6.124 2007/09/18 17:22:11 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/07/20 | Added a nneg test to the imictot test for heating reb   |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/18 | Removed BUCCBITF-UCCB removed from income as per forms  |
* | LP  | 07/01/12 | Changed PYINC to PYINCP for N.B. home energy asst prog  |
* |     |          | and corrected the income test from taxable to total     |
* | LP  | 06/12/13 | Switched iminet to imicnet, imitot to imictot (tax form)|
* | LP  | 06/06/13 | Add nneg to income tests that are reduced by imuccb     |
* | LP  | 06/06/09 | Removed BAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added BUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/06/05 | Revised the application of PYINCP                       |
* | LP  | 06/05/30 | Added N.B. Child Benefit for calendar year              |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate BDTCRLC|
* | LP  | 05/11/14 | Added N.B. Home Heating Oil Benefit program (BHEAT..)   |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 04/02/17 | Added idiworkc to income for tax reduction and NBCTB    |
* |     |          | and updated net income concepts for credit calculations |
* | LP  | 03/12/22 | Added BMEDALL and BMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/09 | Added impothpe, BODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added BCGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/12 | Added ideducpm and BEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred.|
* | LP  | 01/06/08 | Modified N.B. Low-Income Tax Reduction                  |
* | LP  | 01/05/31 | Added N.B. Low-Income Tax Reduction parameters          |
* | LP  | 01/05/09 | Added BAMTPCTM and added another option to BAMTOPT      |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/05/05 | Modified CGTC                                           |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/05/21 | Changed NB Child Tax Benefit phase in                   |
* | CH  | 97/10/03 | Implement NB Child Tax Benefit phase in                 |
* | BBM | 97/09/30 | Implement NB Child Tax Benefit                          |
* | BBM | 96/03/06 | Fix double discounting of foreign tax credit from imtxp |
* | BBM | 92/07/22 | Incorporated non-refundable provincial tax credit var   |
* | BBM | 92/07/08 | Modeled provincial tax credits, drop reference to idptc |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txnb     [global]  Compute provincial taxes for New Brunswick
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxnb[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txnb)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txnb - Compute provincial taxes for New Brunswick
*
*Synopsis:
*    void txnb(           
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   minn     Find the minimum of two numbers
*   lkup1    Table look-up with interpolation/extrapolation
*   taxbak1  One level tax-back function
*   lkup2    Table look-up without interpolation
*
*Structure Members:
*   MP.BAMTOPT      (i)  N.B. alternative minimum tax option
*   MP.BAMTPCTF     (i)  N.B. amt rate as pct of additional fed tax due to minimum tax
*   MP.BAMTPCTM     (i)  N.B. amt rate as pct of federal minimum tax amount
*   MP.BAMTTX       (i)  N.B. amt rate as tax on adjusted income
*   MP.BAXM         (i)  N.B. Age Amount
*   MP.BAXRR        (i)  N.B. Age Amount credit reduction rate
*   MP.BAXTD        (i)  N.B. Age Amount net income turndown
*   MP.BBXM         (i)  N.B. Basic Personal Exemption/amount
*   MP.BCGTC        (i)  N.B. Caregiver Tax Credit
*   MP.BCGTCFLG     (i)  N.B. Caregiver Tax Credit activation flag
*   MP.BCGTCOPT     (i)  N.B. Caregiver Tax Credit Option (1=max, 2=greenbook)
*   MP.BCHATL1      (i)  N.B. Charitable Donations amount level 1
*   MP.BCHATR1      (i)  N.B. Charitable Donations tax credit rate 1
*   MP.BCHATR2      (i)  N.B. Charitable Donations tax credit rate 2
*   MP.BDTCR        (i)  N.B. dividend tax credit rate
*   MP.BDTCRLC      (i)  N.B. dividend tax credit rate from large (eligible) corporations
*   MP.BEDXPM       (i)  N.B. Education Amount per month of full-time studies
*   MP.BEDXPMPT     (i)  N.B. Education Amount per month of part-time studies
*   MP.BEMXM        (i)  N.B. equivalent to married amount
*   MP.BEMXMT       (i)  N.B. equivalent to married turndown level
*   MP.BHEATBEN     (i)  New Brunswick Home Heating Oil Benefit [income, benefit]
*   MP.BHEATBENrows (i)  number of rows in BHEATBEN
*   MP.BHEATFLG     (i)  New Brunswick Home Heating Oil Benefit activation flag
*   MP.BHEATPO      (i)  New Brunswick Home Heating Oil Benefit family income phase out
*   MP.BHEATSA      (i)  New Brunswick Home Heating Oil Benefit for SA recipients flag
*   MP.BLVCMAX      (i)  Maximum N.B. labour-sponsored funds tax credit allowed
*   MP.BLVCRT       (i)  Percent of N.B. labour-sponsored funds cost allowed as credit
*   MP.BMAXDX       (i)  N.B. Maximum Disability deduction/amount
*   MP.BMAXET       (i)  N.B. maximum on transfer of education and tuition amount
*   MP.BMEDALL      (i)  N.B. Medical allowance maximum lower limit
*   MP.BMEDANF      (i)  N.B. Medical allowance lower limit net income fraction
*   MP.BMXM         (i)  N.B. married amount
*   MP.BMXMT        (i)  N.B. married amount turndown level
*   MP.BODISTC      (i)  N.B. Maximum infirm dependent 18 or older tax credit
*   MP.BPNTCR       (i)  N.B. provincial non-refundable tax credit rate
*   MP.BPTC         (i)  New Brunsw. political contribution table [total donations,donation allowed]
*   MP.BPTCBEN      (i)  Maximum New Brunswick political tax credit allowed
*   MP.BPTCrows     (i)  number of rows in BPTC
*   MP.BPTF         (i)  New Brunswick provincial tax fraction
*   MP.BPTX         (i)  N.B. tax table [taxable income,basic provincial tax]
*   MP.BPTXrows     (i)  number of rows in BPTX
*   MP.BSCI         (i)  New Brunswick provincial tax above which surtax applies
*   MP.BSF          (i)  New Brunswick provincial surtax rate
*   MP.BTRBAS       (i)  Base N.B. low income tax reduction
*   MP.BTRRR        (i)  N.B. low income tax reduction Reduction Rate
*   MP.BTRSP        (i)  N.B. low income tax reduction for spouse or equivalent-to-spouse
*   MP.BTRTD        (i)  N.B. low income tax reduction Turndown
*   MP.BTXFLG       (i)  N.B. tax on taxable income activation flag
*   MP.BYPNDL       (i)  N.B. Pension Income Deduction Amount
*   MP.CTCINC       (i)  Refundable CTC family income scaling factor
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.NBCBBAS      (i)  NB Basic child benefit (per child)
*   MP.NBCBBASP     (i)  NB Basic child benefit (per child) for 1st half of year
*   MP.NBCBECI      (i)  NB Child benefits earning supplement cut-in level
*   MP.NBCBECIP     (i)  NB Child benefits earning supplement cut-in level for 1st half of year
*   MP.NBCBEIS      (i)  NB Child benefits earned income supplement
*   MP.NBCBEISP     (i)  NB Child benefits earned income supplement for 1st half of year
*   MP.NBCBERR      (i)  NB Child benefits earning supplement reduction rate
*   MP.NBCBERRP     (i)  NB Child benefits earning supplement reduction rate for 1st half of year
*   MP.NBCBESR      (i)  NB Child benefits earned income supplement rate
*   MP.NBCBESRP     (i)  NB Child benefits earned income supplement rate for 1st half of year
*   MP.NBCBETD      (i)  NB Child benefits earning supplement turndown level
*   MP.NBCBETDP     (i)  NB Child benefits earning supplement turndown level for 1st half of year
*   MP.NBCBPI       (i)  NB Child benefits phase-in rate
*   MP.NBCBPI1P     (i)  NB Child benefits phase-in rate for 1st half of year
*   MP.NBCBPI2P     (i)  NB Child benefits phase-in rate for 2nd half of year
*   MP.NBCBRR       (i)  NB Multi-children family income reduction rate
*   MP.NBCBRRP      (i)  NB Multi-children family income reduction rate for 1st half of year
*   MP.NBCBRRS      (i)  NB Single-child family income reduction rate
*   MP.NBCBRRSP     (i)  NB Single-child family income reduction rate for 1st half of year
*   MP.NBCBTD       (i)  NB child benefits family income turndown
*   MP.NBCBTDP      (i)  NB child benefits family income turndown for 1st half of year
*   MP.NBCTBFLG     (i)  NB child benefit activation flag
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   fx.fxio         (i)  I/O expenditure categories [array]
*   hh.cf           (i)  Census family data [array]
*   hh.fx           (i)  SHS data [struct]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idisefm      (i)  Self-employed income - farming
*   id.idisenf      (i)  Self-employed income - non-farming
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idsex        (i)  Sex
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imamtfg      (i)  Minimum tax flag
*   im.imamtinc     (i)  Net adjusted income used for minimum tax
*   im.imamtprv     (i)  Federal minimum tax concept used for provincial minimum tax
*   im.imbft        (i)  Basic federal tax
*   im.imchara      (i)  Allowable charitable donations and gifts (calculated)
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedfn      (i)  All deductions from net income
*   im.imdepni      (i)  Dependant's net income
*   im.imicnet      (i)  Net Income (line 236)
*   im.imictot      (i)  Total Income (line 150)
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imitax       (i)  Taxable income
*   im.imminamt     (i)  Minimum Amount due to Federal Minimum Tax
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imbpt        (io) Basic provincial tax
*   im.imnptc       (io) Non-refundable provincial tax credits
*   im.impatxc      (io) Prov Age amount tax credit
*   im.impbtc       (io) Prov Basic amount
*   im.impcgtc      (io) Prov Caregiver Tax Credit Level
*   im.impchrtc     (io) Prov Charitable Donations tax credit
*   im.impcpptc     (io) Prov CPP/QPP contributions tax credit
*   im.impdistc     (io) Prov Disability tax credit
*   im.impdtxc      (io) Provincial dividend tax credit
*   im.impedtxc     (io) Prov Education tax credit
*   im.impheatr     (io) Provincial Home Heating Fuel/Energy Rebate
*   im.impintsl     (io) Provincial interest on student loans tax credit
*   im.implvctc     (io) Provincial labour-sponsored funds tax credit
*   im.impmartc     (io) Prov Married and Equivalent to Married tax credit
*   im.impmeda      (io) Prov Medical expenses allowed (computed)
*   im.impmedtc     (io) Prov Medical Expenses tax credit
*   im.impminco     (io) Prov Minimum tax carryover
*   im.impothpe     (io) Provincial infirm dependents 18 or older tax credit
*   im.imppentc     (io) Prov Pension Income tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.imprvtax     (io) Prov gross provincial tax
*   im.impsur       (io) Provincial surtax
*   im.imptaxcr     (io) Prov total tax credits
*   im.imptr        (io) Provincial tax reduction
*   im.impttxct     (io) Prov total tax credits transferred
*   im.imptutxc     (io) Prov Tuition tax credit
*   im.impuictc     (io) Prov UI/EI contributions tax credit
*   im.imtxp        (io) Provincial income tax payable
*   im.imnbcbb      (o)  NB child tax Benefits base amount
*   im.imnbcben     (o)  Total NB child tax Benefits
*   im.imnbwis      (o)  NB child tax Benefits WIS
*   im.impatxcr     (o)  Prov total tax credits applied
*   im.impctxct     (o)  Prov tax credits transferred from children
*   im.impstxct     (o)  Prov tax credits transferred from spouse
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void txnb(
    P_hh hh
)
{
    register int ini;
    register P_in in;
    register P_in ineld;
    register P_in inspo;
    register    P_nf nf;
    register    P_in nfeld;
    register    P_in nfspo;
    register int nfi;
    register P_cf cf;
    register int cfi;
	register P_fx fx;

	fx = &hh->fx;	

	NUMBER elgind = ZERO;      /* eligible individual for home heating oil benefit  */
    int gotheat;               /* CF got the home heating benefit */
	NUMBER heatinc = ZERO;     /* head and spouse total income for heat rebate income test */
	NUMBER sainc = ZERO;       /* head and spouse SA income for heat rebate income test */

    NUMBER poltcr = ZERO;    /* Political tax credit (raw) */
    NUMBER poltc = ZERO;     /* Political tax credit */
    NUMBER cben = ZERO;
    NUMBER cbeneis = ZERO;
    NUMBER cben1 = ZERO;
    NUMBER cbeneis1 = ZERO;
    NUMBER cben2 = ZERO;
    NUMBER cbeneis2 = ZERO;
    NUMBER nfinet = ZERO;
    NUMBER nfiemp = ZERO;
    NUMBER eismax = ZERO;
    NUMBER eismax1 = ZERO;
    NUMBER eismax2 = ZERO;
    NUMBER lvctc = ZERO;     /* Labour-sponsored venture capital tax cred */

    DEBUG_ON("txnb");

    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.BTXFLG) { 

       /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;

          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */

          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni;
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.BPNTCR * nneg(MP.BEMXM - nneg(cinet - MP.BEMXMT));
             }
          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption 
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.BPNTCR * nneg(MP.BMXM - 
                	nneg(inspo->im.imdepni - MP.BMXMT));
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.BPNTCR * nneg(MP.BMXM - 
                	nneg(ineld->im.imdepni - MP.BMXMT));
             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
          }
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/

			 /* calculate caregivers tax credit */
			 if (MP.BCGTCFLG && isnzero(in->id.idcgtc)) {
				/* if BGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.BCGTCOPT == 1) {
					in->im.impcgtc = MP.BCGTC * MP.BPNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.BPNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }

             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.BPNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.BPNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.BPNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.BPNTCR * MP.BBXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc = in->im.imicnet ;

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.BAXM, MP.BAXRR * nneg(netinc - MP.BAXTD));
                adjamt = MP.BAXM - agered;
                in->im.impatxc = MP.BPNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.BPNTCR * minn(MP.BYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.BMEDALL, MP.BMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.BPNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

			 /* calculate amount for infirm dependents age 18 or older */
			 /*  if the filer has an amount for idothpe, then give max credit */

			 if (isnzero(in->id.idothpe)) {
   				in->im.impothpe = MP.BPNTCR * MP.BODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
			 }
		     
             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.BCHATL1, in->im.imchara)  * MP.BCHATR1
                            + nneg(in->im.imchara - MP.BCHATL1) * MP.BCHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.BPNTCR * MP.BMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.BPNTCR * MP.BMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.BPNTCR * (in->id.ideducfm * MP.BEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.BPNTCR * (in->id.ideducpm * MP.BEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.BPNTCR * (in->id.ideducpm * MP.BEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }

			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.BPNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }

             /* calculate federal dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.BDTCR)
								+ (in->im.imildivt * MP.BDTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }

			 /* calculate the New Brunswick minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.BAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use BAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.BAMTPCTF;
				 }
				 else if (MP.BAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using BAMTTX. */
						in->im.impminco = in->id.idmincar * MP.BAMTTX;
					  }
				 else if (MP.BAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using BAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.BAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
            
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */
			 if (in->im.imamtfg && MP.BAMTOPT == 3) {
                in->im.imprvtax = MP.BAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.BPTX, MP.BPTXrows, in->im.imitax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
                        + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
						+ in->im.impminco    /* provincial minimum tax carryover credit */
						+ in->im.impintsl   /* provincial interest on student loans credit */
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.BPNTCR * MP.BMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.BPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.BPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                       + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco   /* provincial minimum tax carryover credit */
					   + in->im.impintsl   /* provincial interest on student loans credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
				    + in->im.impcgtc    /* caregiver tax credit                  */
                    + in->im.imppentc   /* pension income tax credit             */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.BPNTCR * MP.BMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.BPNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.BBXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic));
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

    /* process each person in household */
    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
           ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

        /* original tax as percentage of federal tax */
        if (!MP.BTXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.BPTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if BTXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.BAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.BAMTPCTF;
			}
			else {
				if (in->im.imamtfg && MP.BAMTOPT == 4) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.BAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}
			
        /* compute and add in surtax */
        in->im.impsur = MP.BSF * nneg(in->im.imtxp - MP.BSCI);

        /* apply surtax */
        in->im.imtxp += in->im.impsur;

        /* compute tax credits */
        /* Note: Stock savings plan tax credit started in 1990 and */
        /*       is therfore not implemented here(until release of */
        /*       the 1990 database)                                */  

        /* calculate and apply provincial foreign tax credit */
        in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
        in->im.imtxp -= in->im.imnptc;

	}
    /* NB low income tax reduction */
    /* calculate for the family. */

    /* process each nuclear family in household */
    for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

        /* initialize pointers */
        nfeld = nf->nfineld;
        nfspo = nf->nfinspo;
        NUMBER ptr = ZERO;
        NUMBER faminc = ZERO;

        /* if there is no spouse */
        if (!nf->nfspoflg) {
           ptr = MP.BTRBAS;
           /* since there is no spouse, treat one of the kids as equivalent-to-married */
           if (nf->nfnkids>0) {
              ptr += MP.BTRSP;
           }
           faminc = nneg( nfeld->im.imicnet - nfeld->im.imuccb );
        }
        /* there is a spouse */
        else {
            ptr = MP.BTRSP + MP.BTRBAS;
            faminc = nneg( nfeld->im.imicnet + nfspo->im.imicnet  
            		     - nfeld->im.imuccb - nfspo->im.imuccb ) ;
        }

        /* tax back */
        taxbak1(faminc, MP.BTRTD, &ptr, MP.BTRRR);
        /* give the tax reduction */
        if (!nf->nfspoflg) {
            nfeld->im.imptr = minn(ptr,nfeld->im.imtxp);
            nfeld->im.imtxp -= nfeld->im.imptr;
        }
        else {
            /* give to spouse with highest taxes outstanding first */
            if (nfeld->im.imtxp>nfspo->im.imtxp) {
                /* elder had higher taxes */
                nfeld->im.imptr = minn(ptr,nfeld->im.imtxp);
                nfeld->im.imtxp -= nfeld->im.imptr;
                /* if there is any unused reduction, transfer to the other spouse */
                if (nfeld->im.imptr < ptr) {
                    nfspo->im.imptr = minn((ptr - nfeld->im.imptr),nfspo->im.imtxp);
                    nfspo->im.imtxp -= nfspo->im.imptr;
                }
            }
            else {
                /* spouse had higher taxes */
                nfspo->im.imptr = minn(ptr,nfspo->im.imtxp);
                nfspo->im.imtxp -= nfspo->im.imptr;
                /* if there is any unused reduction, transfer to the other spouse */
                if (nfspo->im.imptr < ptr) {
                    nfeld->im.imptr = minn((ptr - nfspo->im.imptr),nfeld->im.imtxp);
                    nfeld->im.imtxp -= nfeld->im.imptr;
                }
            }
        }
    }

    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        /**
        * N.B. Political Contribution Tax Credit 
        * This credit is calculated to take up any surplus tax after the provincial 
        * foreign tax credit has been applied
        **/
        if ( isnzero(in->id.idprvpol)) {
            poltcr = (NUMBER) lkup1(MP.BPTC, MP.BPTCrows, in->id.idprvpol);
            poltc = minn(MP.BPTCBEN, poltcr);
            in->im.impptc  = minn(poltc, in->im.imtxp);
            DEBUG4("%s nb pol: idpolcon=%.2f poltcr=%.2f poltc=%.2f\n",
               in->id.idprvpol, poltcr, poltc);

            /* apply political contribution tax credit */
            in->im.imnptc += in->im.impptc;
            in->im.imtxp  -= in->im.impptc;

            DEBUG3("%s idpolcon =%.2f impptc =%.2f\n",
               in->id.idprvpol, in->im.impptc);
        }

        /* NB labour-sponsored venture capital credit */
        if ( isnzero(in->id.idlabtxg) ) {
           lvctc = minn(MP.BLVCMAX, MP.BLVCRT * in->id.idlabtxg);
           in->im.implvctc = minn(lvctc, in->im.imtxp);
           in->im.imnptc += in->im.implvctc;
           in->im.imtxp  -= in->im.implvctc;
        }

        DEBUG4("%s NB   bpt=%.2f, txp=%.2f, nptc=%.2f\n",
            in->im.imbpt, in->im.imtxp, in->im.imnptc);
    }


    /**
    * New Brunswick Child Tax Benefit 
    **/

    if (MP.NBCTBFLG==1) {
        DEBUG1("%s child benefit program in effect\n");

      /* process each nuclear family in household */
       for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
            DEBUG4("%s cben: nf %d spoflg=%d, nkids=%d\n",
            nfi, nf->nfspoflg, nf->nfnkids);

       	   if (nf->nfnkids == 0) {
               DEBUG1("%s cben: skipping nuclear family with no `children'\n");
               continue;
           }

           /* Initialize local variables and head and spouse pointers */
           ineld = nf->nfineld;
           inspo = nf->nfinspo;

           /* Compute Family net income and family employment income */
           nfinet = MP.PYINC * nneg ( ineld->im.imicnet - ineld->im.imuccb );
           nfiemp = MP.PYINC *
                    (ineld->id.idiemp + ineld->id.idisenf + ineld->id.idisefm );

           if (nf->nfspoflg) {
               nfinet += MP.PYINC * nneg (inspo->im.imicnet - inspo->im.imuccb );
               nfiemp += MP.PYINC *
                          (inspo->id.idiemp + inspo->id.idisenf + inspo->id.idisefm);
           }

           /* Scale incomes with user specified rates */
           nfinet = nfinet * MP.CTCINC;
           nfiemp = nfiemp * MP.CTCINC;

           DEBUG3("%s cben: neti=%.2f empi=%.2f\n", nfinet, nfiemp);

           /* initialize base benefit*/
           cben2 = ZERO;

           /* Assign base amount which is dependent on number of kids */
           /* this amount is multiplied by a phase-in fraction */
           cben2 = MP.NBCBBAS * nf->nfnkids ;

           /***
           ** Tax Back basic benefits
           ** providing a different(reduced) tax rate for
           ** single children families
           ***/
           if (nf->nfnkids==1) {
               taxbak1(nfinet, MP.NBCBTD, &cben2, MP.NBCBRRS);
           }
           else {
               taxbak1(nfinet, MP.NBCBTD, &cben2, MP.NBCBRR);
           }

           DEBUG4("%s cben: cben after reduction cben=%.2f neti=%.2f empi=%.2f\n",
                    cben2, nfinet, nfiemp);

           /***
           ** calc gross earned income supplement
           ** then use taxbak to reduce the benefit 
           ** for families with higher net incomes
           ***/

           if ( nfiemp > MP.NBCBECI ) {
            
               eismax2 = MP.NBCBEIS  ;
               DEBUG1("%s cben: earned inc benefit calc\n");

               cbeneis2 = minn(eismax2,  MP.NBCBESR * (nfiemp - MP.NBCBECI) );
               DEBUG2("%s cben: gross earned inc benefit cbeneis=%.2f\n",
                    cbeneis2);

               if (isnzero(cbeneis2)) {
                    taxbak1(nfinet, MP.NBCBETD, &cbeneis2, MP.NBCBERR);
                
                    DEBUG2("%s cben: reduced earned inc benefit cbeneis=%.2f\n",
                    cbeneis2);
               }
         
           }
           else {
               DEBUG1("%s cben: earned inc to low for supplement\n");
               cbeneis2 = ZERO;
           }

		   if ( MP.FCBENCAL ) {  /* calculate for the first half of year as well */

	           /* initialize base benefit*/
	           cben1 = ZERO;

			   /* Adjust nfinet to represent net income from 2 years previous */

			   nfinet = ( nfinet / MP.PYINC ) * MP.PYINCP;  
			   nfiemp = ( nfiemp / MP.PYINC ) * MP.PYINCP;
				
	           /* Assign base amount which is dependent on number of kids */
	           /* this amount is multiplied by a phase-in fraction */
	           cben1 = MP.NBCBBASP * nf->nfnkids ;

	           /***
	           ** Tax Back basic benefits
	           ** providing a different(reduced) tax rate for
	           ** single children families
	           ***/
	           if (nf->nfnkids==1) {
	               taxbak1(nfinet, MP.NBCBTDP, &cben1, MP.NBCBRRSP);
	           }
	           else {
	               taxbak1(nfinet, MP.NBCBTDP, &cben1, MP.NBCBRRP);
	           }

	           DEBUG4("%s cben: cben after reduction cben=%.2f neti=%.2f empi=%.2f\n",
	                    cben1, nfinet, nfiemp);

	           /***
	           ** calc gross earned income supplement
	           ** then use taxbak to reduce the benefit 
	           ** for families with higher net incomes
	           ***/

	           if ( nfiemp > MP.NBCBECIP ) {
	            
	               eismax1 = MP.NBCBEISP  ;
	               DEBUG1("%s cben: earned inc benefit calc\n");

	               cbeneis1 = minn(eismax1,  MP.NBCBESRP * (nfiemp - MP.NBCBECIP) );
	               DEBUG2("%s cben: gross earned inc benefit cbeneis=%.2f\n",
	                    cbeneis1);

	               if (isnzero(cbeneis1)) {
	                    taxbak1(nfinet, MP.NBCBETDP, &cbeneis1, MP.NBCBERRP);
	                
	                    DEBUG2("%s cben: reduced earned inc benefit cbeneis=%.2f\n",
	                    cbeneis1);
	               }
	         
	           }
	           else {
	               DEBUG1("%s cben: earned inc to low for supplement\n");
	               cbeneis1 = ZERO;
	           }

			   cben = ((cben1/2) * MP.NBCBPI1P ) + ((cben2/2) * MP.NBCBPI2P);
			   cbeneis = ((cbeneis1/2) * MP.NBCBPI1P)  + ((cbeneis2/2) * MP.NBCBPI2P );
		   }
		   else {  /* use the July to June amounts for the year */
		   	   cben = cben2 * MP.NBCBPI;
		   	   cbeneis = cbeneis2 * MP.NBCBPI;
		   }	

           /* assign child benefits to mother if present */

           if (nf->nfspoflg && (inspo->id.idsex == FEMALE)) {
               DEBUG1("%s cben: spouse is the mother, gets CTC\n");
               inspo->im.imnbcbb  = cben;
               inspo->im.imnbwis  = cbeneis ;
               inspo->im.imnbcben = (cben + cbeneis);

           }
           else {
               DEBUG1("%s cben: elder receives CTC\n");
               ineld->im.imnbcbb   = cben ;
                ineld->im.imnbwis  = cbeneis;
                ineld->im.imnbcben = (cben + cbeneis);

           }
      }
   }

	/* New Brunswick Home Heating Oil Benefit */
    if ( MP.BHEATFLG ) {
		/* initialise flag to indicate that one family has already received the credit*/
		gotheat = 0;

        /* process each census family in household in reverse order */
		for (cfi=0, cf=&hh->cf[hh->hhncf-1]; cfi<hh->hhncf; cfi++, cf--) {


			/* initialise the number of eligible household individuals to 0 */
			elgind = 0;
			heatinc = 0;
			sainc = 0;

			/* initialise elder's pointer */
			ineld = cf->cfineld;

			/* initialise spouse's pointer if spouse is present. */

			if (cf->cfspoflg) {
				inspo = cf->cfinspo;
			}
			
			/* Only those households that use home heating oil as the primary source of heat
			   are eligible.  Assuming those households with net expenditure on natural gas
			   or other fuels are eligible.  */
	
			if ( fx->fxio[14] > 0 ) {
				 /* Need to capture the income of head and spouse's total income */
				 heatinc = MP.PYINCP * nneg(ineld->im.imictot) ;
				 sainc = ineld->im.imisa;
				 if (cf->cfspoflg) {
				 	heatinc += MP.PYINCP * nneg(inspo->im.imictot);
					sainc += inspo->im.imisa;
				 }
				 /* Test SA options - not eligible if self or spouse in receipt of regular sa */
				 
				 if (MP.BHEATSA) {

				 	if (heatinc < MP.BHEATPO ) {
				 		elgind++;
					}
				 }
				 else {  /* if in receipt of SA not eligible */
					if (heatinc < MP.BHEATPO && sainc == ZERO ) {
						elgind++;
					}
			     }
			}

            /* is there an eligible person to receive the benefit, and is this the first 
               family in the household who can receive the benefit */
			if ( elgind > 0 && gotheat==0 ) {


			    /* If this is an eligible household, give to spouse with higher total income.	 */
				if (cf->cfspoflg && (inspo->im.imictot > ineld->im.imictot)) {
					DEBUG1("%s spouse has higher total income and gets heating rebate\n");
        			inspo->im.impheatr = nneg((NUMBER) lkup2(MP.BHEATBEN, MP.BHEATBENrows, heatinc)) ;
				}
				else {
					DEBUG1("%s elder has higher total income and gets heating rebate\n");
        			ineld->im.impheatr = nneg((NUMBER) lkup2(MP.BHEATBEN, MP.BHEATBENrows, heatinc)); 
				}
                
                /* check that family has gotten credit */
				if (ineld->im.impheatr>0 || inspo->im.impheatr>0) {
                   gotheat=1;
				}

			}
		}

   	}
	
		

    DEBUG_OFF("txnb");
}
