/***
*txpei.cpp - Compute provincial taxes for P.E.I.
*
*$Id: txpei.cpp 6.112 2007/09/18 17:22:11 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/18 | Removed PUCCBITF-UCCB removed from income as per forms  |
* | LP  | 07/01/15 | Added impyctxc, PYCTCAPM, PYCTCPI - young child tx cred |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/09 | Removed PAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added PUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate PDTCRLC|
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 04/02/17 | Added idiworkc to income test for P.E.I. tax reduction  |
* | LP  | 03/12/22 | Added PMEDALL and PMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/09 | Added impothpe, PODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added PCGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/12 | Added ideducpm and PEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred.|
* | LP  | 01/06/06 | Moved low income tax reduction as per tax forms         |
* | LP  | 01/05/09 | Added PAMTPCTM and added another option to PAMTOPT      |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/05/05 | Modified CGTC                                           |
* | CH  | 00/03/21 | Added low income tax reduction                          |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/10/20 | Changed political contributions tax credit              |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | BBM | 92/07/22 | Incorporated non-refundable provincial tax credit var   |
* | BBM | 92/07/08 | Modeled provincial tax credits, drop reference to idptc |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txpei    [global]  Compute provincial taxes for P.E.I.
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxpei[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txpei)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txpei - Compute provincial taxes for P.E.I.
*
*Synopsis:
*    void txpei(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   minn     Find the minimum of two numbers
*   lkup1    Table look-up with interpolation/extrapolation
*   taxbak1  One level tax-back function
*
*Structure Members:
*   MP.PAMTOPT      (i)  P.E.I. alternative minimum tax option
*   MP.PAMTPCTF     (i)  P.E.I. amt rate as pct of additional fed tax due to minimum tax
*   MP.PAMTPCTM     (i)  P.E.I. amt rate as pct of federal minimum tax amount
*   MP.PAMTTX       (i)  P.E.I. amt rate as tax on adjusted income
*   MP.PAXM         (i)  P.E.I. Age Amount
*   MP.PAXRR        (i)  P.E.I. Age Amount credit reduction rate
*   MP.PAXTD        (i)  P.E.I. Age Amount net income turndown
*   MP.PBXM         (i)  P.E.I. Basic Personal Exemption/amount
*   MP.PCGTC        (i)  P.E.I. Caregiver Tax Credit
*   MP.PCGTCFLG     (i)  P.E.I. Caregiver Tax Credit activation flag
*   MP.PCGTCOPT     (i)  P.E.I. Caregiver Tax Credit Option (1=max, 2=greenbook)
*   MP.PCHATL1      (i)  P.E.I. Charitable Donations amount level 1
*   MP.PCHATR1      (i)  P.E.I. Charitable Donations tax credit rate 1
*   MP.PCHATR2      (i)  P.E.I. Charitable Donations tax credit rate 2
*   MP.PDTCR        (i)  P.E.I. dividend tax credit rate
*   MP.PDTCRLC      (i)  P.E.I. dividend tax credit rate from large (eligible) corporations
*   MP.PEDXPM       (i)  P.E.I. Education Amount per month of full-time studies
*   MP.PEDXPMPT     (i)  P.E.I. Education Amount per month of part-time studies
*   MP.PEMXM        (i)  P.E.I. equivalent to married amount
*   MP.PEMXMT       (i)  P.E.I. equivalent to married turndown level
*   MP.PLVCMAX      (i)  Maximum P.E.I. labour-sponsored funds tax credit allowed
*   MP.PLVCRT       (i)  Percent of P.E.I. labour-sponsored funds cost allowed as credit
*   MP.PMAXDX       (i)  P.E.I. Maximum Disability deduction/amount
*   MP.PMAXET       (i)  P.E.I. maximum on transfer of education and tuition amount
*   MP.PMEDALL      (i)  P.E.I. Medical allowance maximum lower limit
*   MP.PMEDANF      (i)  P.E.I. Medical allowance lower limit net income fraction
*   MP.PMXM         (i)  P.E.I. married amount
*   MP.PMXMT        (i)  P.E.I. married amount turndown level
*   MP.PODISTC      (i)  P.E.I. Maximum infirm dependent 18 or older tax credit
*   MP.PPNTCR       (i)  P.E.I. provincial non-refundable tax credit rate
*   MP.PPTC         (i)  P.E.I. political contribution table [total donations,donation allowed]
*   MP.PPTCBEN      (i)  Maximum P.E.I. political tax credit allowed
*   MP.PPTCrows     (i)  number of rows in PPTC
*   MP.PPTF         (i)  P.E.I. provincial tax fraction
*   MP.PPTX         (i)  P.E.I. tax table [taxable income,basic provincial tax]
*   MP.PPTXrows     (i)  number of rows in PPTX
*   MP.PSCI         (i)  P.E.I. provincial tax above which surtax applies
*   MP.PSF          (i)  P.E.I. provincial surtax rate
*   MP.PTRBAS       (i)  Base PEI low income tax reduction
*   MP.PTRKID       (i)  PEI low income tax reduction for dependent child
*   MP.PTRRR        (i)  PEI low income tax reduction Reduction Rate
*   MP.PTRSP        (i)  PEI low income tax reduction for spouse or equivalent-to-spouse
*   MP.PTRTD        (i)  PEI low income tax reduction Turndown
*   MP.PTXFLG       (i)  P.E.I. tax on taxable income activation flag
*   MP.PYCTCAPM     (i)  P.E.I. Young Child Tax Credit Amount per Month
*   MP.PYCTCPI      (i)  P.E.I. Young Child Tax Credit Phase in
*   MP.PYPNDL       (i)  P.E.I. Pension Income Deduction Amount
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.ididiv       (i)  Dividend income (actual)
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imamtfg      (i)  Minimum tax flag
*   im.imamtinc     (i)  Net adjusted income used for minimum tax
*   im.imamtprv     (i)  Federal minimum tax concept used for provincial minimum tax
*   im.imbft        (i)  Basic federal tax
*   im.imchara      (i)  Allowable charitable donations and gifts (calculated)
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedfn      (i)  All deductions from net income
*   im.imdepni      (i)  Dependant's net income
*   im.imicnet      (i)  Net Income (line 236)
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imitax       (i)  Taxable income
*   im.imminamt     (i)  Minimum Amount due to Federal Minimum Tax
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imbpt        (io) Basic provincial tax
*   im.imnptc       (io) Non-refundable provincial tax credits
*   im.impatxc      (io) Prov Age amount tax credit
*   im.impbtc       (io) Prov Basic amount
*   im.impcgtc      (io) Prov Caregiver Tax Credit Level
*   im.impchrtc     (io) Prov Charitable Donations tax credit
*   im.impcpptc     (io) Prov CPP/QPP contributions tax credit
*   im.impdistc     (io) Prov Disability tax credit
*   im.impdtxc      (io) Provincial dividend tax credit
*   im.impedtxc     (io) Prov Education tax credit
*   im.impintsl     (io) Provincial interest on student loans tax credit
*   im.implvctc     (io) Provincial labour-sponsored funds tax credit
*   im.impmartc     (io) Prov Married and Equivalent to Married tax credit
*   im.impmeda      (io) Prov Medical expenses allowed (computed)
*   im.impmedtc     (io) Prov Medical Expenses tax credit
*   im.impminco     (io) Prov Minimum tax carryover
*   im.impothpe     (io) Provincial infirm dependents 18 or older tax credit
*   im.imppentc     (io) Prov Pension Income tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.imprvtax     (io) Prov gross provincial tax
*   im.impsur       (io) Provincial surtax
*   im.imptaxcr     (io) Prov total tax credits
*   im.imptr        (io) Provincial tax reduction
*   im.impttxct     (io) Prov total tax credits transferred
*   im.imptutxc     (io) Prov Tuition tax credit
*   im.impuictc     (io) Prov UI/EI contributions tax credit
*   im.impyctxc     (io) Prov Young Child tax credit
*   im.imtxp        (io) Provincial income tax payable
*   im.impatxcr     (o)  Prov total tax credits applied
*   im.impctxct     (o)  Prov tax credits transferred from children
*   im.impstxct     (o)  Prov tax credits transferred from spouse
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void txpei(
    P_hh hh
)
{
    register int ini;
    register P_in in;
    register P_in ineld;
    register P_in inspo;
    register P_cf cf;
    register int cfi;
    register    P_nf nf;
    register    P_in nfeld;
    register    P_in nfspo;
    register int nfi;
    NUMBER poltc = ZERO;     /* Political tax credit */
	NUMBER lvctc = ZERO;     /* Labour-sponsored venture capital tax cred */
	int yngkids;             /* count of kids under age 6 for young child tax credit */

    DEBUG_ON("txpei");

    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.PTXFLG) { 

       /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;

          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */
		  yngkids = 0;

          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni ;
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.PPNTCR * nneg(MP.PEMXM - nneg(cinet - MP.PEMXMT));
             }
			 
			 /* while in this loop, count the young kids in census family */
			 if (in->id.idage <= 5) {
				yngkids += 1;
			 }

          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption 
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.PPNTCR * nneg(MP.PMXM - nneg(inspo->im.imdepni - MP.PMXMT));
                	 
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.PPNTCR * nneg(MP.PMXM - nneg(ineld->im.imdepni - MP.PMXMT));
             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
          }


		  /* New for 2006, young child tax credit.  The parent with the lower
		     net income can claim an amount per month per child under 6 as a tax credit.
		     Begins in July 2006.  Parameters are set to match tax form, which grants
		     an amount per month.  Hard coding the number of months per year as 12 and
		     adjusting for 2006 with a phase in rate of 0.5. */	
		 
		  if (cf->cfspoflg) {
		  	  if ( ineld->im.imicnet <= inspo->im.imicnet) {
		  		 /* elder has the lower income, elder get young child tax credit */
		  		 ineld->im.impyctxc = MP.PYCTCAPM * 12.0 * MP.PYCTCPI * yngkids * MP.PPNTCR;
		  	  }
		  	  else {  /* spouse has the lower income, gets young child tax credit */
		  	  	  inspo->im.impyctxc = MP.PYCTCAPM * 12.0 * MP.PYCTCPI * yngkids * MP.PPNTCR;
		  	  }
		  }
		  else {  /* no spouse present */
		  	  ineld->im.impyctxc = MP.PYCTCAPM * 12.0 * MP.PYCTCPI * yngkids * MP.PPNTCR;
		  }		
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/

			 /* calculate caregivers tax credit */
			 if (MP.PCGTCFLG && isnzero(in->id.idcgtc)) {
				/* if PGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.PCGTCOPT == 1) {
					in->im.impcgtc = MP.PCGTC * MP.PPNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.PPNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }


             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.PPNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.PPNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.PPNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.PPNTCR * MP.PBXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc = in->im.imicnet; 

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.PAXM, MP.PAXRR * nneg(netinc - MP.PAXTD));
                adjamt = MP.PAXM - agered;
                in->im.impatxc = MP.PPNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.PPNTCR * minn(MP.PYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.PMEDALL, MP.PMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.PPNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

 			 /* calculate amount for infirm dependents age 18 or older */
 			 /*  if the filer has an amount for idothpe, then give max credit */

 		     if (isnzero(in->id.idothpe)) {
    			in->im.impothpe = MP.PPNTCR * MP.PODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
 			 }

             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.PCHATL1, in->im.imchara)  * MP.PCHATR1
                            + nneg(in->im.imchara - MP.PCHATL1) * MP.PCHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.PPNTCR * MP.PMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.PPNTCR * MP.PMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.PPNTCR * (in->id.ideducfm * MP.PEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.PPNTCR * (in->id.ideducpm * MP.PEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.PPNTCR * (in->id.ideducpm * MP.PEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }

			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.PPNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }

             /* calculate federal dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.PDTCR)
								+ (in->im.imildivt * MP.PDTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }

			 /* calculate the P.E.I. minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.PAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use PAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.PAMTPCTF;
				 }
				 else if (MP.PAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using PAMTTX. */
						in->im.impminco = in->id.idmincar * MP.PAMTTX;
					  }
				 else if (MP.PAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using PAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.PAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
            
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */
			 if (in->im.imamtfg && MP.PAMTOPT == 3) {
                in->im.imprvtax = MP.PAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.PPTX, MP.PPTXrows, in->im.imitax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
						+ in->im.impothpe   /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
						+ in->im.impminco   /* provincial minimum tax carryover credit */
						+ in->im.impintsl   /* provincial interest on student loans credit */
						+ in->im.impyctxc   /* provincial young child tax credit */
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.PPNTCR * MP.PMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.PPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.PPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
					   + in->im.impothpe   /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco  /* provincial minimum tax carryover credit */
					   + in->im.impintsl   /* provincial interest on student loans credit */
 					   + in->im.impyctxc   /* provincial young child tax credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
				    + in->im.impcgtc    /* caregiver tax credit                  */
                    + in->im.imppentc   /* pension income tax credit             */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.PPNTCR * MP.PMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.PPNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.PBXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic));
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

    /* process each person in household */
    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
           ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

        /* original tax as percentage of federal tax */
        if (!MP.PTXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.PPTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if PTXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.PAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.PAMTPCTF;
			}
			else {
				if ( in->im.imamtfg && MP.PAMTOPT == 4 ) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.PAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}

        /* compute and add in surtax */
        in->im.impsur = MP.PSF * nneg(in->im.imtxp - MP.PSCI);

        /* apply surtax */
        in->im.imtxp += in->im.impsur;

        /* COMPUTE TAX CREDITS */
        /* calculate and apply provincial foreign tax credit */
        in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
        in->im.imtxp -= in->im.imnptc;
	}
 
    /* PEI low income tax reduction */
    /* calculate for the family. */

    /* process each nuclear family in household */
    for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

        /* initialize pointers */
        nfeld = nf->nfineld;
        nfspo = nf->nfinspo;
        NUMBER ptr = ZERO;
        NUMBER faminc = ZERO;

        /* if there is no spouse */
        if (!nf->nfspoflg) {
           ptr = MP.PTRBAS;
           /* since there is no spouse, treat one of the kids as equivalent-to-married */
           if (nf->nfnkids>0) {
              ptr += MP.PTRSP + nneg(nf->nfnkids-1)*MP.PTRKID;
           }
           faminc = nneg(nfeld->im.imicnet - nfeld->im.imuccb );
        }
        /* there is a spouse */
        else {
            ptr = MP.PTRSP + MP.PTRBAS + (nf->nfnkids*MP.PTRKID);
            faminc += nneg(nfspo->im.imicnet - nfspo->im.imuccb);
        }

        /* tax back */
        taxbak1(faminc, MP.PTRTD, &ptr, MP.PTRRR);
        /* give the tax reduction */
        if (!nf->nfspoflg) {
            nfeld->im.imptr = minn(ptr,nfeld->im.imtxp);
            nfeld->im.imtxp -= nfeld->im.imptr;
        }
        else {
            /* give to spouse with highest taxes outstanding first */
            if (nfeld->im.imtxp>nfspo->im.imtxp) {
                /* elder had higher taxes */
                nfeld->im.imptr = minn(ptr,nfeld->im.imtxp);
                nfeld->im.imtxp -= nfeld->im.imptr;
                /* if there is any unused reduction, transfer to the other spouse */
                if (nfeld->im.imptr < ptr) {
                    nfspo->im.imptr = minn((ptr - nfeld->im.imptr),nfspo->im.imtxp);
                    nfspo->im.imtxp -= nfspo->im.imptr;
                }
            }
            else {
                /* spouse had higher taxes */
                nfspo->im.imptr = minn(ptr,nfspo->im.imtxp);
                nfspo->im.imtxp -= nfspo->im.imptr;
                /* if there is any unused reduction, transfer to the other spouse */
                if (nfspo->im.imptr < ptr) {
                    nfeld->im.imptr = minn((ptr - nfspo->im.imptr),nfeld->im.imtxp);
                    nfeld->im.imtxp -= nfeld->im.imptr;
                }
            }
        }
    }

    /* process each person in household */
    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
	

        /* PEI Political Contribution Tax Credit */
        if ( isnzero(in->id.idprvpol) ) {
            poltc = minn(MP.PPTCBEN,
                (NUMBER) lkup1(MP.PPTC, MP.PPTCrows, in->id.idprvpol)
                    * in->id.idprvpol );
            in->im.impptc  = minn(poltc, in->im.imtxp);

            /* apply political contribution tax credit */
            in->im.imnptc += in->im.impptc;
            in->im.imtxp  -= in->im.impptc;

            DEBUG3("%s idpolcon =%.2f impptc =%.2f\n",
               in->id.idprvpol, in->im.impptc);
        }

        /* PEI labour-sponsored venture capital credit */
        if ( isnzero(in->id.idlabtxg) ) {
           lvctc = minn(MP.PLVCMAX, MP.PLVCRT * in->id.idlabtxg);
           in->im.implvctc = minn(lvctc, in->im.imtxp);
           in->im.imnptc += in->im.implvctc;
           in->im.imtxp  -= in->im.implvctc;
        }

        DEBUG4("%s PEI  bpt=%.2f, txp=%.2f, ptc=%.2f\n",
            in->im.imbpt, in->im.imtxp, in->im.imptc);
    }
 

    DEBUG_OFF("txpei");
}
