/***
*txqhstr.cpp - Compute family-related deductions or credits (Quebec)
*
*$Id: txqhstr.cpp 6.107 2007/09/18 17:22:12 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 07/07/31 | Changed calculation of eligible terms                   |
* | CH  | 07/07/17 | Added tranfer of credit to parents for older student    |
* | CH  | 07/07/17 | Added parameter to adjust student income for ps credit  |
* | LP  | 07/02/16 | Modifications to dep. child credit, dropped QDTCWCA     |
* | LP  | 07/01/16 | Deleted imuccb from net inc test, imuccb only in tax inc|
* | LP  | 06/06/01 | Removed imuccb from income tested credits               |
* | LP  | 04/08/31 | Minor edit imqndc assignment                            |
* | CH  | 04/08/10 | Edited imqndc                                           |
* | CH  | 04/08/06 | Changed dependant tax credit                            |
* | LP  | 04/02/13 | Updated income concept for imqdepni                     |
* | LP  | 03/12/22 | Updated idestat to reflect version 10 codes             |
* | LP  | 03/12/18 | Added QODISTC - infirm dependents 18 and over tax credit|
* | LP  | 01/03/12 | Removed pre-1988 programs (QREFOPT, etc.)               |
* | LP  | 00/11/16 | Apply QSPBFLAG, social prog benefits in total inc. flag |
* | CH  | 00/05/17 | Added imqcchc and imqndc                                |
* | CH  | 99/05/21 | Moved Medical credit to txqitax                         |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | EGA | 98/02/12 | add QMEDOPT                                             |
* | EGA | 98/02/11 | add QLAXRR, QLAXTD, QLAXPI                              |
* | SFG | 92/09/11 | initialise qndc in right place!                         |
* | SFG | 90/09/26 | Simplified calculation of who is used for married equiv.|
* | SFG | 90/08/27 | Corrected living alone tax credit (for old dependents)  |
* | SFG | 90/08/24 | Corrected education expenses                            |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txqhstr  [global]  Apply tax transfers between head and spouse (Quebec)
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== MANIFEST CONSTANTS ================================ */

#define HUGE_VAL 9999999999.0		/* huge value */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxqhstr[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txqhstr)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txqhstr - Apply tax transfers between head and spouse (Quebec)
*
*Synopsis:
*    void txqhstr(        
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   minn     Find the minimum of two numbers
*
*Structure Members:
*   MP.QCAFLAG      (i)  Quebec Child Assistance refundable tax credit flag
*   MP.QCALRFLG     (i)  Quebec combined age, living alone retirement credit
*   MP.QDTCNCA1     (i)  Quebec Dependent Tax Credit Amount for first dependent without child assistance
*   MP.QDTCNCA2     (i)  Quebec Dependent Tax Credit Amount for other dependents without child assistance
*   MP.QDTCSPA      (i)  Quebec Dependent Tax Credit Single Parent Amount (no child assistance)
*   MP.QITSFSEW     (i)  Quebec improved tax system for self-employed worker activation flag
*   MP.QLAXM        (i)  Quebec living alone exemption/amount
*   MP.QLAXPI       (i)  Quebec living alone phase-in
*   MP.QLAXRR       (i)  Quebec living alone reduction rate
*   MP.QLAXTD       (i)  Quebec living alone turn down
*   MP.QLPXM        (i)  Quebec lone parent exemption/amount
*   MP.QMEDINC      (i)  Income definition for Que. refundable tax credit for medical expenses
*   MP.QMXM         (i)  Quebec married exemption/amount
*   MP.QNTCR        (i)  Quebec nominal tax credit rate
*   MP.QODISTC      (i)  Quebec infirm dependent amount (18 and over)
*   MP.QPSXM        (i)  Quebec post-secondary exemption/amount
*   MP.QPSXMINC     (i)  Quebec income ratio for post-secondary exemption/amount
*   MP.QREPFAMI     (i)  Quebec repayment of UI/OAS benefits deduct from income flag
*   MP.QSPBFLAG     (i)  Quebec inclusion of social program benefits in income flag
*   MP.QTPCFLG      (i)  Quebec transfer of recognized parental contribution to education activation flag
*   MP.QYCXM1       (i)  Quebec dependent child #1 amount
*   MP.QYCXM2       (i)  Quebec dependent child #2 amount
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   hd.hdnpers      (i)  Number of persons in household
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.idestat      (i)  Educational status
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idschtp      (i)  School type
*   im.imigis       (i)  GIS benefits
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imispa       (i)  Spouse's allowance
*   im.imqcppse     (i)  Quebec self-employ. QPP/CPP Cont. deduction
*   im.imqinet      (i)  Quebec net income
*   im.imqitot      (i)  Quebec total income
*   im.imrepay      (i)  Social Benefits Repayments
*   im.imqdepni     (io) Quebec dependant's net income
*   im.imqcchc      (o)  Quebec child's non-refundable credit
*   im.imqdctc      (o)  Quebec dependent child tax credits
*   im.imqlatc      (o)  Quebec living alone tax credit
*   im.imqmtc       (o)  Quebec married tax credit
*   im.imqndc       (o)  Quebec number of dependent children
*
*Called by:
*   txque    Compute provincial taxes for Quebec
*
*****************************************************************************/
void txqhstr(
	P_hh hh
)
{
	register P_in in;
	register P_in ineld;
	register P_in inspo;
	P_cf cf;
	register int cfi;
	register int ini;
	NUMBER qinet = ZERO;
	NUMBER qitax = ZERO;
	NUMBER cdedt = ZERO;
	NUMBER cdedm = ZERO;
	NUMBER cmarex = ZERO;
	NUMBER exmt = ZERO;
	NUMBER exmtpi = ZERO;
	NUMBER dedt = ZERO;
	NUMBER livred = ZERO;
	NUMBER adjamt = ZERO;

	DEBUG_ON("txqhstr");

	/**
	* First off, we compute the net income concept to use for reducing
	* dependant credits.  This variable will also be used to reduce
	* the spousal tax credit in txcalc().
	**/

	/* process persons in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
		if (MP.QMEDINC == 1) {
			in->im.imqdepni = in->im.imqinet;
		}
		else { /* family income concept no longer based on net income */
			in->im.imqdepni = nneg(in->im.imqitot  
							  -	in->id.idrpp 
							  -	in->id.idrrsp 
							  -	(MP.QITSFSEW?in->im.imqcppse:ZERO ) /* improved tax system for self-employed workers */
							  - (MP.QREPFAMI?in->im.imrepay:ZERO) ) /* Repayment of OAS/UI */
							  ;
		}
		/* Add Social Program Benefits if not already included in net income calculations */
		if (!MP.QSPBFLAG) {
			in->im.imqdepni += in->im.imigis + in->im.imispa + in->im.imisa + in->id.idiworkc;
		}
		 
	}


	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

		/**
		* The algorithm applied here is as follows:
		*  (1) find the child with the least net income,
		*      for potential use as the equivalent to married tax credit.
		*  (3) Pass through the kids again, and assign them to child #1,
		*      child #2, child #3, etc. or equivalent to married
		*      in such a way as to optimize the credits received.
		*  (4) Give the credits to one or the other of the spouses.
		**/


		/* Do needed declarations/initialisations for each census family */

		P_in   p_min  = NULL;   /* kid with smallest net income      */
		P_in   ad_min  = NULL;   /* adult student with smallest net income      */
		int    chclm  = 0;      /* number of children claimed        */
		NUMBER ctxcs  = ZERO;   /* total value of child tax credits  */
		NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
		NUMBER cchc   = ZERO;   /* Child's non-refundable credit             */
		NUMBER inet   = ZERO;   /* working variable for net income */
		NUMBER cftps  = ZERO;   /* full-time post-secondary credit */
		NUMBER famnet = ZERO;   /* family net income               */
		NUMBER medgro = ZERO;   /* family medical expenses         */
		NUMBER infirmdp = ZERO;  /* infirm dependents 18 and over amount */
        NUMBER elgterm = ZERO;   /* number of eligible school terms */

		DEBUG1("%s process census family children Pass #1\n");

		/* process children in census family to set pointer for kid with lowest income */
		for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {

			DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
		         	ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

			/**
			* Filter condition (ineligible for dependant credit)
			*  consists of children over 18 who did not attend
			*  a post-secondary institution full-time.
			**/

		  	if (in->id.idage > 18 && (in->id.idestat ==0 || in->id.idestat ==2)) {
				continue;
			}

			/* When the child assistance program is in effect, only dependents in
			   at least one term of full time 
			   post-secondary are eligible for the non-refundable dependent credits */

		  	if ((in->id.idschtp < 2 || in->id.ideducfm < 4) && MP.QCAFLAG) {
				continue;
			}

			inet = in->im.imqdepni;

			/* Update pointer for lowest net income child */

			if (p_min == NULL || p_min->im.imqdepni > inet) {
				p_min = in;
			}

			/* Update pointer for lowest net income dependent in families with kids over the age of 18 */
            if ( MP.QCAFLAG && in->id.idage>=18 ) {
                /*  full-time students (at least 3 months) */
   				if ( (in->id.idestat == 1 || in->id.idestat == 3) && in->id.ideducfm > 3 ) {
                    if (ad_min == NULL || ad_min->im.imqdepni > inet) {
                        ad_min = in;
                    }
                }
            }
		}


		DEBUG1("%s process census family children Pass #2\n");

		/* Initialise tax credit variables */
		cftps = ZERO;   /* full-time post-secondary amount */
		ctxcs = ZERO;   /* total value of child tax credits */


		/* process children in census family */
		for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {

			/* Filter condition (ineligible for dependent credit)  */

			if (in->id.idage > 18 && (in->id.idestat ==0 || in->id.idestat == 2)) {
				continue;
			}

			/* When the child assistance program is in effect, only dependents in
			   at least one term of full time 
			   post-secondary are eligible for the non-refundable dependent credits */

		  	if ((in->id.idschtp < 2 || in->id.ideducfm < 4) && MP.QCAFLAG) {
				continue;
			}

			/* create working variables */
			inet = nneg(in->im.imqdepni);
			cchc = ZERO;

			/* compute amount for full-time post-secondary studies */

            if (in->id.ideducfm<4) elgterm=0;
            else if (in->id.ideducfm<8) elgterm=1;
            else elgterm=2;
			
			cftps = MP.QPSXM * elgterm;

			if (MP.QCAFLAG) {
				if (cf->cfspoflg) {
				   if (in->id.idage < 18) {
				       cchc = nneg(cftps - (MP.QPSXMINC * inet));
				   }
				   else if (!MP.QTPCFLG) { 
					   if (in == ad_min && cf->cfnkids == 0) {
					       /* may add an additional amount for this child if
					          the family has not claimed any child assistance */
					       cchc = nneg(MP.QDTCNCA1 + cftps - (MP.QPSXMINC * inet));
					   }
					   else {  /* the family has children under 18 */
					   	   cchc = nneg(MP.QDTCNCA2 + cftps - (MP.QPSXMINC * inet));
					   }
				   }
				   in->im.imqcchc = MP.QNTCR * cchc;
				}
				else {  /* no spouse present */
				   if (in->id.idage < 18) {
				       cchc = nneg(cftps - (MP.QPSXMINC * inet));
				   }
                   /* if dependent amount for kids >18 hasn't been replaced with a tranfer of credits */
				   else if (!MP.QTPCFLG) { 
					   if (in == ad_min ) { /* assign the single parent amount to lowest income */
					       if (cf->cfnkids == 0) {
					       /* may add an additional amount for this child if
					          the family has not claimed any child assistance */
					       		cchc = nneg(MP.QDTCNCA1 + MP.QDTCSPA + cftps - (MP.QPSXMINC * inet));
					   	   }
					   	   else {  /* the family has children under 18 */
					   	   		cchc = nneg(MP.QDTCNCA2 + cftps - (MP.QPSXMINC * inet));
						   }
					   }
					   else { /* not the lowest income, single parent amount already applied */
						   cchc = nneg(MP.QDTCNCA2 + cftps - (MP.QPSXMINC * inet));
					   }
					}
					in->im.imqcchc = MP.QNTCR * cchc;
				}
				ctxcs += cchc;
			}
			else { /* amounts for dependents prior to Child Assistance program	 */
				if (cf->cfspoflg) {
					if (in == p_min) {
						cchc = nneg(MP.QYCXM1 + cftps - inet);
					}
					else {
						cchc = nneg(MP.QYCXM2 +	cftps - inet);
					}
					in->im.imqcchc = MP.QNTCR * cchc;
				}
				else { /* no spouse present */
					/* single parent claims kid with lowest income as kid #1 */
					if (in == p_min) {
						cchc = nneg(MP.QYCXM1 + MP.QLPXM + cftps - inet);
					}
					else { /* all other kids */
						cchc = nneg(MP.QYCXM2 + cftps - inet);
					}
					in->im.imqcchc = MP.QNTCR * cchc;
				}
				ctxcs += cchc;

			}
									
			if (isnzero(cchc)) {
                /* when the child assistance program is in effect, only count kids 18 or older 
                   to prevent double counting (all kids <18 will be included later */
                if (!MP.QCAFLAG) chclm++;
                else if (in->id.idage>=18) chclm++;
			}
			else {
				if (!MP.QCAFLAG && chclm < cf->cfnkids) {
					chclm = cf->cfnkids;
				}
			}

		}

		DEBUG3("%s chclm=%d ctxcs=%.2f\n", chclm, ctxcs);

		DEBUG1("%s Apply credits to head (and spouse if present)\n");

		/* initialise pointers for head and spouse modelled variables */
		ineld = cf->cfineld;
		inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

		if (cf->cfspoflg) {

			DEBUG1("%s spouse present\n");

			/*  determine if either spouse has claimed an amount for an infirm dependent 
			*   aged 18 or over (idothpe). If so, calculate the amount and then assign
			*   to the appropriate parent.
			**/

			if (ineld->id.idothpe > 0 || inspo->id.idothpe > 0) {
				/*  if an amount is present then give the max */
				infirmdp = (MP.QYCXM2 + MP.QODISTC) * MP.QNTCR;
			}

			/**
			* If the net income of the head is greater than the net
			* income of the spouse, the head gets the married
			* credit and the credits of the wholly dependent
			* children.
			**/

			if (ineld->im.imqinet >= inspo->im.imqinet) {
				DEBUG1("%s head receives married tax credit\n");
				ineld->im.imqmtc = MP.QNTCR * nneg(MP.QMXM - nneg(inspo->im.imqdepni));
				ineld->im.imqdctc = (MP.QNTCR * ctxcs) + infirmdp;
                /* when the child assistance program is in place, must add kids under 18 back */
                if (MP.QCAFLAG) ineld->im.imqndc = chclm + cf->cfnkids;
				else ineld->im.imqndc = chclm;
				DEBUG3("%s head's qmtc=%.2f, qdctc=%.2f\n",
				      ineld->im.imqmtc, ineld->im.imqdctc);

			}
			else {
				DEBUG1("%s spouse receives married tax credit\n");
				inspo->im.imqmtc = MP.QNTCR * nneg(MP.QMXM - nneg(ineld->im.imqdepni));
				inspo->im.imqdctc = (MP.QNTCR * ctxcs) + infirmdp;
                /* when the child assistance program is in place, must add kids under 18 back */
                if (MP.QCAFLAG) inspo->im.imqndc = chclm + cf->cfnkids;
				else inspo->im.imqndc = chclm;
				DEBUG3("%s spouse's qmtc=%.2f, qdtc=%.2f\n",
				      inspo->im.imqmtc, inspo->im.imqdctc);
			}
		}

		else { /* no spouse present */

			/**
			* Claim amount for person living alone, if the only other
			* persons in this household qualify for a dependent child
			* amount.
			**/

			/* Note that when the Combined age, living alone, retirement income
			   credit is in place, this credit is not applicable */
			adjamt = ZERO;
			if ((hh->hd.hdnpers == (1+chclm)) && !MP.QCALRFLG) {
			    livred = minn(MP.QLAXM, MP.QLAXRR * nneg(ineld->im.imqinet - MP.QLAXTD ));
			    adjamt = MP.QLAXM - MP.QLAXPI*livred;
				ineld->im.imqlatc = MP.QNTCR * adjamt;
			}

			/* Calculate an amount for other infirm dependent if claimed */
			if (ineld->id.idothpe > 0 ) {
				/*  if an amount is present then give the max */
				infirmdp = MP.QODISTC * MP.QNTCR;
			}
			
			ineld->im.imqdctc = (MP.QNTCR * ctxcs) + infirmdp;
            /* when the child assistance program is in place, must add kids under 18 back */
            if (MP.QCAFLAG) ineld->im.imqndc = chclm + cf->cfnkids;
			else ineld->im.imqndc = chclm;

			DEBUG4("%s head's qmtc=%.2f, qdctc=%.2f, qlatc=%.2f\n",
			      ineld->im.imqmtc, ineld->im.imqdctc, ineld->im.imqlatc);
		   
		}
	}
	DEBUG_OFF("txqhstr");
}




