/***
*vsdu.cpp - Make user-supplied variables accessible to SPSM facilities
*
*$Id: vsdu.cpp 6.87 2007/09/18 17:23:49 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the vsdu.c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 95/10/18 | Add pragma to suppress near/far warnings in MSDOS 16-bit|
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Usage:
*   This function is provided to allow users to define their own variables
*   so that the various variable-oriented facilities of SPSM (such as the
*   X-tab facility) will be able to recognize these variables by name.
*   Two files are used when adding a new variable.
*
*   vsu.h    - is a header file which declares the new variables.  It
*                should be modified first when adding a new variable.
*   vsdu.c   - is a C-language program which enables SPSM to understand and
*                reference the new variables.  Each new variable in vsu.h
*                should have a corresponding entry in vsdu.c
*
*   This file contains example definitions of variables.  These examples can
*   be copied and edited to add the new variable.
*   Further examples may be seen in the file /spsm/model/vsdm.c
*                                                                           
*Definitions:
*   vsdu     [global]  Make user-supplied variables accessible to SPSM facilities
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "vm.h"     /* Variable manipulation package declarations           */


/* ==================== COMPILER PRAGMAS ================================== */

// eliminate spurious 'near/far mismatch' from MS-DOS compiler
#if defined(_MSDOS)
#pragma warning(disable : 4762)
#endif

#ifdef MSCOVL
#pragma alloc_text(DIALOG_TEXT,vsdu)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */

/*****************************************************************************
*vsdu - Make user-supplied variables accessible to SPSM facilities
*
*Synopsis:
*    void vsdu()          
*
*Description:
*
*Return Value:
*   None.
*
*Called by:
*   cpc      Calculate derived control parameters and do edits
*   phase23  Engage in user dialogue, and prepare for simulation
*
*****************************************************************************/
void  vsdu()
{

	DEBUG_ON("vsdu");

	/**
	* Variable definitions for new variables follow.  As mentioned above, new
	* variables should first be added to the file 'vsu.h'.  As shown below, a
	* variable is defined to the SPSM variable facilities through a call to the
	* 'vardef' macro.  'vardef' takes five arguments whose values are
	* explained in the example below.  If defining a new integer variable,
	* SPSM must be informed about the number of levels the variable takes, and
	* must also be supplied with a label for each of the allowed levels.  This
	* information is supplied through a call to the 'stradd' function.  An
	* optional label for the variable itself can also be supplied through an
	* appropriate 'stradd' call.  Please see the SPSD/M Programmer's Guide for
	* more information.
	*
	* Please see the files '/spsm/defs/vs.h' and '/spsm/model/vsdm.c' for
	* examples of variable declarations and definitions.  The easiest way to
	* add a variable is to copy and modify the definition of an existing
	* variable.
	*
	*
	* A brief description of the meaning of the arguments to 'vardef' and
	* 'stradd' is illustrated by the following example for classification
	* variables:
	*
	*    vardef("_uvew",       <= the name of the variable, quoted, with '_'
	*           IN,            <= home structure (leave at 'IN')
	*           im.uv.ew,      <= variable location (always in im.uv)
	*           C_INT,         <= C-type (C_INT or C_NUM)
	*           V_CLAS         <= type of variable (V_CLAS or V_ANAL)
	*                    );
	*
	*    stradd("uvew",        <= the name of the variable, quoted
	*           "Region"       <= a printing label for the variable
	*                    );
	*
	*    stradd("Luvew",       <= the name of the variable, preceded by 'L'
	*           "\tEast\tWest" <= string containing a label for each valid
	*                    );       level, preceded by a tab '\t' character.
	*                                                                         
	*                                                                         
	* It is essential that the C-type be identical to the type indicated in   
	* the vsu.h file.  Failure to ensure this can result in obscure
	* run-time errors (probably floating point exceptions).
	*
	* The following definitions can be used as templates if desired.
	*
	*  ------------------
	*  A numeric variable:
	*  ------------------

	vardef("_xxxxxxxx",  IN, im.uv.xxxxxxxx,    C_NUM, V_ANAL);
	stradd("xxxxxxxx",   "Variable label");

	*  -----------------------------------------------------
	*  An integer analysis variable, with values 0 through 4:
	*  -----------------------------------------------------

	vardef("_yyxxxxxx",  IN, im.uv.yyxxxxxx,    C_INT, V_ANAL);
	stradd("yyxxxxxx",   "Variable label");
	stradd("Lyyxxxxxx",  "\t0\t1\t2\t3\t4");

	*  --------------------------------------------------
	*  An integer class variable, with values 0 through 4:
	*  --------------------------------------------------

	vardef("_yyxxxxxx",  IN, im.uv.yyxxxxxx,    C_INT, V_CLAS);
	stradd("yyxxxxxx",   "Variable label");
	stradd("Lyyxxxxxx",  "\tLABEL0\tLABEL1\tLABEL2\tLABEL3\tLABEL4");

	* End of template examples.  Calls to 'vardef' should immediately follow
	* this comment block.
	**/


	DEBUG_OFF("vsdu");
}


